#include "absrefdialog.h"
#include "ui_absrefdialog.h"
#include "floatdelegate.h"
#include "mainwindow.h"

#include <QDebug>

extern "C" void get_nabsref(int rowspace, int *nabsref_forC);
extern "C" void get_absref(absref_type *absref_forC);

AbsRefDialog::AbsRefDialog(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::AbsRefDialog)
{
    ui->setupUi(this);

    absRefModel = new QStandardItemModel(this);
    absRefProxyModel = new QSortFilterProxyModel(this);
    absRefProxyModel->setSourceModel(absRefModel);

    absRefModel->setColumnCount(5);
    QStringList titles = { "Num", "  h  k  l", "Extinction condition",
                           "Probability for extinction", "Type"};
    absRefModel->setHorizontalHeaderLabels(titles);

    ui->absRefTableView->setModel(absRefProxyModel);
    ui->absRefTableView->verticalHeader()->setDefaultSectionSize(22);
    ui->absRefTableView->setSortingEnabled(true);
    ui->absRefTableView->sortByColumn(0,Qt::AscendingOrder);
    FloatDelegate *probDelegate = new FloatDelegate(ui->absRefTableView);
    ui->absRefTableView->setItemDelegateForColumn(3, probDelegate);

    connect(ui->absRefTableView->selectionModel(), &QItemSelectionModel::selectionChanged, this,
            [this]() {
        QModelIndexList selection = ui->absRefTableView->selectionModel()->selectedRows();        
        QVector<int> selRef(selection.count());
        for(int i=0; i < selection.count(); i++)
        {
            QModelIndex index = absRefProxyModel->index(selection.at(i).row(),0);            
            selRef[i] = index.data().toInt() - 1;
        }        
        MainWindow *mw = qobject_cast<MainWindow *>(this->parent()->parent());
        mw->setSelectedRef(selRef);
        mw->drawSelectedRef();
    });
}

AbsRefDialog::~AbsRefDialog()
{
    delete ui;
}

void AbsRefDialog::updateAbsRefList(int selectedSpace)
{
    int nAbsRef;
    get_nabsref(selectedSpace+1, &nAbsRef);
    if (nAbsRef == 0) {
        absRefModel->setRowCount(0);
        return;
    }
    absRefModel->setRowCount(nAbsRef);

    absref_type *absref = new absref_type[nAbsRef];
    get_absref(absref);    

    for (int r = 0; r < nAbsRef; r++) {
        QStandardItem *itemNum = new QStandardItem;
        itemNum->setData(absref[r].num, Qt::DisplayRole);
        itemNum->setTextAlignment(Qt::AlignRight | Qt::AlignVCenter);
        absRefModel->setItem(r,0,itemNum);

        QString hklString = QString("%1%2%3").arg(absref[r].hkl[0],3).
                arg(absref[r].hkl[1],3).arg(absref[r].hkl[2],3);
        QStandardItem *itemHKL = new QStandardItem(hklString);
        itemHKL->setTextAlignment(Qt::AlignCenter | Qt::AlignVCenter);
        absRefModel->setItem(r,1,itemHKL);

        QStandardItem *itemExt = new QStandardItem(absref[r].string);
        itemExt->setTextAlignment(Qt::AlignCenter | Qt::AlignVCenter);
        absRefModel->setItem(r,2,itemExt);

        QStandardItem *itemProb = new QStandardItem;
        itemProb->setData(absref[r].prob, Qt::DisplayRole);
        absRefModel->setItem(r,3,itemProb);

        QStandardItem *itemType = new QStandardItem(absref[r].ktype == 1 ? "Single" : "Overlapped" );
        absRefModel->setItem(r,4,itemType);
    }

    delete [] absref;

    ui->absRefTableView->resizeColumnsToContents();
    ui->absRefTableView->horizontalHeader()->setSectionResizeMode(4,QHeaderView::Stretch);
    ui->absRefTableView->selectRow(0);
}

void AbsRefDialog::on_AbsRefDialog_finished(int)
{
    MainWindow *mw = qobject_cast<MainWindow *>(this->parent()->parent());
    mw->clearSelectedRef();
}
