#include "bondsettingsdialog.h"
#include "ui_bondsettingsdialog.h"
#if defined(JAV_WIDGET)
#include "crystalviewwidget.h"
#else
#include "javwindow.h"
#endif
#include "libcomune.h"
#include <QtMath>
#include <QStringList>
#include <QStandardItem>

extern "C" void jav_edit_bonds_(int *code, _BondTable bt[], _BondSetting *bs, int*n, int *wm);

BondSettingsDialog::BondSettingsDialog(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::BondSettingsDialog),
    CurBondTable(nullptr)
{
#if defined(JAV_WIDGET)
    jav = qobject_cast<CrystalViewWidget*>(parent);
#else
    jav = qobject_cast<JavWindow*>(parent);
#endif
    ui->setupUi(this);

    bondModel = new QStandardItemModel(this);
    m_delegate = new MinMaxDelegate(this);
    m_delegate->setMinMaxcol(3,4);
    bondModel->setColumnCount(6);
    QStringList labels = {"No.","Atom 1","Atom 2","MInDist","MaxDist","Type"};
    bondModel->setHorizontalHeaderLabels(labels);    

    ui->bondTable->setModel(bondModel);
    ui->bondTable->setItemDelegateForColumn(3,m_delegate);
    ui->bondTable->setItemDelegateForColumn(4,m_delegate);
    ui->bondTable->verticalHeader()->setDefaultSectionSize(22);
    ui->bondTable->verticalHeader()->hide();
    ui->bondTable->resizeColumnsToContents();
}

BondSettingsDialog::~BondSettingsDialog()
{
    delete ui;
}

void BondSettingsDialog::set_values_for_Bsetting(_BondSetting BSetting)
{
    ui->comboMinDist->setCurrentIndex(BSetting.tolmin > 0 ? 0 : 1);
    ui->MinDistSpin->setValue(qFabs(BSetting.tolmin));
    ui->comboMaxDist->setCurrentIndex(BSetting.tolmax >= 0 ? 0 : 1);
    ui->MaxDistSpin->setValue(qFabs(BSetting.tolmax));
    ui->bondAngleSpin->setValue(BSetting.angle);
}

int BondSettingsDialog::on_bondsetting()
{
    int wm = Properties::GetActiveModel();
    if (wm <= 0) return 1;
    int maxnumT = jav->GetModel(wm)->numT;
    if (!maxnumT) return 2;
    _BondTable *btable = new _BondTable[maxnumT];
    _BondSetting BSetting;
    int icode = 0;
    int numT = 0;
    jav_edit_bonds_(&icode, btable, &BSetting, &numT,&wm);    
    CurBondTable = new _BondTable[numT];
    CurBondTableSav = new _BondTable[numT];
    std::copy(btable,btable+numT,CurBondTable);
    std::copy(btable,btable+numT,CurBondTableSav);
    BSettingSav = BSetting;
    delete [] btable;

    set_values_for_Bsetting(BSetting);

    ui->bondTable->sortByColumn(0,Qt::AscendingOrder);

    bondModel->setRowCount(numT);
/*
    for (int row=0; row < numT; row++){
        QModelIndex index = bondModel->index(row,0,QModelIndex());
        bondModel->setData(index,row,Qt::DisplayRole);
    }
*/
    //test disconnect(bondModel,SIGNAL(itemChanged(QStandardItem*)),this,SLOT(onItemChanged(QStandardItem*)));
    for (int i=0; i < numT; i++){
        QStandardItem *itemNo = new QStandardItem;
        itemNo->setData(QVariant(i+1),Qt::DisplayRole);
        itemNo->setFlags(itemNo->flags() ^ Qt::ItemIsEditable);
        bondModel->setItem(i,0,itemNo);

        QString elem1 = get_nth_element(CurBondTable[i].z1-1);
        QStandardItem *itemElem1 = new QStandardItem(elem1);
        itemElem1->setFlags(itemElem1->flags() ^ Qt::ItemIsEditable);
        bondModel->setItem(i,1,itemElem1);

        QString elem2 = get_nth_element(CurBondTable[i].z2-1);
        QStandardItem *itemElem2 = new QStandardItem(elem2);
        itemElem2->setFlags(itemElem2->flags() ^ Qt::ItemIsEditable);
        bondModel->setItem(i,2,itemElem2);

        QStandardItem *itemDmin= new QStandardItem;
        itemDmin->setData(QVariant(static_cast<double>(CurBondTable[i].distmin)),Qt::EditRole);
        bondModel->setItem(i,3,itemDmin);

        QStandardItem *itemDmax = new QStandardItem;
        itemDmax->setData(QVariant(static_cast<double>(CurBondTable[i].distmax)),Qt::EditRole);
        bondModel->setItem(i,4,itemDmax);

        QStandardItem *itemType = new QStandardItem;
        itemType->setData(QVariant(CurBondTable[i].typeb),Qt::DisplayRole);
        itemType->setFlags(itemType->flags() ^ Qt::ItemIsEditable);
        bondModel->setItem(i,5,itemType);
    }

    ui->bondTable->resizeColumnsToContents();
    //test connect(bondModel,SIGNAL(itemChanged(QStandardItem*)),this,SLOT(onItemChanged(QStandardItem*)));
    return 0;
}

void BondSettingsDialog::on_buttonBox_clicked(QAbstractButton *button)
{
    QDialogButtonBox::StandardButton stdButton = ui->buttonBox->standardButton(button);

    if(stdButton == QDialogButtonBox::Ok || stdButton == QDialogButtonBox::Apply)
    {
        _BondSetting BSetting;
        get_values_for_BondSetting(CurBondTable,BSetting);
        int icode = stdButton == QDialogButtonBox::Ok ? 1 : 3;
        int wm = Properties::GetActiveModel();
        int numT = bondModel->rowCount();
        jav_edit_bonds_(&icode, CurBondTable, &BSetting, &numT,&wm);
        if (stdButton == QDialogButtonBox::Ok) {
            delete [] CurBondTable;
            delete [] CurBondTableSav;
        }
    } else if (stdButton == QDialogButtonBox::RestoreDefaults) {        
        _BondSetting BSetting;
        int icode = 4;
        int wm = Properties::GetActiveModel();
        int numT;
        jav_edit_bonds_(&icode, CurBondTable, &BSetting, &numT,&wm);
        set_values_for_Bsetting(BSetting);
        ui->bondTable->clearSelection();
        ui->bondTable->sortByColumn(0,Qt::AscendingOrder);                
        for(int row=0; row < bondModel->rowCount(); row++){
            QStandardItem *itemDmin= new QStandardItem;
            itemDmin->setData(QVariant(CurBondTable[row].distmin),Qt::EditRole);
            bondModel->setItem(row,3,itemDmin);

            QStandardItem *itemDmax = new QStandardItem;
            itemDmax->setData(QVariant(CurBondTable[row].distmax),Qt::EditRole);
            bondModel->setItem(row,4,itemDmax);
        }
        ui->bondTable->resizeColumnsToContents();
        ui->bondTable->horizontalHeader()->stretchLastSection(); //It is ignored if combined with resizeColumnsToContents. Why?
    }
}

void BondSettingsDialog::reject()  //invoked at Cancel and closing window by 'x'
{
    int icode = 2;
    int wm = Properties::GetActiveModel();
    int numT = bondModel->rowCount();
    jav_edit_bonds_(&icode, CurBondTableSav, &BSettingSav, &numT,&wm);
    delete [] CurBondTable;
    delete [] CurBondTableSav;
    QDialog::reject();
}

void BondSettingsDialog::get_values_for_BondSetting(_BondTable btable[], _BondSetting& BSetting)
{
    BSetting.tolmin = ui->MinDistSpin->value();
    if (ui->comboMinDist->currentIndex() == 1) BSetting.tolmin = -BSetting.tolmin;
    BSetting.tolmax = ui->MaxDistSpin->value();
    if (ui->comboMaxDist->currentIndex() == 1) BSetting.tolmax = -BSetting.tolmax;
    BSetting.angle = ui->bondAngleSpin->value();

    for(int row=0; row < bondModel->rowCount(); row++){
        QModelIndex index = bondModel->index(row,0,QModelIndex());
        int row0 = bondModel->data(index).toInt() - 1;
        index = bondModel->index(row,3,QModelIndex());
        btable[row0].distmin = bondModel->data(index).toFloat();
        index = bondModel->index(row,4,QModelIndex());
        btable[row0].distmax = bondModel->data(index).toFloat();        
    }
}
