#include "celllistdialog.h"
#include "ui_celllistdialog.h"
#include "floatdelegate.h"
#include "mainwindow.h"
#include "qt_utils.h"

#include <QPushButton>

extern "C" void viewrefcell(int icell);

static const QString progs[] = {"N", "D", "M"};

CellListDialog::CellListDialog(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::CellListDialog)
{
    ui->setupUi(this);

    QPushButton *exportButton = new QPushButton("&Export");
    exportButton->setIcon(QIcon(":/pixmaps/pixmaps/exportb.png"));
    ui->buttonBox->addButton(exportButton, QDialogButtonBox::HelpRole);    

    cellListModel = new QStandardItemModel(this);
    cellListModel->setColumnCount(15);
    QStringList titles = { "Nr.", "Prog.", "a", "b", "c", "alpha", "beta", "gamma",
                         "Vol.","M20","FOMnew","McM20","Shift","NIX","Symmetry Info"};
    cellListModel->setHorizontalHeaderLabels(titles);

    ui->cellTableView->setModel(cellListModel);
    ui->cellTableView->verticalHeader()->setDefaultSectionSize(22);
    ui->cellTableView->setSortingEnabled(true);
    FloatDelegate *abcDelegate = new FloatDelegate(ui->cellTableView,5);
    ui->cellTableView->setItemDelegateForColumn(2, abcDelegate);
    ui->cellTableView->setItemDelegateForColumn(3, abcDelegate);
    ui->cellTableView->setItemDelegateForColumn(4, abcDelegate);
    FloatDelegate *float3decDelegate = new FloatDelegate(ui->cellTableView,3);
    ui->cellTableView->setItemDelegateForColumn(5, float3decDelegate);
    ui->cellTableView->setItemDelegateForColumn(6, float3decDelegate);
    ui->cellTableView->setItemDelegateForColumn(7, float3decDelegate);
    ui->cellTableView->setItemDelegateForColumn(12, float3decDelegate);
    FloatDelegate *float2decDelegate = new FloatDelegate(ui->cellTableView,2);
    ui->cellTableView->setItemDelegateForColumn(8, float2decDelegate);
    ui->cellTableView->setItemDelegateForColumn(9, float2decDelegate);
    ui->cellTableView->setItemDelegateForColumn(10, float2decDelegate);
    ui->cellTableView->setItemDelegateForColumn(11, float2decDelegate);

    connect(ui->cellTableView->selectionModel(),&QItemSelectionModel::currentRowChanged,this,[this](const QModelIndex &current){
        QModelIndex index = cellListModel->index(current.row(),0);        
        viewrefcell(index.data().toInt());
    });

    connect(exportButton,&QPushButton::clicked, this, [this](){
        MainWindow *w = qobject_cast<MainWindow *>(this->parent());
        w->exportCellList();
    });
}

CellListDialog::~CellListDialog()
{
    delete ui;
}

void CellListDialog::createModel(cell_info_type cell[], int ncell, int selectedCell)
{
    cellListModel->setRowCount(ncell);
    for (int r = 0; r < ncell; r++) {
        QStandardItem *itemNr = new QStandardItem;
        itemNr->setData(r+1,Qt::DisplayRole);
        itemNr->setTextAlignment(Qt::AlignRight | Qt::AlignVCenter);
        cellListModel->setItem(r,0,itemNr);

        QStandardItem *itemProg = new QStandardItem(progs[cell[r].pcode - 1]);
        cellListModel->setItem(r,1,itemProg);

        for (int c = 2; c < 8; c++) {
            QStandardItem *itemPar = new QStandardItem;
            itemPar->setData(cell[r].par[c-2],Qt::DisplayRole);            
            cellListModel->setItem(r,c,itemPar);
        }

        QStandardItem *itemVol = new QStandardItem;
        itemVol->setData(cell[r].vol,Qt::DisplayRole);
        cellListModel->setItem(r,8,itemVol);

        QStandardItem *itemM20 = new QStandardItem;
        itemM20->setData(cell[r].m20,Qt::DisplayRole);
        cellListModel->setItem(r,9,itemM20);

        QStandardItem *itemFom = new QStandardItem;
        if (cell[r].alt_fom > 0) {
            itemFom->setData(cell[r].alt_fom,Qt::DisplayRole);
        } else {
            itemFom->setData("-",Qt::DisplayRole);
        }
        cellListModel->setItem(r,10,itemFom);

        QStandardItem *itemMc20 = new QStandardItem;
        if (cell[r].mc20 > 0) {
            itemMc20->setData(cell[r].mc20,Qt::DisplayRole);
        } else {
            itemMc20->setData("-",Qt::DisplayRole);
        }
        cellListModel->setItem(r,11,itemMc20);

        QStandardItem *itemShift = new QStandardItem;
        itemShift->setData(cell[r].zshift,Qt::DisplayRole);
        cellListModel->setItem(r,12,itemShift);

        QStandardItem *itemNind = new QStandardItem;
        itemNind->setData(cell[r].nund,Qt::DisplayRole);
        itemNind->setTextAlignment(Qt::AlignRight | Qt::AlignVCenter);
        cellListModel->setItem(r,13,itemNind);

        QStandardItem *itemSymm = new QStandardItem(cell[r].syminfo);
        cellListModel->setItem(r,14,itemSymm);
    }
    ui->cellTableView->resizeColumnsToContents();

    int selectedRow = selectedCell - 1;
    if (selectedRow == ui->cellTableView->currentIndex().row()) {
        QModelIndex index = cellListModel->index(selectedRow,0);
        viewrefcell(index.data().toInt());
    }
    else if (selectedRow >= 0 && selectedRow < ncell)
        ui->cellTableView->selectRow(selectedRow);
}

void CellListDialog::waitSignalIsEmitted()
{
    bool isWaitCursor = stopWaitCursor();
    QEventLoop loop;
    connect(this,&QDialog::finished,&loop,&QEventLoop::quit);
    loop.exec();
    if (isWaitCursor) QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));
}

int CellListDialog::selectedCell() const
{    
   QModelIndex index = cellListModel->index(ui->cellTableView->currentIndex().row(),0);
   return cellListModel->data(index).toInt();
}
