#include "centroidsdlg.h"
#include "ui_centroidsdlg.h"
#include "properties.h"
#include "colorbutton.h"
#include "itemdelegate.h"
#if defined(JAV_WIDGET)
#include "crystalviewwidget.h"
#else
#include "javwindow.h"
#endif
#include <QPainter>

extern "C" void jav_find_centroids_(int *kmodel);

enum {COL_NUM, COL_VIS, COL_LABEL, COL_SHOWLAB, COL_COLOR, COL_TRANSP};

float jav_centroid_type::radN = 1;
#if defined(JAV_WIDGET)
CrystalViewWidget *jav_centroid_type::m_jav = nullptr;
#else
JavWindow *jav_centroid_type::m_jav = nullptr;
#endif

void jav_centroid_type::DrawSelected(float add)
{
     glPushMatrix();
     glTranslatef(xyzN[0], xyzN[1], xyzN[2]);
     disegnaSfera(radN+add, 100, false);
     glPopMatrix();
}

void jav_centroid_type::Draw(QOpenGLFunctions *f)
{
     float color1[4];
     FromQColor(colore, color1);
     if(transparency)
     {
        f->glDepthMask (GL_FALSE);
        color1[3] = (float) (100 - transparency)/100.f;
     }
     else
        f->glDepthMask (GL_TRUE);

     glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, color1);
     glPolygonMode( GL_FRONT_AND_BACK, GL_FILL);
     f->glEnable (GL_BLEND);
     glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
     glPushMatrix();
     glTranslatef(xyzN[0], xyzN[1], xyzN[2]);
     disegnaSfera(radN, 100, false);
     glPopMatrix();
     f->glDisable (GL_BLEND);
     f->glDepthMask (GL_TRUE);
}

void jav_centroid_type::DrawLabel(QColor c)
{
    GLdouble winx, winy, winz;
    GLdouble x, y, z;
    double d = radN * (Properties::sc_atoms*Properties::Scale*
                        Properties::sc_factor)/SCALA;
    double delta[3] = {
         d * Properties::xoffset * Properties::npixels / 2,
         d * Properties::yoffset * Properties::npixels / 2 - 10,
         d};

    FromCoordToWin(xyzN[0],xyzN[1], xyzN[2],
          Properties::modelG, Properties::projG, Properties::viewG, &winx, &winy, &winz);
    FromWinToCoord(winx+delta[0], winy+delta[1],
                    winz-delta[2]/10, Properties::modelG, Properties::projG,
            Properties::viewG, &x, &y, &z);
    vet3 pos = {{x, y, z}};
    if(m_jav)
       m_jav->JavDrawText(pos, label, c);
}

void jav_centroid_type::DrawCentroid(QOpenGLFunctions *f, QColor c)
{
        if(!visible)
           return;
        Draw(f);
        if(selected)
        {
            SetLightForSelected(f, 1);
            float add = (float)(0.3 / Properties::deltav);
            DrawSelected(add);
            ResetLightForSelected(f);
        }
        if(showlab)
           DrawLabel(c);
}

void jav_centroid_type::Normalizza()
{
    vet3 xyzo = Properties::FractToCart(xyz);

    vet3 a = Properties::Trasforma(xyzo);
    for(int j=0; j<3; j++)
        xyzN[j] = (float) a.v[j];
}

void CentroidsDlg::DrawCentroids(int m, QOpenGLFunctions *f,  QColor c)
{
    int nCentroids = (int) AllCentr[m].size();
    for(int i=0; i<nCentroids; i++)
        AllCentr[m][i].DrawCentroid(f, c);
}

void CentroidsDlg::rowSelectionChanged(const QItemSelection &, const QItemSelection &)
{
    int m = Properties::GetActiveModel() - 1;
    for(int i=0; i<AllCentr[m].size(); i++)
        AllCentr[m][i].selected = false;
    QModelIndexList selection = ui->tableCentroids->selectionModel()->selectedIndexes();
    for (const QModelIndex &index : selection) {
       if(index.column() == 0)  {
           int row = index.row();
           AllCentr[m][row].selected = true;
       }
    }
    emit centroidChanged();
}

CentroidsDlg::CentroidsDlg(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::CentroidsDlg)
{
#if defined(JAV_WIDGET)
    jav = qobject_cast<CrystalViewWidget*>(parent);
#else
    jav = qobject_cast<JavWindow*>(parent);
#endif
    ui->setupUi(this);
    AllCentr = new QVector<jav_centroid_type> [MAX_NUM_MODELS]();
    jav_centroid_type::setJav(jav);
    QStringList titles = { "Number", "Show", "Label", "Show Label", "Colour",
                         "Transparency"};
    centModel = new QStandardItemModel(this);
    centModel->setHorizontalHeaderLabels(titles);
    CenterDelegate *delegate1 = new CenterDelegate(ui->tableCentroids);
    ui->tableCentroids->setItemDelegate(delegate1);
    CheckBoxDelegate *delegate = new CheckBoxDelegate(this);
    //delegate->setColumn(1);
    ui->tableCentroids->setItemDelegateForColumn(1,delegate);
    delegate = new CheckBoxDelegate(this);
    //delegate->setColumn(3);
    ui->tableCentroids->setItemDelegateForColumn(3,delegate);
    connect(centModel,SIGNAL(dataChanged(QModelIndex,QModelIndex)),
            this, SLOT(tableCentroids_dataChanged(QModelIndex,QModelIndex)));
    ui->tableCentroids->setModel(centModel);
    ui->tableCentroids->resizeColumnsToContents();
    QItemSelectionModel *selectionModel = ui->tableCentroids->selectionModel();
    connect(selectionModel, &QItemSelectionModel::selectionChanged,
           this, &CentroidsDlg::rowSelectionChanged);
}

CentroidsDlg::~CentroidsDlg()
{
    delete ui;
    delete [] AllCentr;
}

void CentroidsDlg::tableCentroids_dataChanged(const QModelIndex & current,
                                             const QModelIndex & )
{
    int whichModel = Properties::GetActiveModel() - 1;
    int col = current.column();
    int k = current.row();
    if(col == COL_LABEL)
        AllCentr[whichModel][k].label = current.data().toString();
    else if(col == COL_VIS)
        AllCentr[whichModel][k].visible = current.data(Qt::DisplayRole).toBool();
    else if(col == COL_SHOWLAB)
        AllCentr[whichModel][k].showlab = current.data(Qt::DisplayRole).toBool();
    else
        return;
    emit centroidChanged();
}

void CentroidsDlg::clearAll()
{
    for(int i=0; i<Properties::nModels;i++)
        AllCentr[i].clear();
    int nrows = centModel->rowCount();
    centModel->removeRows(0, nrows);
}

void CentroidsDlg::on_closeButton_clicked()
{
    reject();
}

void CentroidsDlg::on_findButton_clicked()
{
    int whichModel = Properties::GetActiveModel();
    jav_find_centroids_(&whichModel);
}

void CentroidsDlg::colorChanged(QColor newColor, int k)
{
    int whichModel = Properties::GetActiveModel() - 1;
    AllCentr[whichModel][k].colore = newColor;
    emit centroidChanged();
}

void CentroidsDlg::alphaChanged(int value, int k)
{
    int whichModel = Properties::GetActiveModel() - 1;
    AllCentr[whichModel][k].transparency = value;
    emit centroidChanged();
}

void CentroidsDlg::addCentroid(int whichModel, jav_centroid_type c)
{
    AllCentr[whichModel].push_back(c);
}

void CentroidsDlg::FillList()
{
    int nrows = centModel->rowCount();
    centModel->removeRows(0, nrows);
    int whichModel = Properties::GetActiveModel() - 1;
    if(whichModel == -1)
       return;
    if(AllCentr[whichModel].empty())
       return;
    int nCentroids = (int) AllCentr[whichModel].size();
    QVector<jav_centroid_type> Centr = AllCentr[whichModel];
    for(int i=0; i<nCentroids; i++)
    {
        QList<QStandardItem *> items;
        QStandardItem *item;
        item = new QStandardItem();               // Number
        item->setData(i+1, Qt::DisplayRole);
        item->setFlags(item->flags() &  ~Qt::ItemIsEditable);  //Not editable
        items.append(item);                       // Visible
        item = new QStandardItem();
        item->setData(Centr[i].visible?1:0, Qt::DisplayRole);
        items.append(item);
        item = new QStandardItem(Centr[i].label); // Label
        items.append(item);
        item = new QStandardItem();               // Label visible
        item->setData(Centr[i].showlab?1:0, Qt::DisplayRole);
        items.append(item);
        item = new QStandardItem();               // Button
        items.append(item);
        item = new QStandardItem();               // Slider
        items.append(item);
        centModel->appendRow(items);
        QWidget *wid = new QWidget(this);
        QGridLayout *layout = new QGridLayout();
        wid->setLayout(layout);
        //layout->setMargin(2); // Qt6 not compatible
        layout->setContentsMargins(2, 2, 2, 2);
        ColorButton *button = new ColorButton(Centr[i].colore, i, this);
        connect(button, &ColorButton::colorChanged, this,
                   &CentroidsDlg::colorChanged);
        layout->addWidget(button, 0, 0, 1, 1, Qt::AlignCenter);
        ui->tableCentroids->setIndexWidget(centModel->index(i,4), wid);
        QSlider *slider = new QSlider(Qt::Horizontal, this);
        slider->setRange(0, 100);
        slider->setValue(Centr[i].transparency);
        connect(slider, &QSlider::valueChanged, [=] {alphaChanged(slider->value(), i);});
        ui->tableCentroids->setIndexWidget(centModel->index(i,5), slider);
    }
    ui->tableCentroids->horizontalHeader()->setStretchLastSection(false);
    ui->tableCentroids->resizeColumnsToContents();
    ui->tableCentroids->resizeRowsToContents();
    ui->tableCentroids->horizontalHeader()->setStretchLastSection(true);
    int larg = 0;
    for(int i=0; i<6;i++)
        larg += ui->tableCentroids->columnWidth(i);
    int h = size().height();
    this->resize(larg+20, h);
}

void CentroidsDlg::NormalizzaCentroids(int kmodel)
{
    jav_centroid_type::radN = 1/Properties::deltav;
    if(AllCentr[kmodel].empty())
       return;
    int nCentroids = (int) AllCentr[kmodel].size();
    for(int i=0; i<nCentroids; i++)
        AllCentr[kmodel][i].Normalizza();
}

void CentroidsDlg::clearCentroids(int whichModel)
{
    AllCentr[whichModel].clear();
}
