#include "coddialog.h"
#include "ui_coddialog.h"
#include "qtdownload.h"
#include "json.h"
#include "savedialog.h"
#include "javkeysettings.h"
#include "myheader.h"
#if defined(JAV_WIDGET)
#include "crystalviewwidget.h"
#else
#include "javwindow.h"
#endif

#include <QDebug>
#include <algorithm>
#include <QMessageBox>
#include <QIntValidator>
#include <QJsonDocument>
#include <QJsonObject>
#include <QJsonArray>
#include <QChar>
#include <string>  
#include <fstream>
#include <memory>
#include <QDir>
#include <QUrl>
#include <QUrlQuery>
#include <QMenu>
#include <QClipboard>
#include <QDesktopServices>
#include <QSettings>
#include <itemdelegate.h>
//#include <javwindow.h>
#include <QRegularExpression>

//#define NEW 1

//int importFileCOD(const QString &fileName, int tipo);
extern "C" int get_hill_formula_(const char *cform, char *hform, size_t idum1, size_t idum2);
extern "C" void  jav_cod_(int *act, int *whichModel, int *kadd);

CODDialog::CODDialog(QWidget *parent, int WhichModel, int kadd) :
    QDialog(parent),
    ui(new Ui::CODDialog)
{
#if defined(JAV_WIDGET)
    jav = qobject_cast<CrystalViewWidget*>(parent);
#else
    jav = qobject_cast<JavWindow*>(parent);
#endif
    ui->setupUi(this);

    //Disable same buttons
    ui->buttonBox->button(QDialogButtonBox::Apply)->setEnabled(false);

    //ensure that some fields are valid
    QValidator *validator = new QIntValidator(1, 110, this);
    ui->lineEditMin->setValidator(validator);
    ui->lineEditMax->setValidator(validator);
    ui->lineEditYear->setValidator(new QIntValidator(1900,9999,this));
    QDoubleValidator *doubleValidator = new QDoubleValidator(this);
    ui->lineEditAmin->setValidator(doubleValidator);
    ui->lineEditAmax->setValidator(doubleValidator);
    ui->lineEditBmin->setValidator(doubleValidator);
    ui->lineEditBmax->setValidator(doubleValidator);
    ui->lineEditCmin->setValidator(doubleValidator);
    ui->lineEditCmax->setValidator(doubleValidator);
    ui->lineEditAlpmin->setValidator(doubleValidator);
    ui->lineEditAlpmax->setValidator(doubleValidator);
    ui->lineEditBetmin->setValidator(doubleValidator);
    ui->lineEditBetmax->setValidator(doubleValidator);
    ui->lineEditGamin->setValidator(doubleValidator);
    ui->lineEditGamax->setValidator(doubleValidator);
    ui->lineEditVmin->setValidator(doubleValidator);
    ui->lineEditVmax->setValidator(doubleValidator);
    QValidator *intValidator = new QIntValidator(this);
    ui->lineEditMinZ->setValidator(intValidator);
    ui->lineEditMaxZ->setValidator(intValidator);
    ui->lineEditMinZprime->setValidator(intValidator);
    ui->lineEditMaxZprime->setValidator(intValidator);
    ui->lineEditYear->setValidator(intValidator);
    ui->lineEditVolume->setValidator(intValidator);
    ui->lineEditIssue->setValidator(intValidator);

    // valid character for ID field are integers, ',' and '%'
    //ui->lineEditID->setValidator(new QRegExpValidator(QRegExp("[0-9,% ]*"), ui->lineEditID));
    ui->lineEditID->setValidator(new QRegularExpressionValidator(QRegularExpression("[0-9,% ]*"), ui->lineEditID));

    //signals
    connect(ui->lineEditText, SIGNAL(returnPressed()), this, SLOT(editingCompleted()));
    connect(ui->lineEditFormula, SIGNAL(returnPressed()), this, SLOT(editingCompleted()));
    connect(ui->lineEditElements, SIGNAL(returnPressed()), this, SLOT(editingCompleted()));
    connect(ui->lineEditNOT, SIGNAL(returnPressed()), this, SLOT(editingCompleted()));
    connect(ui->lineEditMin, SIGNAL(returnPressed()), this, SLOT(editingCompleted()));
    connect(ui->lineEditMax, SIGNAL(returnPressed()), this, SLOT(editingCompleted()));
    connect(ui->lineEditID, SIGNAL(returnPressed()), this, SLOT(editingCompleted()));
    connect(ui->lineEditJournal, SIGNAL(returnPressed()), this, SLOT(editingCompleted()));
    connect(ui->lineEditYear, SIGNAL(returnPressed()), this, SLOT(editingCompleted()));
    connect(ui->lineEditVolume, SIGNAL(returnPressed()), this, SLOT(editingCompleted()));
    connect(ui->lineEditIssue, SIGNAL(returnPressed()), this, SLOT(editingCompleted()));
    connect(ui->lineEditDoi, SIGNAL(returnPressed()), this, SLOT(editingCompleted()));
    connect(ui->lineEditAmin, SIGNAL(returnPressed()), this, SLOT(editingCompleted()));
    connect(ui->lineEditAmax, SIGNAL(returnPressed()), this, SLOT(editingCompleted()));
    connect(ui->lineEditBmin, SIGNAL(returnPressed()), this, SLOT(editingCompleted()));
    connect(ui->lineEditBmax, SIGNAL(returnPressed()), this, SLOT(editingCompleted()));
    connect(ui->lineEditCmin, SIGNAL(returnPressed()), this, SLOT(editingCompleted()));
    connect(ui->lineEditCmax, SIGNAL(returnPressed()), this, SLOT(editingCompleted()));
    connect(ui->lineEditAlpmin, SIGNAL(returnPressed()), this, SLOT(editingCompleted()));
    connect(ui->lineEditAlpmax, SIGNAL(returnPressed()), this, SLOT(editingCompleted()));
    connect(ui->lineEditBetmin, SIGNAL(returnPressed()), this, SLOT(editingCompleted()));
    connect(ui->lineEditBetmax, SIGNAL(returnPressed()), this, SLOT(editingCompleted()));
    connect(ui->lineEditGamin, SIGNAL(returnPressed()), this, SLOT(editingCompleted()));
    connect(ui->lineEditGamax, SIGNAL(returnPressed()), this, SLOT(editingCompleted()));
    connect(ui->lineEditVmin, SIGNAL(returnPressed()), this, SLOT(editingCompleted()));
    connect(ui->lineEditVmax, SIGNAL(returnPressed()), this, SLOT(editingCompleted()));
    connect(ui->lineEditMinZ, SIGNAL(returnPressed()), this, SLOT(editingCompleted()));
    connect(ui->lineEditMaxZ, SIGNAL(returnPressed()), this, SLOT(editingCompleted()));
    connect(ui->lineEditMinZprime, SIGNAL(returnPressed()), this, SLOT(editingCompleted()));
    connect(ui->lineEditMaxZprime, SIGNAL(returnPressed()), this, SLOT(editingCompleted()));

    connect(ui->lineEditText, SIGNAL(textChanged(QString)), this, SLOT(textingLine(QString)));
    connect(ui->lineEditFormula, SIGNAL(textChanged(QString)), this, SLOT(textingLine(QString)));
    connect(ui->lineEditElements, SIGNAL(textChanged(QString)), this, SLOT(textingLine(QString)));
    connect(ui->lineEditNOT, SIGNAL(textChanged(QString)), this, SLOT(textingLine(QString)));
    connect(ui->lineEditMin, SIGNAL(textChanged(QString)), this, SLOT(textingLine(QString)));
    connect(ui->lineEditMax, SIGNAL(textChanged(QString)), this, SLOT(textingLine(QString)));
    connect(ui->lineEditID, SIGNAL(textChanged(QString)), this, SLOT(textingLine(QString)));
    connect(ui->lineEditJournal, SIGNAL(textChanged(QString)), this, SLOT(textingLine(QString)));
    connect(ui->lineEditYear, SIGNAL(textChanged(QString)), this, SLOT(textingLine(QString)));
    connect(ui->lineEditVolume, SIGNAL(textChanged(QString)), this, SLOT(textingLine(QString)));
    connect(ui->lineEditIssue, SIGNAL(textChanged(QString)), this, SLOT(textingLine(QString)));
    connect(ui->lineEditDoi, SIGNAL(textChanged(QString)), this, SLOT(textingLine(QString)));
    connect(ui->lineEditAmin, SIGNAL(textChanged(QString)), this, SLOT(textingLine(QString)));
    connect(ui->lineEditAmax, SIGNAL(textChanged(QString)), this, SLOT(textingLine(QString)));
    connect(ui->lineEditBmin, SIGNAL(textChanged(QString)), this, SLOT(textingLine(QString)));
    connect(ui->lineEditBmax, SIGNAL(textChanged(QString)), this, SLOT(textingLine(QString)));
    connect(ui->lineEditCmin, SIGNAL(textChanged(QString)), this, SLOT(textingLine(QString)));
    connect(ui->lineEditCmax, SIGNAL(textChanged(QString)), this, SLOT(textingLine(QString)));
    connect(ui->lineEditAlpmin, SIGNAL(textChanged(QString)), this, SLOT(textingLine(QString)));
    connect(ui->lineEditAlpmax, SIGNAL(textChanged(QString)), this, SLOT(textingLine(QString)));
    connect(ui->lineEditBetmin, SIGNAL(textChanged(QString)), this, SLOT(textingLine(QString)));
    connect(ui->lineEditBetmax, SIGNAL(textChanged(QString)), this, SLOT(textingLine(QString)));
    connect(ui->lineEditGamin, SIGNAL(textChanged(QString)), this, SLOT(textingLine(QString)));
    connect(ui->lineEditGamax, SIGNAL(textChanged(QString)), this, SLOT(textingLine(QString)));
    connect(ui->lineEditVmin, SIGNAL(textChanged(QString)), this, SLOT(textingLine(QString)));
    connect(ui->lineEditVmax, SIGNAL(textChanged(QString)), this, SLOT(textingLine(QString)));
    connect(ui->lineEditMinZ, SIGNAL(textChanged(QString)), this, SLOT(textingLine(QString)));
    connect(ui->lineEditMaxZ, SIGNAL(textChanged(QString)), this, SLOT(textingLine(QString)));
    connect(ui->lineEditMinZprime, SIGNAL(textChanged(QString)), this, SLOT(textingLine(QString)));
    connect(ui->lineEditMaxZprime, SIGNAL(textChanged(QString)), this, SLOT(textingLine(QString)));

    // create QActions for ContextMenu
    m_copyAct = new QAction("Copy to Clipboard",this);
    connect(m_copyAct, SIGNAL(triggered()), this, SLOT(copy()));
    m_downloadAct = new QAction("Download CIF",this);
    connect(m_downloadAct, SIGNAL(triggered()), this, SLOT(download()));
    m_openUrl = new QAction("Open CIF in Browser",this);
    connect(m_openUrl, SIGNAL(triggered()), this, SLOT(myOpenUrl()));

    // set download for json file
    jsonDownloder = new QtDownload(this);
    connect(jsonDownloder, &QtDownload::downloadComplete, this, &CODDialog::jsonDownloadComplete);
    // set download for cif file by contextual menu
    CIFdownloder = new QtDownload(this);
    // set download cif files from tableView
    tableViewDownloder = new NetworkDownload(this);
    tableViewDownloder->setDownloadFolder(QDir::tempPath());
    connect(tableViewDownloder, &NetworkDownload::downloadCompleted, this, &CODDialog::downloadCompleteCIF);

    // set model for table
    model = new QStandardItemModel(this);
    model->setColumnCount(NUM_COLS);

    //set header of table
    QStringList labels = {"","COD number", "Spacegroup", "Formula", "Cell Volume",
                          "Cell Parameters", "Chemical Names","Bibliography"};
    model->setHorizontalHeaderLabels(labels);

    //mr
    cbHeader = new CheckBoxHeader(Qt::Horizontal, {COLPOS_CHECK}, false, ui->tableView);
    ui->tableView->setHorizontalHeader(cbHeader);
    cbHeader->setChecked(COLPOS_CHECK, false);
    connect(cbHeader, &CheckBoxHeader::checkBoxClicked, this, &CODDialog::on_checkBoxClicked);
    //end mr

    // TEST: add checkbox to header (sort appear disabled) TODO
//    QStandardItem* item0 = new QStandardItem();
//    item0->setCheckable(true);
//    item0->setCheckState(Qt::Unchecked);
//    //item0->setText("some text");
//    model->setHorizontalHeaderItem(COLPOS_CHECK,item0);
//    MyHeader *myheader = new MyHeader(Qt::Horizontal, ui->tableView);
//    ui->tableView->setHorizontalHeader(myheader);
//    //connect(ui->tableView->horizontalHeader(), SIGNAL(sectionClicked(int)), this, SLOT(my_slot(int)));

    // settings of table
    ui->tableView->setModel(model);
    ui->tableView->setSortingEnabled(true);
    ui->tableView->sortByColumn(COLPOS_ID,Qt::AscendingOrder);
    ui->tableView->setColumnHidden(COLPOS_NAME, true);
    ui->tableView->setColumnHidden(COLPOS_BIB, true);
    ui->tableView->resizeColumnsToContents();
    ui->tableView->setContextMenuPolicy(Qt::CustomContextMenu);

    //delegate for column with checkbox
    delegate = new CheckBoxDelegate(this);
    //delegate->setColumn(COLPOS_CHECK);
    ui->tableView->setItemDelegateForColumn(COLPOS_CHECK,delegate);

    connect(ui->tableView, SIGNAL(customContextMenuRequested(QPoint)),
            SLOT(customMenuRequested(QPoint)));
    connect(ui->tableView->selectionModel(),SIGNAL(currentRowChanged(QModelIndex,QModelIndex)),
            this, SLOT(tableView_currentRowChanged(QModelIndex,QModelIndex)));    
    connect(model,SIGNAL(dataChanged(QModelIndex,QModelIndex)),
            this, SLOT(tableView_dataChanged(QModelIndex,QModelIndex)));
}

void CODDialog::showCOD(int WhichModel, int kadd)
{
    m_kadd = kadd;
    m_WhichModel = WhichModel;
    if (m_kadd == 0) {
        ui->buttonBox->button(QDialogButtonBox::Apply)->setText("Import");
        this->setWindowTitle("Import Fragment from COD");
    }
    else {
        ui->buttonBox->button(QDialogButtonBox::Apply)->setText("Add");
        this->setWindowTitle("Add Fragment from COD");
    }
    int act = 0;
    jav_cod_(&act, &m_WhichModel, &m_kadd);
    if (!geometry.isEmpty()) this->setGeometry(geometry);
    this->show();
}

CODDialog::~CODDialog()
{
    delete ui;
    delete jsonDownloder;
    delete CIFdownloder;
}

void CODDialog::on_searchButton_clicked()
{        
    m_text = ui->lineEditText->text();
    m_formula = ui->lineEditFormula->text().trimmed();
    m_el = ui->lineEditElements->text();
    m_nel = ui->lineEditNOT->text();
    m_strictmin = ui->lineEditMin->text();
    m_strictmax = ui->lineEditMax->text();
    m_ID = ui->lineEditID->text();

    m_journal = ui->lineEditJournal->text();
    m_year = ui->lineEditYear->text();
    m_volume = ui->lineEditVolume->text();
    m_issue = ui->lineEditIssue->text();
    m_doi = ui->lineEditDoi->text();

    m_amin = ui->lineEditAmin->text();
    m_amax = ui->lineEditAmax->text();
    m_bmin = ui->lineEditBmin->text();
    m_bmax = ui->lineEditBmax->text();
    m_cmin = ui->lineEditCmin->text();
    m_cmax = ui->lineEditCmax->text();
    m_alpmin = ui->lineEditAlpmin->text();
    m_alpmax = ui->lineEditAlpmax->text();
    m_betmin = ui->lineEditBetmin->text();
    m_betmax = ui->lineEditBetmax->text();
    m_gamin = ui->lineEditGamin->text();
    m_gamax = ui->lineEditGamax->text();
    m_vmin = ui->lineEditVmin->text();
    m_vmax = ui->lineEditVmax->text();
    m_minZ = ui->lineEditMinZ->text();
    m_maxZ = ui->lineEditMaxZ->text();
    m_minZprime = ui->lineEditMinZprime->text();
    m_maxZprime = ui->lineEditMaxZprime->text();

    QUrl CODcomm = CODCommand();
    if (!CODcomm.isEmpty()) {        
        QApplication::setOverrideCursor(QCursor(Qt::WaitCursor));        
        jsonDownloder->startRequest(CODcomm,QDir::tempPath()+"/"+"tmp.json");
    }
}

void CODDialog::jsonDownloadComplete(bool success, QString errMsg)
{
    if (success) {
        QString err;
        int ier = fillList(err);
        QApplication::restoreOverrideCursor();
        if (ier) {
            clearResults();
            QMessageBox::information(this, tr("HTTP"),
                         tr("Download failed: %1.").arg(err));
        }
        ui->labelResults->setText(QString("%1 results found").arg(model->rowCount()));
    }
    else {
        QApplication::restoreOverrideCursor();
        QMessageBox::information(this, tr("HTTP"),
                     tr("Download failed: %1.").arg(errMsg));
    }
}

QString CODDialog::getElements(QString s, QString w, int lim)
{
    QStringList items = s.split(" ");
    QString s1, str;
    //int fin = std::min(items.size(),lim);
    int fin = std::min<int>(items.size(),lim);
    if (!s.isEmpty()) {
        for (int i=0; i < fin; i++){
             if (i) s1.append("&");
             s1 = s1 + w + str.setNum(i+1) + "=" + items.at(i);
        }
    }
    return s1;
}

void CODDialog::addItemsToQuery(QUrlQuery& q, QString s, QString w, int lim)
{
    QStringList items = s.split(" ");
    //int fin = std::min(items.size(),lim);
    int fin = std::min<int>(items.size(),lim);
    if (!s.isEmpty()) {
        for (int i=0; i < fin; i++){
             q.addQueryItem(w+QString::number(i+1),items.at(i));
        }
    }
}

bool CODDialog::form_per_COD(const QString &form1, QString &form2)
{
    char *cstr = new char[form1.length()*2+1];
    int ier = get_hill_formula_(form1.toStdString().c_str(), cstr, form1.length(), form1.length()*2);
    if (ier == 0) form2 = cstr;
    delete [] cstr;
    return (ier != 0);
}

#ifdef NEW
string filtrastr(const string &s, const char c)
{
   string s1;
   for(int i=0; i<(int)s.length(); i++)
   {
       if(s.at(i) != c)
          s1.append(1, s.at(i));
   }
   return s1;
}
string filtraFormula(string formula)
{
     string f2;
     f2 = filtrastr(formula, '"');
     size_t found = f2.find("- ");
     if(found != string::npos)
     {
        size_t found2 = f2.find(" -");
        if(found2 != string::npos)
           f2 = f2.substr(found+2, found2-found-2);
     }
     return f2;
}

int CODDialog::fillList(QString &errMsg)
{
    fstream ifs("tmp.json");
    if(!ifs.is_open()) {
        errMsg = "Failed to open JSON file tmp.json";
        return 1;
    }

    Json::CharReaderBuilder rbuilder;
    unique_ptr<Json::CharReader> const reader(rbuilder.newCharReader());
    string errs;
    Json::Value val;
    bool b = Json::parseFromStream(rbuilder, ifs, &val, &errs);
    ifs.close();
    if (!b)
    {
        errMsg = "Error: ";
        errMsg.append(errs.c_str());
        return 2;
    }
    size_t rowcount = 0;
    if(val.type() == Json::arrayValue)
        rowcount = val.size();
    else {
        errMsg = "JSON doc is not an array";
        return 4;
    }
    if(rowcount == 0) {
        errMsg = "The array is empty";
        clearResults();
        return 0;
    }
    model->setRowCount(rowcount);
    for (Json::ArrayIndex row=0; row<rowcount; row++)
    {
        QString stringa(filtrastr(val[row]["file"].toStyledString(), '"').c_str());
        QStandardItem *item = new QStandardItem(stringa);
        model->setItem(row,0,item);
        stringa.clear();
        if(val[row]["sg"].type() != Json::nullValue) {
            stringa.append(filtrastr(val[row]["sg"].toStyledString(), '"').c_str());
            item = new QStandardItem(stringa);
            model->setItem(row, 1, item);
            stringa.clear();
        }
        if(val[row]["formula"].type() != Json::nullValue) {
            stringa.append(filtraFormula(val[row]["formula"].asString()).c_str());
            item = new QStandardItem(stringa);
            model->setItem(row, 2, item);
            stringa.clear();
        }
        if(val[row]["vol"].type() != Json::nullValue) {
            stringa.append(filtrastr(val[row]["vol"].asString(), '"').c_str());
            item = new QStandardItem(stringa);
            model->setItem(row, 3, item);
            stringa.clear();
        }
        if(val[row]["a"].type() != Json::nullValue) {
            stringa.append(filtrastr(val[row]["a"].asString(), '"').c_str());
            stringa.append(" ");
            stringa.append(filtrastr(val[row]["b"].asString(), '"').c_str());
            stringa.append(" ");
            stringa.append(filtrastr(val[row]["c"].asString(), '"').c_str());
            stringa.append(" ");
            stringa.append(filtrastr(val[row]["alpha"].asString(), '"').c_str());
            stringa.append(" ");
            stringa.append(filtrastr(val[row]["beta"].asString(), '"').c_str());
            stringa.append(" ");
            stringa.append(filtrastr(val[row]["gamma"].asString(), '"').c_str());
            stringa.append(" ");
            item = new QStandardItem(stringa);
            model->setItem(row, 4, item);
            stringa.clear();
        }
        QString chemnames;
        if(val[row]["commonname"].type() != Json::nullValue) {
            chemnames.append(filtrastr(val[row]["commonname"].asString(), '"').c_str());
            chemnames.append("; ");
        }
        if(val[row]["chemname"].type() != Json::nullValue) {
            chemnames.append(filtrastr(val[row]["chemname"].asString(), '"').c_str());
            chemnames.append("; ");
        }
        if(val[row]["mineral"].type() != Json::nullValue) {
            chemnames.append(filtrastr(val[row]["mineral"].asString(), '"').c_str());
            chemnames.append("; ");
        }
        if (!chemnames.isEmpty()) {
             chemnames.remove(chemnames.lastIndexOf(QChar(';')),2);
             item = new QStandardItem(chemnames);
             model->setItem(row,5,item);
         }
        QString biblio;
        if(val[row]["authors"].type() != Json::nullValue) {
            biblio.append(filtrastr(val[row]["authors"].asString(), '"').c_str());
        }
        if(val[row]["title"].type() != Json::nullValue) {
            biblio.append("<br/>");
            biblio.append(filtrastr(val[row]["title"].asString(), '"').c_str());
        }
        if(val[row]["journal"].type() != Json::nullValue) {
            biblio.append("<br/>");
            biblio.append(filtrastr(val[row]["journal"].asString(), '"').c_str());
            if(val[row]["year"].type() != Json::nullValue) {
                biblio.append(", <b>");
                biblio.append(filtrastr(val[row]["year"].asString(), '"').c_str());
                biblio.append("</b>");
                if(val[row]["volume"].type() != Json::nullValue) {
                    biblio.append(", <i>");
                    biblio.append(filtrastr(val[row]["volume"].asString(), '"').c_str());
                    biblio.append("</i>");
                }
                if(val[row]["firstpage"].type() != Json::nullValue) {
                    biblio.append(", ");
                    biblio.append(filtrastr(val[row]["firstpage"].asString(), '"').c_str());
                    if(val[row]["lastpage"].type() != Json::nullValue) {
                        biblio.append("-");
                        biblio.append(filtrastr(val[row]["lastpage"].asString(), '"').c_str());
                    }
                }
            }
            if(val[row]["doi"].type() != Json::nullValue) {
                stringa.append(filtrastr(val[row]["doi"].asString(), '"').c_str());
                QString doi = QString("<a href=%1>%1</a>").arg("https://doi.org/"+stringa);
                biblio = biblio + "<br/>" + doi;
            }
            if (!biblio.isEmpty()) {
                item = new QStandardItem(biblio);
                model->setItem(row,6,item);
            }
        }
    }
    ui->tableView->setColumnHidden(5, true);
    ui->tableView->setColumnHidden(6, true);
    ui->tableView->resizeColumnsToContents();
    ui->tableView->horizontalHeader()->setSectionResizeMode(4,QHeaderView::Stretch);
    ui->tableView->selectRow(0);

    return 0;
}

#else

int CODDialog::fillList(QString &errMsg)
{
    QFile json_file(QDir::tempPath()+ "/" + "tmp.json");
    //qDebug() << "Read file" << json_file.fileName() << endl;
    if(!json_file.open(QIODevice::ReadOnly)){
        errMsg = "Failed to open JSON file " + json_file.fileName();
        return 1;
    }

    QTextStream file_text(&json_file);
    //QString json_string;
    //json_string = file_text.readAll();
    //QByteArray json_bytes = json_string.toLocal8Bit();
    QByteArray json_bytes = json_file.readAll();
    json_file.close();

    if (json_bytes.isEmpty()) {
        errMsg = "No data was currently available for reading from JSON file " + json_file.fileName();
        return 2;
    }
    //auto json_doc=QJsonDocument::fromJson(json_bytes);
    QJsonParseError err;
    QJsonDocument json_doc=QJsonDocument::fromJson(json_bytes,&err);
    if (err.error) {
        errMsg = err.errorString();
        return 3;
    }

//    if(json_doc.isNull()){
//        qDebug()<<"Failed to create JSON doc.";
//        return 2;
//    }
    if(!json_doc.isArray()){
        errMsg = "JSON doc is not an array";
        return 4;
    }

    QJsonArray json_array = json_doc.array();
    if(json_array.isEmpty()){
        errMsg = "The array is empty";
        clearResults();
        return 0;
    }

    model->setRowCount(json_array.count());    

    //Generate data
    for (int row = 0; row < json_array.count(); row++) {
//         QStandardItem *check = new QStandardItem();
//         check->setFlags(Qt::ItemIsUserCheckable | Qt::ItemIsEnabled);
//         check->setData(Qt::Unchecked, Qt::CheckStateRole);
//         model->setItem(row, COLPOS_CHECK, check);
         QModelIndex index = model->index(row, COLPOS_CHECK, QModelIndex());
         //model->setData(index,QVariant(0));
         bool checked = cbHeader->isChecked(COLPOS_CHECK);
         model->setData(index,QVariant(checked?1:0));

         QJsonObject subtree = json_array.at(row).toObject();

         QStandardItem *item = new QStandardItem(subtree.value("file").toString());
         model->setItem(row,COLPOS_ID,item);

         if (!subtree.value("sg").isNull()) {
             item = new QStandardItem(subtree.value("sg").toString());
             model->setItem(row,COLPOS_SG,item);
         }

         if (!subtree.value("formula").isNull()) {
             QString formula = subtree.value("formula").toString();
             formula.remove("- ").remove(" -");
             item = new QStandardItem(formula);
             model->setItem(row,COLPOS_FORM,item);
         }

         if (!subtree.value("vol").isNull()) {
             //item = new QStandardItem(subtree.value("vol").toString());
             item = new QStandardItem;
             //item->setValue(subtree.value("vol"));

             //use setData to have numeric sort on volume.
             //subtree.value("vol").toDouble doesn't work. Why?
             item->setData(subtree.value("vol").toVariant().toDouble(), Qt::DisplayRole);
             model->setItem(row,COLPOS_VOL,item);
         }

         if (!subtree.value("a").isNull()) {
             QString cell = subtree.value("a").toString() + " "
                          + subtree.value("b").toString() + " "
                          + subtree.value("c").toString() + " "
                          + subtree.value("alpha").toString() + " "
                          + subtree.value("beta").toString() + " "
                          + subtree.value("gamma").toString() + " ";
             item = new QStandardItem(cell);
             model->setItem(row,COLPOS_CELL,item);
         }

         QString chemnames;
         if (!subtree.value("commonname").isNull()) {
             chemnames.append(subtree.value("commonname").toString());
             chemnames.append("; ");
         }
         if (!subtree.value("chemname").isNull()) {
             chemnames.append(subtree.value("chemname").toString());
             chemnames.append("; ");
         }
         if (!subtree.value("mineral").isNull()) {
             chemnames.append(subtree.value("mineral").toString());
             chemnames.append("; ");
         }
         if (!chemnames.isEmpty()) {
             chemnames.remove(chemnames.lastIndexOf(QChar(';')),2);
             item = new QStandardItem(chemnames);
             model->setItem(row,COLPOS_NAME,item);
         }

         QString biblio;
         if (!subtree.value("authors").isNull()){
             biblio.append(subtree.value("authors").toString());
         }
         if (!subtree.value("title").isNull()){
             biblio.append("<br/>");
             biblio.append(subtree.value("title").toString());
         }
         if (!subtree.value("journal").isNull()){
             biblio.append("<br/>");
             biblio.append(subtree.value("journal").toString());
             if (!subtree.value("year").isNull()){
                 biblio.append(", <b>");
                 biblio.append(subtree.value("year").toString());
                 biblio.append("</b>");
                 if (!subtree.value("volume").isNull()){
                     biblio.append(", <i>");
                     biblio.append(subtree.value("volume").toString());
                     biblio.append("</i>");
                 }
                 if (!subtree.value("firstpage").isNull()){
                     biblio.append(", ");
                     biblio.append(subtree.value("firstpage").toString());
                     if (!subtree.value("lastpage").isNull()){
                         biblio.append("-");
                         biblio.append(subtree.value("lastpage").toString());
                     }
                 }
             }
         }
         if (!subtree.value("doi").isNull()){
             QString doi = QString("<a href=%1>%1</a>").arg("https://doi.org/"+subtree.value("doi").toString());            
             biblio = biblio + "<br/>" + doi;
         }
         if (!biblio.isEmpty()) {
             item = new QStandardItem(biblio);
             model->setItem(row,COLPOS_BIB,item);
         }

    }
    ui->tableView->setColumnHidden(COLPOS_NAME, true);
    ui->tableView->setColumnHidden(COLPOS_BIB, true);
    ui->tableView->resizeColumnsToContents();
    //ui->tableView->horizontalHeader()->stretchLastSection(); //NOT WORKING!
    ui->tableView->horizontalHeader()->setSectionResizeMode(COLPOS_CELL,QHeaderView::Stretch);        

    // Get selected row
    QModelIndexList indexList = ui->tableView->selectionModel()->selectedIndexes();
    int row=-1;
    foreach (QModelIndex index, indexList) {
        row = index.row();
    }

    // Select row 0
    ui->tableView->selectRow(0);
    if (row == 0) {
        // force update when the row 0 has been already selected
        tableView_currentRowChanged(model->index(0,0,QModelIndex()),model->index(0,0,QModelIndex()));
    }

    ui->buttonBox->button(QDialogButtonBox::Apply)->setEnabled(true);

    return 0;
}
#endif

QUrl CODDialog::CODCommand()
{
    QUrlQuery q;
    QString command;
    bool first_com = true;
    if(!m_ID.isEmpty())
       q.addQueryItem("id",m_ID);

    if(!m_text.isEmpty())
       addItemsToQuery(q,m_text,"text",2);

    if(!m_formula.isEmpty()){
       QString formula;
       bool ier = form_per_COD(m_formula,formula);
       if (ier)
       {
           int ret = QMessageBox::warning(this, tr("Error"),
                     tr("Sorry. Wrong formula."),QMessageBox::Cancel);
       }
       else
       {
          q.addQueryItem("formula",formula);
       }
    }

    if(!m_el.isEmpty())
       addItemsToQuery(q,m_el,"el",8);

    if(!m_nel.isEmpty())
       addItemsToQuery(q,m_nel,"nel",4);

    if(!m_strictmin.isEmpty())
       q.addQueryItem("strictmin",m_strictmin);

    if(!m_strictmax.isEmpty())
       q.addQueryItem("strictmax",m_strictmax);

    if (!m_amin.isEmpty())
        q.addQueryItem("amin",m_amin);

    if (!m_amax.isEmpty())
        q.addQueryItem("amax",m_amax);

    if (!m_bmin.isEmpty())
        q.addQueryItem("bmin",m_bmin);

    if (!m_bmax.isEmpty())
        q.addQueryItem("bmax",m_bmax);

    if (!m_cmin.isEmpty())
        q.addQueryItem("cmin",m_cmin);

    if (!m_cmax.isEmpty())
        q.addQueryItem("cmax",m_cmax);

    if (!m_alpmin.isEmpty())
        q.addQueryItem("alpmin",m_alpmin);

    if (!m_alpmax.isEmpty())
        q.addQueryItem("alpmax",m_alpmax);

    if (!m_betmin.isEmpty())
        q.addQueryItem("betmin",m_betmin);

    if (!m_betmax.isEmpty())
        q.addQueryItem("betmax",m_betmax);

    if (!m_gamin.isEmpty())
        q.addQueryItem("gamin",m_gamin);

    if (!m_gamax.isEmpty())
        q.addQueryItem("gamax",m_gamax);

    if (!m_vmin.isEmpty())
        q.addQueryItem("vmin",m_vmin);

    if (!m_vmax.isEmpty())
        q.addQueryItem("vmax",m_vmax);

    if (!m_maxZ.isEmpty())
        q.addQueryItem("maxZ",m_maxZ);

    if (!m_minZ.isEmpty())
        q.addQueryItem("minZ",m_minZ);

    if (!m_maxZprime.isEmpty())
        q.addQueryItem("maxZprime",m_maxZprime);

    if (!m_minZprime.isEmpty())
        q.addQueryItem("minZprime",m_minZprime);

    if(!m_journal.isEmpty())
       q.addQueryItem("journal",m_journal);

    if(!m_year.isEmpty())
       q.addQueryItem("year",m_year);

    if(!m_volume.isEmpty()){
       if (!first_com) command.append("&");
       first_com = false;
       command.append(QString("volume=%1").arg(m_volume));
       q.addQueryItem("volume",m_volume);
    }

    if(!m_issue.isEmpty())
       q.addQueryItem("issue",m_issue);

    if(!m_doi.isEmpty())
       q.addQueryItem("doi",m_doi);

    if (q.isEmpty()) return QUrl();

    QUrl url("http://www.crystallography.net/cod/result.php");
    q.addQueryItem("format","json");
    url.setQuery(q);
    qDebug() << "URL=" << url;

    return url;
}

void CODDialog::showCrystalDetails(const QModelIndex &index)
{
    QModelIndex index_data = model->index(index.row(),COLPOS_NAME,QModelIndex());
    ui->chemicalNames->setText(model->data(index_data).toString());

    index_data = model->index(index.row(),COLPOS_BIB,QModelIndex());
    ui->bibliography->setText(model->data(index_data).toString());
}

void CODDialog::clearResults()
{
    model->removeRows(0,model->rowCount());
    ui->chemicalNames->clear();
    ui->bibliography->clear();
    ui->buttonBox->button(QDialogButtonBox::Apply)->setEnabled(false);
}

// OLD VERSION
//void CODDialog::tableView_currentRowChanged(const QModelIndex &current, const QModelIndex &previous)
//{
//    Q_UNUSED(previous);

//    showCrystalDetails(current);

//    QString fileName = getFileName(current);
//    QUrl url("http://www.crystallography.net/cod/" + fileName);
//    m_fileCIFSelected = QDir::tempPath()+ "/" +fileName;

//    if (!QFile(m_fileCIFSelected).exists()) {
//        QCoreApplication::processEvents(); //try processing pending downlods

//        QtDownload *m_downloadCIF = new QtDownload(this);
//        connect(m_downloadCIF, SIGNAL(downloadComplete(bool,QString)), this, SLOT(downloadCompleteCIF(bool,QString)));
//        //qDebug() << "Start download!! File: " << m_fileCIFSelected;
//        m_downloadCIF->startRequest(url,m_fileCIFSelected);

////        QEventLoop loop;
////        connect(m_downloadCIF, SIGNAL(downloadComplete(bool,QString)), &loop, SLOT(quit()));
////        loop.exec();

//    }
//    else
//    {
//        //qDebug() << "File exists!! File: " << m_fileCIFSelected;
//        importFileCOD(m_fileCIFSelected, 0);
//    }
//}

void CODDialog::tableView_currentRowChanged(const QModelIndex &current, const QModelIndex &previous)
{
    Q_UNUSED(previous);

    showCrystalDetails(current);

    QString fileName = getFileName(current);
    QUrl url("http://www.crystallography.net/cod/" + fileName);
    m_fileCIFSelected = QDir::tempPath()+ "/" +fileName;

    if (!QFile(m_fileCIFSelected).exists()) {
        tableViewDownloder->doDownload(url);
    }
    else
        jav->importFileCOD(m_fileCIFSelected, 0);
}

void CODDialog::downloadCompleteCIF(bool success, const QString &errMsgFile)
{
    if (success) {
        //QFileInfo fi(m_fileCIFSelected);
        //QFileInfo fiErr(errMsg);
        //qDebug() << "Download finished!!" << "File: " << m_fileCIFSelected << " Size: " << fi.size() << "Err: " << errMsg << " Size: " << fiErr.size();
        //importFileCOD(m_fileCIFSelected, 0);
        jav->importFileCOD(errMsgFile, 0);
    }
}

bool anyCheckON(const QStandardItemModel& model, int colpos)
{
    for (int i = 0; i < model.rowCount(); i++) {
         if (model.index(i,colpos).data().toBool()) return true;
    }
    return false;
}

int numCheckON(const QStandardItemModel& model, int colpos)
{
    int num=0;
    for (int i = 0; i < model.rowCount(); i++) {
         if (model.index(i,colpos).data().toBool()) num++;
    }
    return num;
}

void CODDialog::tableView_dataChanged(const QModelIndex &current, const QModelIndex &previous)
{
    if (previous.column() != COLPOS_CHECK) return;    
    int nCheck = numCheckON(*model, COLPOS_CHECK);
    ui->downloadButton->setEnabled(nCheck>0);
    QString text("Downloads");
    if (nCheck > 0) text.append(QString("(%1)").arg(nCheck));
    ui->downloadButton->setText(text);
}

void CODDialog::customMenuRequested(QPoint pos)
{
    indexMenu=ui->tableView->indexAt(pos);
    QMenu *menu=new QMenu(this);
    menu->addAction(m_copyAct);
    menu->addAction(m_downloadAct);
    menu->addAction(m_openUrl);

    m_copyAct->setEnabled(indexMenu.isValid());
    m_downloadAct->setEnabled(indexMenu.isValid());
    m_openUrl->setEnabled(indexMenu.isValid());

    menu->popup(ui->tableView->viewport()->mapToGlobal(pos));
}

void CODDialog::copy()
{    
    QClipboard *p_Clipboard = QApplication::clipboard();
    p_Clipboard->setText(model->data(indexMenu).toString());
}

QString CODDialog::getFileName(const QModelIndex &index)
{
    QModelIndex indexID = model->index(index.row(),COLPOS_ID,QModelIndex());
    QString fileNameCIF = model->data(indexID).toString()+".cif";

    return fileNameCIF;
}

bool CODDialog::linesEmpty()
{
    if (!ui->lineEditText->text().isEmpty()) return false;
    if (!ui->lineEditFormula->text().isEmpty()) return false;
    if (!ui->lineEditElements->text().isEmpty()) return false;
    if (!ui->lineEditNOT->text().isEmpty()) return false;
    if (!ui->lineEditMin->text().isEmpty()) return false;
    if (!ui->lineEditMax->text().isEmpty()) return false;
    if (!ui->lineEditID->text().isEmpty()) return false;
    if (!ui->lineEditJournal->text().isEmpty()) return false;
    if (!ui->lineEditYear->text().isEmpty()) return false;
    if (!ui->lineEditVolume->text().isEmpty()) return false;
    if (!ui->lineEditIssue->text().isEmpty()) return false;
    if (!ui->lineEditDoi->text().isEmpty()) return false;
    if (!ui->lineEditAmin->text().isEmpty()) return false;
    if (!ui->lineEditAmax->text().isEmpty()) return false;
    if (!ui->lineEditBmin->text().isEmpty()) return false;
    if (!ui->lineEditBmax->text().isEmpty()) return false;
    if (!ui->lineEditCmin->text().isEmpty()) return false;
    if (!ui->lineEditCmax->text().isEmpty()) return false;
    if (!ui->lineEditAlpmin->text().isEmpty()) return false;
    if (!ui->lineEditAlpmax->text().isEmpty()) return false;
    if (!ui->lineEditBetmin->text().isEmpty()) return false;
    if (!ui->lineEditBetmax->text().isEmpty()) return false;
    if (!ui->lineEditGamin->text().isEmpty()) return false;
    if (!ui->lineEditGamax->text().isEmpty()) return false;
    if (!ui->lineEditVmin->text().isEmpty()) return false;
    if (!ui->lineEditVmax->text().isEmpty()) return false;
    if (!ui->lineEditMinZ->text().isEmpty()) return false;
    if (!ui->lineEditMaxZ->text().isEmpty()) return false;
    if (!ui->lineEditMinZprime->text().isEmpty()) return false;
    if (!ui->lineEditMaxZprime->text().isEmpty()) return false;
    return true;
}

void CODDialog::download()
{
    QString fileNameCIF = getFileName(indexMenu);
    QSettings settings;
    QString FilesToExport = "CIF file (*.cif);;All Files (*.*)";
    QString selectedFilter = "CIF file (*.cif)";
    QString fileName = SaveDialog::run(this,
                                   tr("Save File As"),
                                   settings.value(DEFAULT_DIR_KEY).toString(),
                                   fileNameCIF,
                                   FilesToExport,
                                   "cif",
                                   selectedFilter);
    if (!fileName.isEmpty()) {
        QFileInfo finfo(fileName);
        settings.setValue(DEFAULT_DIR_KEY,finfo.absolutePath());
        QUrl url("http://www.crystallography.net/cod/" + fileNameCIF);
        CIFdownloder->startRequest(url,fileName);
    }
}

void CODDialog::myOpenUrl()
{
    QModelIndex indexID = model->index(indexMenu.row(),COLPOS_ID,QModelIndex());
    QString link = "http://www.crystallography.net/cod/" + model->data(indexID).toString() + ".html";
    QDesktopServices::openUrl(QUrl(link));
}

void CODDialog::editingCompleted()
{
    on_searchButton_clicked();
}

void CODDialog::textingLine(QString line)
{
    //qDebug() << "Edit: " << line << "Line: " << ui->lineEditText->text().trimmed().isEmpty();
    QLineEdit* lineEdit = qobject_cast<QLineEdit*>(sender());
    //qInfo() << "Name: " << lineEdit->objectName() <<  lineEdit->hasAcceptableInput();
    if (!line.isEmpty())
        //ui->searchButton->setEnabled(true);
        ui->searchButton->setEnabled(lineEdit->hasAcceptableInput());
    else
        ui->searchButton->setEnabled(!linesEmpty());
}

void CODDialog::on_downloadButton_clicked()
{
    bool filesToDownload = false;
    for(int row=0; row < model->rowCount(); row++){
        QModelIndex index = model->index(row,COLPOS_CHECK,QModelIndex());
        //if (model->data(model->index(row,COLPOS_CHECK)).toBool()) {
        if (model->data(index).toBool()) {
            filesToDownload = true;
            break;
        }
    }
    if (filesToDownload) {
        QSettings settings;
        QString dir = QFileDialog::getExistingDirectory(this, tr("Select Download Directory"),
                                                        settings.value(DEFAULT_DIR_KEY).toString(),
                                                        QFileDialog::ShowDirsOnly
                                                        | QFileDialog::DontResolveSymlinks);
        if (!dir.isEmpty()) {
            settings.setValue(DEFAULT_DIR_KEY,dir);

            for(int row=0; row < model->rowCount(); row++){
                QModelIndex index = model->index(row,COLPOS_CHECK,QModelIndex());
                if (model->data(index).toBool()) {                    
                    QtDownload* dlerCIF = new QtDownload(this);
                    QString cifName = getFileName(index);
                    QUrl url("http://www.crystallography.net/cod/" + cifName);
                    dlerCIF->startRequest(url,dir+'/'+cifName);
                }
            }
        }
    }
}

void CODDialog::on_buttonBox_clicked(QAbstractButton *button)
{
    QDialogButtonBox::StandardButton stdButton = ui->buttonBox->standardButton(button);

   // qDebug() << "Name: " << stdButton.text();
    if(stdButton == QDialogButtonBox::Apply)
    {
        int act = 1;
        jav_cod_(&act, &m_WhichModel, &m_kadd);
        geometry = QDialog::geometry();
        QDialog::accept();
    } else if (stdButton == QDialogButtonBox::Close) {
        int act = 2;
        jav_cod_(&act, &m_WhichModel, &m_kadd);
        geometry = QDialog::geometry();
        QDialog::reject();
    }
}

void CODDialog::on_checkBoxClicked()
{
    for (int row = 0; row < model->rowCount(); row++) {
         bool checked = cbHeader->isChecked(COLPOS_CHECK);
         QModelIndex index = model->index(row, COLPOS_CHECK, QModelIndex());
         model->setData(index,QVariant(checked?1:0));
    }
}
