#include "constraintsdialog.h"
#include "ui_constraintsdialog.h"

extern "C" void get_constraint_info(int idph, int *riding, float *ride_adp_factor, int *constr_adp_type);
extern "C" void put_constraint_info(int idph, int riding, float ride_adp_factor, int constr_adp_type);

ConstraintsDialog::ConstraintsDialog(int idph, QWidget *parent) :
    QDialog(parent),
    ui(new Ui::ConstraintsDialog),
    phaseId(idph)
{
    ui->setupUi(this);

    int riding;
    float adpFactor;
    int constrAdpType;
    get_constraint_info(phaseId, &riding, &adpFactor, &constrAdpType);
    ui->ridingCheckBox->setChecked(riding);
    ui->adpSpinBox->setValue(adpFactor);
    setConstrAdpType(constrAdpType);

    connect(ui->buttonBox, &QDialogButtonBox::clicked, this, &ConstraintsDialog::buttonBoxClicked);
}

ConstraintsDialog::~ConstraintsDialog()
{
    delete ui;
}

void ConstraintsDialog::buttonBoxClicked(QAbstractButton *button)
{
    QDialogButtonBox::StandardButton stdButton = ui->buttonBox->standardButton(button);

    if (stdButton == QDialogButtonBox::Ok) {
        int riding = ui->ridingCheckBox->isChecked();
        float adpFactor = ui->adpSpinBox->value();
        int constrAdpType = getConstrAdpType();
        put_constraint_info(phaseId, riding, adpFactor, constrAdpType);
    }
}

void ConstraintsDialog::setConstrAdpType(int constrAdpType)
{
    switch (constrAdpType) {
    case CONSTR_ADP_NONE:
        ui->adpNoneRadioButton->setChecked(true);
        break;
    case CONSTR_ADP_OVERALL:
        ui->overallAdpRadioButton->setChecked(true);
        break;
    case CONSTR_ADP_SPECIES:
        ui->speciesAdpRadioButton->setChecked(true);
        break;
    case CONSTR_ADP_MOLECULES:
        ui->moleculesAdpRadioButton->setChecked(true);
    }
}

int ConstraintsDialog::getConstrAdpType()
{
    if (ui->overallAdpRadioButton->isChecked()) return CONSTR_ADP_OVERALL;
    if (ui->speciesAdpRadioButton->isChecked()) return CONSTR_ADP_SPECIES;
    if (ui->moleculesAdpRadioButton->isChecked()) return CONSTR_ADP_MOLECULES;
    return CONSTR_ADP_NONE;
}
