#include "crystalviewwidget.h"
#include "ui_crystalviewwidget.h"
#include "qt_utils.h"
#include "filtri.h"
#include "savedialog.h"
#include "properties.h"
#include "javkeysettings.h"
#include "mainwindow.h"

#include <QOffscreenSurface>
#include <QSettings>
#include <QtGlobal>

#define LEN_SPEC 8

CrystalViewWidget *CRYSTALVIEW;

extern "C" void jav_import_structure_cod_(const char *filename, int *tipo, int *wm, int *ier, size_t sz);
extern "C" void jav_export_structure_(const char *filename, int *tipo, int *wm, int *ifD, size_t sz);
extern "C" void jav_undo_redo_(int *code);
extern "C" void getsname_(char *name, size_t dum);
extern "C" void getstructureinfo_(structure_info_type *StructInfo, int *ifCalc, int *wm);
extern "C" void jav_get_au_(atom_type *atomi, int *num_atomi, int *kmodel);
extern "C" void jav_polyhedra_(int *wm);
extern "C" int jav_hydrogen_bonds_(int *wm);
extern "C" int jav_short_contacts_(int *wm);
extern "C" void applicasimmetria_(int *which);
extern "C" int jav_grow_fragment_(int *wm);
extern "C" void jav_show_multbonds_(int *i);
extern "C" int  jav_modify_object_(int *kptra,int *nptra,int *kptrb,int *nptrb,
                                  int *cod, float *threshold, int *wm, int *spec=nullptr,
                                  const char *newname=nullptr, size_t idum=0);
extern "C" void jav_optimize_geometry_(int *whichModel, int *kptra, int *nptra);
extern "C" void jav_expand_selection_(int *kptra,int *nptra, int *wm);
extern "C" void jav_expand_contacts_(int *whichAtom, int *whichModel, int *code, int *ncontact);
extern "C" void overlay_struct_(const char *filnam, size_t ldum);
extern "C" void jav_sphere_(int *kat,float *dist, int *wm);
extern "C" void jav_get_info_overlay1_(float *dist, int *nass, int *n0, int *n1,
                                       int *n2, float *distmin, float *distm, float *rmsd,
                                       float *rmsd_nonH, float *errph, int *nref, int *whichModel);
extern "C" void jav_get_info_overlay2_(float *dist, int *nv1, int *nv2,
                                       float *vdist, atom_type *at1,
                                       atom_type *at2, float *s, int *whichModel);
extern "C" void jav_delete_model_(int *Which);
extern "C" void jav_normalize_structure_(int *code, float vett[], int *wm);
extern "C" void jav_standardization_(int *kmodel);
extern "C" void Lanciajav(int itypef, int iAction, int *ierf);

CrystalViewWidget::CrystalViewWidget(QWidget *parent)
    : QWidget(parent)
    , ui(new Ui::CrystalViewWidget),
    LastModel(""),
    defaultBondMult(0),
    compare_info(nullptr)
{
    ui->setupUi(this);

    int maxSamples = getMaxSamples();
    Properties::FillVectSamples(maxSamples);
    mOpenGLWindow = ui->openGLWidget;
    mOpenGLWindow->setMaxSamples(maxSamples);

    CRYSTALVIEW = this;
    mPolyhedraDialog = new PolyhedraDialog(this);
    mCODDialog = new CODDialog(this);
    mOptionsDialog = new OptionsDialog(this);
    mHBOptionsDialog = new HBondsOptionsDialog(this);
    mInfoDialog = new InfoDialog(this);
    mBondSettingsDialog = new BondSettingsDialog(this);
    mPdPDialog = new PowderPattern(this);
    mPDFDialog = new PairDist(this);
    mRenumDlg = new RenumberDialog(this);
    mGeoCalcDlg = new GeoCalc(this);
    mSphereDlg = new SphereDialog(this);
    connect(mSphereDlg, &SphereDialog::rejected, this, &CrystalViewWidget::on_Rejected);
    connect(mSphereDlg, &SphereDialog::selectedCentre, this, &CrystalViewWidget::on_SelectedCentre);
    connect(mSphereDlg, &SphereDialog::selectedRadius, this, &CrystalViewWidget::on_SelectedRadius);
    connect(mSphereDlg, &SphereDialog::sphereSelectionChanged, this, &CrystalViewWidget::on_SelectedRow);
    mBondValenceDlg = new BondValenceDialog(this);
    mMopacDlg = new MopacDialog(this);
    mOverlay = new OverlayInfo(this);
    connect(mOverlay, &OverlayInfo::distChanged, this, &CrystalViewWidget::on_DistChanged);
    connect(mOverlay, &OverlayInfo::overlaySelectionChanged, this, &CrystalViewWidget::on_OverlaySelectionChanged);
    connect(mOverlay, &OverlayInfo::overlayModelSelected, this, &CrystalViewWidget::on_OverlayModelSelected);
    mCentDlg = new CentroidsDlg(this);
    connect(mCentDlg, &CentroidsDlg::centroidChanged, this, &CrystalViewWidget::on_centroidChanged);
    mTransformDlg = new TransformDialog(this);
    connect(mTransformDlg, &TransformDialog::trasformDlgClosed, this, &CrystalViewWidget::on_trasformDlgClosed);
    mPropDlg = new PropertiesDialog(this);
    connect(mPropDlg, &PropertiesDialog::propertiesDialogClosed, this, &CrystalViewWidget::on_propertiesDialogClosed);
    connect(mPropDlg, &PropertiesDialog::selectedAtom, this, &CrystalViewWidget::on_selectedAtom);
    connect(mPropDlg, &PropertiesDialog::selectedBond, this, &CrystalViewWidget::on_selectedBond);
    connect(mPropDlg, &PropertiesDialog::selectedAngle, this, &CrystalViewWidget::on_selectedAngle);
    connect(mPropDlg, &PropertiesDialog::selectedTorsion, this, &CrystalViewWidget::on_selectedTorsion);
    mNormHydroDlg = new NormHydroDlg(this);
    connect(mNormHydroDlg, &NormHydroDlg::normHydroDlgClosed, this, &CrystalViewWidget::on_normHydroDlgClosed);
#if defined(JAV_WIDGET)
    ModelInfo = new Model[MAX_NUM_MODELS]();
    MapInfo = new Mappa[2*MAX_NUM_MAPS]();
    for(int i=0; i<2*MAX_NUM_MAPS; i++)
        MapInfo[i].SetColor(i%7, false);
    for(int i=0; i<MAX_NUM_MODELS; i++)
        ModelInfo[i].VWSurface.SetColor(i, true);
#endif
    Properties::Initialize();
    //this->setMinimumSize(600,800);

    //get top level window
    mainWindow = getMainWindow();

    //set size based on main window
    //QSize size = mainWindow->size();
    //this->resize(size.width()/3, size.height()/2);
}

CrystalViewWidget::~CrystalViewWidget()
{
    delete ui;
}

int CrystalViewWidget::IsValidModel()
{
    int whichModel = Properties::GetActiveModel() - 1;
    if(whichModel == -1)
        return -1;
    if(!ModelInfo[whichModel].created)
        return -1;
    return whichModel;
}

int CrystalViewWidget::GetInfo(int code, int whichModel)
{
    int result;
    switch(code)
    {
    default:
        result = 0;
        break;
    case IFCELL:
        //result = mainWindow->getActionView_Cell_Edges()->isChecked()?1:0;
        result = menu()->actionView_Cell_Edges->isChecked()?1:0;
        break;
    case IFGROW:
        //result = mainWindow->getActionGrow_Fragments()->isChecked()?1:0;
        result = menu()->actionGrow_Fragments->isChecked()?1:0;
        break;
    case IFHBONDS:
        //result = mainWindow->getActionHydrogen_Bonds()->isChecked()?1:0;
        result = menu()->actionHydrogen_Bonds->isChecked()?1:0;
        break;
    case IFSHORTC:
        //result = mainWindow->getActionShort_Contacts()->isChecked()?1:0;
        result = menu()->actionShort_Contacts->isChecked()?1:0;
        break;
    case IFDELHANG:
        //result = mainWindow->getActionDelete_Hanging_Contacts()->isChecked()?1:0;
        result = menu()->actionDelete_Hanging_Contacts->isChecked()?1:0;
        break;
    case IFSYMM:
        result = ModelInfo[whichModel].ifSymmetry?1:0;
        break;
    case PLOTSTYLE:
        result = ModelInfo[whichModel].plotStyle + 1;
        break;
    case IFLABELS:
        result = ModelInfo[whichModel].ifLabels?1:0;
        break;
    case SYMLEVEL:
        result = 0;
        break;
    case IFPOLYHEDRA:
        result = ModelInfo[whichModel].ifPoly?1:0;
        break;
    }
    return result;
}

ColorWidget::ColorType CrystalViewWidget::GetBgType()
{
    return mOpenGLWindow->GetBgType();
}

void CrystalViewWidget::SetUndoRedo(int but, bool bstato, char *str)
{
    QString label_menu;
    QAction *item;
    if(but == 0)
    {
        //item = mainWindow->getActionUndo();
        item = menu()->actionUndo;
        label_menu = "Undo ";
        m_statusUndo = bstato;
    }
    else
    {
        //item = mainWindow->getActionRedo();
        item = menu()->actionRedo;
        label_menu = "Redo ";
        m_statusRedo = bstato;
    }
    item->setEnabled(bstato);
    if(str != nullptr)
    {
        label_menu.append(str);
        item->setText(label_menu);
    }
}

void CrystalViewWidget::readSettings(int which)
{
    QSettings settings;
    int sz = settings.beginReadArray(MODEL_OPTIONS_KEYS);
    if(which < sz)
    {
        settings.setArrayIndex(which);
        ModelInfo[which].plotStyle = settings.value(PLOT_STYLE_KEY).toInt();
        ModelInfo[which].colorMode = settings.value(COLOUR_STYLE_KEY).toInt();
        ModelInfo[which].labelcolor = settings.value(LABEL_COLOUR_KEY, "#000000").value<QColor>();
        ModelInfo[which].light_diffuse[MONOCOL] = settings.value(COLOR_MONO_KEY, "#000000").value<QColor>();
        ModelInfo[which].ifLabels = settings.value(DRAW_LABELS_KEY).toBool();
        ModelInfo[which].drawLigand = settings.value(DRAW_LIG_KEY).toInt();
        ModelInfo[which].ifDrawH = settings.value(DRAW_HYD_KEY).toBool();
        ModelInfo[which].ifDrawW = settings.value(DRAW_WAT_KEY).toBool();
        //ModelInfo[which].ifSymmetry = settings.value(PACKING_KEY).toBool();
        ModelInfo[which].ifSymmetry = false;
        ModelInfo[which].klevel[0] = settings.value(PACKLEV0_KEY, 0.1f).toFloat();
        ModelInfo[which].klevel[1] = settings.value(PACKLEV1_KEY, 0.1f).toFloat();
        ModelInfo[which].klevel[2] = settings.value(PACKLEV2_KEY, 0.1f).toFloat();
        ModelInfo[which].FitOpt = settings.value(FITOPT_KEY, 1).toInt();
    }
    else   // default values
    {
        ModelInfo[which].plotStyle = STICK_AND_BALLS;
        ModelInfo[which].colorMode = BY_SPECIES;
        ModelInfo[which].ifLabels = false;
        ModelInfo[which].drawLigand = 0;
        ModelInfo[which].ifDrawH = true;
        ModelInfo[which].ifDrawW = false;
        ModelInfo[which].ifSymmetry = false;
        ModelInfo[which].klevel[0] = 0.1f;
        ModelInfo[which].klevel[1] = 0.1f;
        ModelInfo[which].klevel[2] = 0.1f;
        ModelInfo[which].FitOpt = 1;
    }
    setLabelOnOff(ModelInfo[which].ifLabels);
}

void CrystalViewWidget::saveSettings()
{
    Properties::WriteSettings();
}


void CrystalViewWidget::PassaModello (atom_type *atml, f_bond_type *legm,
                             int *nats, int *nats_au, int *nlegs, int *nlegs_au,
                             element_type *celem, int *nelem,
                             int *which,
                             int *nmodels, int *ifPdb, char *name, bool *ifNewModel,
                             int ifResize)

{
    structure_info_type StructInfo;
    int status[50], colorMode;
    float fstatus[3];
    statusSav vec;
    *ifNewModel = true;

    int k = *which - 1;
    Properties::nModels = *nmodels;

    if(k == 0)
        Properties::Normalized = 0;

    if(ModelInfo[k].created)
    {
        *ifNewModel = Properties::FromImport; //false;
        status[0]  = (ModelInfo[k].ifVis)? 1 : 0;
        status[1]  = (ModelInfo[k].ifSymmetry)? 1 : 0;
        status[2]  = (ModelInfo[k].ifDrawH)? 1 : 0;
        status[3]  = ModelInfo[k].plotStyle;
        status[4]  = ModelInfo[k].colorMode;
        status[5]  = ModelInfo[k].ifLabels?1:0;
        status[6]  = ModelInfo[k].ifChangedSymmetry;
        status[7]  = ModelInfo[k].drawLigand;
        status[9]  = ModelInfo[k].ifPoly?1:0;
        status[10] = (ModelInfo[k].ifDrawW)? 1 : 0;
        status[11] = ModelInfo[k].specialLab;
        status[12] = ModelInfo[k].labelType;
        fstatus[0] = ModelInfo[k].klevel[0];
        fstatus[1] = ModelInfo[k].klevel[1];
        fstatus[2] = ModelInfo[k].klevel[2];
        status[16] = ModelInfo[k].FitOpt;
        for(int i=0; i<NUM_COLORS; i++)
            vec.light_diffuse[i] = ModelInfo[k].light_diffuse[i];
        vec.labelcolor = ModelInfo[k].labelcolor;
        vec.light_chain = ModelInfo[k].light_chain;
        memcpy(vec.LabelS, ModelInfo[k].LabelS, N_ELEMENTS*sizeof(int));
        string oldname = ModelInfo[k].modname;
        ModelInfo[k].freeModel();
        //ModelInfo[k] = Model (status, fstatus, vec);
        ModelInfo[k] = Model();
        ModelInfo[k].SetModel (status, fstatus, vec);
        ModelInfo[k].modname = oldname;
        ModelInfo[k].ifResize = (ifResize != 0);
    }
    else
    {
        ModelInfo[k] = Model ();
        readSettings(k);
        Properties::defaultPlotStyle =  ModelInfo[k].plotStyle;
        Properties::defaultColor = ModelInfo[k].colorMode;
        Properties::defaultLabels = ModelInfo[k].ifLabels;
        colorMode = ModelInfo[k].colorMode;
        if(*nats <= NATS_LIM)
        {
            Properties::Fog = false;
            Properties::ClipFactor = 1;
        }
        else
        {
            Properties::Fog = true;
            Properties::ClipFactor = 0.5;
        }
        if(Properties::defaultFog >= 0)
            Properties::Fog = (Properties::defaultFog > 0);
        if(Properties::defaultClip >= 0)
        {
            Properties::ClipFactor = (Properties::defaultClip > 0)?0.5:1;
            //SetClip((defaultClip > 0), false);
        }
        if((Properties::defaultColor >= 0) && (Properties::defaultColor <COLOR_MODES))
            colorMode = Properties::defaultColor;
        ModelInfo[k].colorMode = colorMode;
        if(Properties::defaultLabels >= 0)
            ModelInfo[k].ifLabels = (Properties::defaultLabels != 0);
        else
            ModelInfo[k].ifLabels = false;

        if(name)
            ModelInfo[k].modname = name;
        else
        {
            char nome[80];
            getsname_(nome, 80);
            ModelInfo[k].modname = nome;
        }
        if(ModelInfo[k].modname.empty())
            ModelInfo[k].modname = "Untitled";
        ModelInfo[k].ifResize = 1;
    }
    ModelInfo[k].nModello = k;
    ModelInfo[k].created = true;
    ModelInfo[k].isPDB = (*ifPdb != 0);
    Properties::FromImport = false;
    if(*ifNewModel)
        ModelInfo[k].Surface_c = false;

    int ifCalc = 0;
    int wm = k+1;
    getstructureinfo_(&StructInfo, &ifCalc, &wm);
    for(int j=0; j<6; j++)
        Properties::Cell[j] = StructInfo.cell[j];
    for(int i1=0; i1<16; i1++)
        Properties::sy.spg[i1] = StructInfo.spaceg[i1];
    Properties::sy.spg[16] = '\0';
    Properties::sy.icent = StructInfo.icent;
    ModelInfo[k].ifCentro = (StructInfo.icent == 1);
    Properties::CalcolaMatrici();
    Properties::calcolava();
    ModelInfo[k].numSpec = *nelem;
    int *countz = new int[N_ELEMENTS]();
    std::fill(&countz[0], &countz[N_ELEMENTS-1], 0);
    ModelInfo[k].radMax = 0;
    for (int i=0; i< ModelInfo[k].numSpec; i++)
    {
        QChar lab[LEN_SPEC];
        ModelInfo[k].Species[i] = celem[i].z;
        ModelInfo[k].Content[i] = static_cast<int>(celem[i].nw);
        ModelInfo[k].ptab[i] = celem[i].ptab;
        ModelInfo[k].radii[i][0] = celem[i].c_radius;
        ModelInfo[k].radii[i][1] = celem[i].w_radius;
        ModelInfo[k].radMax = max(ModelInfo[k].radMax, celem[i].w_radius);
        if((celem[i].z > 0) && (celem[i].z <= N_ELEMENTS))
            countz[celem[i].z-1]++;
        for(int kl=0; kl<LEN_SPEC; kl++)
            lab[kl] =  celem[i].lab[kl];
        ModelInfo[k].LabSpec[i] = QString(lab, LEN_SPEC);
    }
    ModelInfo[k].radMax *= 2.f;
    if(PreviewMode)
        ModelInfo[k].radMax /= 2;

    ModelInfo[k].numT = 0;
    int Nelem = 0;
    for (int i=0; i< N_ELEMENTS; i++)
    {
        if(countz[i] > 0)
            Nelem++;
    }
    for (int i=0; i< Nelem; i++)
        ModelInfo[k].numT += (i+1);
    ModelInfo[k].numT *= 3;
    delete [] countz;
    std::vector <int> vector_spec;
    for (int kk=0; kk< Properties::nModels; kk++)
    {
        for (int i=0; i< ModelInfo[kk].numSpec; i++)
            vector_spec.push_back(ModelInfo[kk].Species[i]);
    }
    Properties::AggiornaListaSpecie(vector_spec);
    if(!vector_spec.empty())
        vector_spec.clear();
    ModelInfo[k].RF = StructInfo.r_factor;
    Properties::sy.nsymop = StructInfo.nsymmstr;

    ModelInfo[k].nats_au = *nats_au;
    ModelInfo[k].nats= *nats;
    if(ModelInfo[k].nats <= 0)
    {
        mainWindow->crystalViewMenuEnabled(false);
        updateModify(-1);
        Draw();
        if( ModelInfo[k].nats_au <= 0)      // 24032016
            return;
    }
    size_t nats1 = static_cast<size_t>(ModelInfo[k].nats);
    size_t natsau = static_cast<size_t>(ModelInfo[k].nats_au);
    ModelInfo[k].atm = new jav_atom_type [nats1]();
    ModelInfo[k].atm_au = new jav_atom_type [natsau]();
    ModelInfo[k].iEqui = new int [nats1]();
    ModelInfo[k].Secondary = new char [natsau+1]();
    ModelInfo[k].Secondary[natsau] = '\0';
    ModelInfo[k].nChains = 0;
    ModelInfo[k].Bmin = 1.e10;
    ModelInfo[k].Bmax = -1.e10;
    for(int j=0; j<3; j++)
        ModelInfo[k].baricentro[j] = 0;
    if(ModelInfo[k].nats > 0)          // 24032016
        ModelInfo[k].FillAtoms(atml, StructInfo, false);

    //NEW Get Asymmetric unit

    int whichModel = k+1;
    int natoms = ModelInfo[k].nats_au;
    atom_type *atml_au = new atom_type [static_cast<size_t>(natoms)]();
    jav_get_au_(atml_au, &natoms,&whichModel);
    ModelInfo[k].FillAtoms(atml_au, StructInfo, true);

    for(int j=0; j<3; j++)
        ModelInfo[k].baricentro[j] /= ModelInfo[k].nats;

    bool set_centre = true;
#ifdef MAPS
    if(!Properties::Normalized)
    {
        if(Properties::nMaps)
        {
            for(int j=0; j<Properties::nMaps;j++)
            {
                if(IsMapVisible(j))
                    set_centre = false;
            }
        }
    }
#endif
    if(set_centre)
    {
        for(int j=0; j<3; j++)
            Properties::xyzC.v[j] = static_cast<double>(ModelInfo[k].baricentro[j]);
    }
    ModelInfo[k].FillLegs(nlegs, nlegs_au, legm);

    ModelInfo[k].PreparaEllissoidi();
    ModelInfo[k].GetEquivs();

    if(ModelInfo[k].isPDB)
    {
        ModelInfo[k].PreparaTabelle();
        if(*ifNewModel && ModelInfo[k].IsProtein())
        {
            ModelInfo[k].plotStyle = CARTOONS;
            ModelInfo[k].colorMode = BY_STRUCT;
        }
        else if(*ifNewModel)
        {
            if(ModelInfo[k].plotStyle >= POLYPEPTIDE)
                ModelInfo[k].plotStyle = STICK_AND_BALLS;
            if(ModelInfo[k].colorMode >= BY_RESIDUE)
                ModelInfo[k].colorMode = BY_SPECIES;
        }
    }

    mainWindow->crystalViewMenuEnabled(true);
    mCentDlg->clearAll();
    UpdateInfoAtoms();
    RiNormalizza();
    SetSphereCentre(-1, -1, false);
    SetSphereRadius(-1, false);
    aggiornaMenuSym();
    aggiornaProperties(k, *ifNewModel);
    if(*ifNewModel || Properties::defaultModify)
        updateModify(Properties::defaultModify);
    if(!(Properties::useDefaultB) && (*ifNewModel) )
        mOpenGLWindow->ViewPlane('L', ModelInfo);
    else if(Properties::useDefaultB)
    {
        mOpenGLWindow->SetRotB(MRMatrixMakeFromArray(Properties::GBB));
        mOpenGLWindow->SetQuatFromRotB();
        mOpenGLWindow->SetRotmFromQuat();
        Properties::useDefaultB = false;
    }
}

void CrystalViewWidget::PassaMappa(const char *name, int nmaps, int which, int np[], int nrefl, float resm, int ifd, float denmin, float denmax, float avden, float stden)
{
    Mappa *mapinfo;
    int currentModel;
    int k = which - 1;
    Properties::nMaps = nmaps;

    int ifDifferenceMap = ifd;
    int mk = k;
    currentModel = Properties::GetActiveModel();
    if(currentModel <= 0)
        currentModel = 1;
    for(int ik=0; ik<ifDifferenceMap; ik++)
    {
        mapinfo = &(MapInfo[mk]);
        if(mapinfo->created)
        {
            mapinfo->freeMap();
            if((mapinfo->ifDifference <= 1))
                Properties::usedC[mk] = -1;
        }
        mapinfo->created = true;
        mapinfo->name = QString(name);
        mapinfo->isSurface = false;
        mapinfo->multicolor = false;

        mapinfo->ifDifference = ifd + ik;
        mapinfo->nrefl = nrefl;
        mapinfo->resm = resm;
        mapinfo->FillMap(np, nullptr, nullptr, denmin, denmax, avden, stden);
        if(mapinfo->ifDifference != 3)
        {
            Properties::activeMap = mk;
#if 0
     //aggiorna scala ed entry per sigma
         GtkWidget *entry = get_widget(GLWINDOW, "box_Sigma");
         if(entry != nullptr)
         {
            gtk_widget_set_sensitive(entry, TRUE);
            entry = get_widget(GLWINDOW, "Sigma");
            gtk_scale_set_digits(GTK_SCALE(entry), 2); //numDigits);
            gtk_range_set_range(GTK_RANGE(entry), minSigma, MRmax(0.2, mapinfo->sigmaMax));
            gtk_range_set_value(GTK_RANGE(entry), rap);
         }
         update_infos_map(iflag);
#endif
        }
        mk += MAX_NUM_MAPS;
    }
    //update_infos_cell(Properties::nMaps + Properties::nModels);
    RiNormalizza();
    Draw();
}

void CrystalViewWidget::map_set_visible(int which, int code)
{
    int whichMap = which - 1;
    if(whichMap >= Properties::nMaps)
        return;
    if(MapInfo[whichMap].created)
        MapInfo[whichMap].ifVis = (code != 0);
    Draw();
}

void CrystalViewWidget::UpdateInfoAtoms()
{
    int whichModel = IsValidModel();
    if(whichModel == -1)
        return;
    if(mInfoDialog->isVisible())
    {
        int curPage = mInfoDialog->CurrentPage();
        mInfoDialog->FillPages();
        mInfoDialog->FillInfo(curPage);
    }
}

void CrystalViewWidget::updateModify(int active)
{
    static bool stat0_sav = true, stat1_sav=true;
    stat0_sav = (menu()->actionUndo->iconText().compare("Undo") != 0);
    stat1_sav = (menu()->actionRedo->iconText().compare("Redo") != 0);
    bool act_modi = (active > 0);
    bool act_info = (active > 0);
    if(active == 3)
    {
        act_modi = false;
        act_info = true;
    }
    Properties::defaultModify = active;
    bool stat0 = m_statusUndo;
    bool stat1 = m_statusRedo;
    if(act_modi)
    {
        stat0 = stat0_sav;
        stat1 = stat1_sav;
    }
    else
    {
        stat0_sav = stat0;
        stat1_sav = stat1;
        stat0 = false;
        stat1 = false;
    }
    UpdateMenus();
    mainWindow->enableMenu(menu()->menu_Modify, act_modi);
    mainWindow->enableMenu(menu()->menuD_elete_All_Types, act_modi);
    mainWindow->enableMenu(menu()->menu_Change_Species, act_modi);
    mainWindow->enableMenu(menu()->menuAdd_At_om, act_modi);
    mainWindow->enableMenu(menu()->menuSele_ct_Type, true);
    menu()->actionSelection_Mode->setEnabled(true);
    mainWindow->enableMenu(menu()->menu_Select, act_modi);
    mainWindow->enableMenu(menu()->menuSelection_Mode, true);
    int whichModel = IsValidModel();
    if(whichModel != -1)
        mainWindow->enableMenu(menu()->menuProperties, !ModelInfo[whichModel].IsProtein());

    if(active == -99)
    {
        mainWindow->enableMenu(menu()->menu_Modify, false);
        mainWindow->enableMenu(menu()->menuD_elete_All_Types, false);
        mainWindow->enableMenu(menu()->menu_Change_Species, false);
        mainWindow->enableMenu(menu()->menuAdd_At_om, false);
        mainWindow->enableMenu(menu()->menu_Select, false);
        menu()->actionSelection_Mode->setEnabled(false);
        mainWindow->enableMenu(menu()->menuSele_ct_Type, false);
        //enableMenu(ui->menuSelection_Mode, false);
        mainWindow->enableMenu(menu()->menu_Tools, false);
        menu()->actionGeometry->setEnabled(false);
        mainWindow->enableMenu(menu()->menuImport_Fragment, false);
//TO FIX        mainWindow->enableMenu(menu()->menuRecent, false);
//TO FIX        menu()->actionImport_Structure->setEnabled(false);
        //set_mode('M', true);
        return;
    }

    mainWindow->enableMenu(menu()->menuSelection_Mode, act_modi);
    mainWindow->enableMenu(menu()->menu_Modify, act_modi);
    menu()->actionSelection_Mode->setEnabled(act_modi);
//TO FIX     menu()->actionImport_Structure->setEnabled(act_modi);
    mainWindow->enableMenu(menu()->menuImport_Fragment, act_modi);
    if(active < 0)                // se non ci sono atomi
    {
        QMenu *menuAdd = menu()->menu_Modify->findChild<QMenu *>("menuAdd_Fragment");
        menu()->menu_Modify->setEnabled(true);
        menuAdd->setEnabled(true);
        mainWindow->enableMenu(menuAdd, true);
//TO FIX        menu()->actionImport_Structure->setEnabled(true);
        mainWindow->enableMenu(menu()->menuImport_Fragment, true);
        mainWindow->enableMenu(menu()->menu_Info, false);
        mainWindow->enableMenu(menu()->menu_Tools, false);
        return;
    }
    mainWindow->enableMenu(menu()->menu_Tools, act_modi);
    mainWindow->enableMenu(menu()->menu_Info, act_info);
//TO FIX    mainWindow->enableMenu(menu()->menuRecent, act_modi);
    menu()->actionGeometry->setEnabled(act_modi);
    menu()->actionUndo->setEnabled(stat0);
    menu()->actionRedo->setEnabled(stat1);

    bool stat_hc = menu()->actionShort_Contacts->isChecked();
    stat_hc |= menu()->actionHydrogen_Bonds->isChecked();
    menu()->actionDelete_Hanging_Contacts->setEnabled(stat_hc);
    update_items_modify(act_modi);
}

void CrystalViewWidget::update_items_modify(bool status, bool updateList)
{
    if(!status)
        return;
    int nAtomsSel = 0;
    int nBondsSel = 0;
    int nResidSel = 0;
    bool ncent = false;
    int inModify = Properties::GetActiveModel() - 1;
    int wM = (inModify > -1)?inModify:0;
    nAtomsSel = (int) ModelInfo[wM].atoms_sel.size();
    nBondsSel = (int) ModelInfo[wM].bonds_sel.size();
    nResidSel = (int) ModelInfo[wM].resid_sel.size();
    ncent = !ModelInfo[wM].ifCentro;
    mainWindow->enableMenu(menu()->menuAdd_At_om, (nAtomsSel>0));
    mainWindow->enableMenu(menu()->menu_Change_Species, (nAtomsSel>0));
    menu()->menuAdd_At_om->setEnabled(nAtomsSel>0);
    menu()->menu_Change_Species->setEnabled(nAtomsSel>0);
    menu()->actionExpand_to_Molecule->setEnabled(nAtomsSel>0);
    menu()->actionDuplicate_Selected_Atoms->setEnabled(nAtomsSel>0);
    menu()->actionMove_Atoms->setEnabled(nAtomsSel>0);
    menu()->actionDelete_Selected_Atoms_or_Bonds->setEnabled((nAtomsSel+nBondsSel+nResidSel)>0);
    menu()->actionDelete_Bonds->setEnabled(nBondsSel>0);
    menu()->actionAdd_Bonds->setEnabled(nAtomsSel>0);
    menu()->actionRename->setEnabled(nAtomsSel>0);
    menu()->actionRenumber->setEnabled(nAtomsSel>0);
    menu()->actionComplete->setEnabled(nAtomsSel>0);
    menu()->actionRegularize->setEnabled(nAtomsSel>0);
    menu()->actionEnantiomorph->setEnabled(ncent);
    menu()->actionAcetylenic_C_H->setEnabled(nAtomsSel>0);
    menu()->actionMethine_C_H->setEnabled(nAtomsSel>0);
    menu()->actionMethylene_Group->setEnabled(nAtomsSel>0);
    menu()->actionMethyl_Group->setEnabled(nAtomsSel>0);
    menu()->actionAromatic_C_H->setEnabled(nAtomsSel>0);
    menu()->actionHydroxyl_O_H->setEnabled(nAtomsSel>0);
    menu()->actionTerminal_B_H->setEnabled(nAtomsSel>0);
    menu()->actionTerminal_X_CH2->setEnabled(nAtomsSel>0);
    menu()->actionSelected_Plane->setEnabled(nAtomsSel>0);
    menu()->actionOptimize_Selected_Atoms->setEnabled(nAtomsSel>0);

    if(updateList && mInfoDialog->isVisible())
        mInfoDialog->UpdateInfoSelections();
}

void CrystalViewWidget::aggiornaProperties(int whichModel, bool ifSave)
{
    if(mOptionsDialog->isVisible())
    {
        if(whichModel >= 0)
        {
            mOptionsDialog->fillOptionsModel(whichModel+1, ifSave);
            mOptionsDialog->fillPageAtoms();
            mOptionsDialog->fillPageBonds();
            mOptionsDialog->fillPagePoly();
            mOptionsDialog->fillPageDraw();
            mOptionsDialog->fillPageDisp();
            mOptionsDialog->fillPageLight();
        }
    }
}

void CrystalViewWidget::Draw()
{
    mOpenGLWindow->update();
}

void CrystalViewWidget::DrawModels()
{
    QOpenGLFunctions *context = mOpenGLWindow->GetContext();
    ColorWidget::ColorType BgType = mOpenGLWindow->GetBgType();
    context->glDepthMask (GL_TRUE);
    context->glEnable (GL_DEPTH_TEST);
    for(int i=0; i<Properties::nModels;i++)
    {
        if (ModelInfo[i].ifVis)
        {
            ModelInfo[i].DrawModel(context);
            ModelInfo[i].DrawPeaks(context);
        }
#ifdef MAPS
        if(ModelInfo[i].ifSurface && ModelInfo[i].nats_au)
        {
            if(!ModelInfo[i].Surface_c)
            {
                ModelInfo[i].CreaMappa(i+1);
                RunMCAll();
            }
            mOpenGLWindow->DisableClip();
            ModelInfo[i].DrawSurface(context);
            mOpenGLWindow->EnableClip();
        }
#endif
    }
    bool in_selection = mOpenGLWindow->InSelection();
    for(int i=0; i<Properties::nModels;i++)
    {
        if (ModelInfo[i].ifVis)
        {
            if(!Properties::Stereo)
            {
                context->glDisable(GL_FOG);
                if((ModelInfo[i].ifLabels) || (ModelInfo[i].specialLab != 0))
                    ModelInfo[i].DrawLabels(context);
                else
                    ModelInfo[i].DrawLabels(context, true);
                ModelInfo[i].DrawSelected(context);
                if(in_selection)
                    DrawSelection(context);
                if(Properties::Fog && (BgType < ColorWidget::TYPE_FLAT))
                    context->glEnable(GL_FOG);
            }
        }
    }
}

void CrystalViewWidget::DrawCentroids()
{
    if(!mCentDlg->isVisible())
        return;
    QOpenGLFunctions *context = mOpenGLWindow->GetContext();
    for(int k=0; k<Properties::nModels; k++)
        mCentDlg->DrawCentroids(k, context, ModelInfo[k].labelcolor);
}

void CrystalViewWidget::JavDrawText(vet3 pos, QString str, QColor color)
{
    mOpenGLWindow->JavDrawText(pos, str, color);
}

void CrystalViewWidget::setLabelOnOff(bool status)
{
    bool oldState = menu()->actionLabels_On_Off->blockSignals(true);
    menu()->actionLabels_On_Off->setChecked(status);
    menu()->actionLabels_On_Off->blockSignals(oldState);
}

void CrystalViewWidget::IsAtom(int x, int y)
{
    QOpenGLFunctions *context = mOpenGLWindow->GetContext();
    setStatusBarText("");
    for(int i=0;  i<Properties::nModels; i++)
    {
        if(ModelInfo[i].ifVis)
        {
            int jat = ModelInfo[i].IsAtom(context, x,y);
            if(jat < 0)
                continue;
            QString str;
            if(ModelInfo[i].plotStyle >= POLYPEPTIDE)
            {
                QString Label = ModelInfo[i].atm[jat].lab.c_str();
                str = Label.trimmed();
                //               str = get_chain(Label);
                //               str.append(get_res_name(Label));
                //               str.append(get_res_num(Label));
            }
            else
                str = infoAtom(i, static_cast<size_t>(jat));
            setStatusBarText(str);
            break;
        }
    }
}

void CrystalViewWidget::DrawForSelect(bool only_atoms)
{
    for(int i=0; i<Properties::nModels;i++)
        mOpenGLWindow->DrawForSelect(only_atoms, ModelInfo[i], i);
}

int CrystalViewWidget::selectObject(int x, int y, bool sel_flag, bool only_atoms)
{
    int result, whichModel, whichType, *p;
    size_t whichObject;

    p = mOpenGLWindow->pickObjects(x, y, only_atoms);
    result = p[0];
    whichType  = p[1];
    whichModel = p[2];
    whichObject  = static_cast<size_t>(p[3]);
    QString str;
    if(result)
    {
        switch (whichType)
        {
        case 0:             // Atoms
            if(sel_flag)
            {
                if(mOpenGLWindow->MotionType() != SELECT_DEST)
                    ModelInfo[whichModel].UpdateSelectedAtom(whichObject);
                else
                    ModelInfo[whichModel].UpdateSelectedAtom(whichObject, true);
            }
            str = infoAtom(whichModel, whichObject);
            break;
        case 1:            // Bonds
        case 2:           // Bonds in c_alpha chain
        case 3:           // Bonds in backbone
        case 4:           // Bonds in ligands
            if(sel_flag)
                ModelInfo[whichModel].UpdateSelectedBond(whichObject,whichType);
            str = infoBond(whichModel, whichObject, whichType);
            break;
        case 5:           // Residues
            if(sel_flag)
                ModelInfo[whichModel].UpdateSelectedResidue(whichObject);
            str = ModelInfo[whichModel].c_alphab[whichObject].ResLab.c_str();
            break;
        }
        update_items_modify(IsModifyEnabled());
    }
    else
        str = "";
    setStatusBarText(str);
    return result;
}

int CrystalViewWidget::selectObjects(int x, int y, double wp, double hp)
{
    int result, whichType, whichModel, whichObject, *p;

    if(wp < 1)
        wp = 1;
    if(hp < 1)
        hp = 1;
    p = mOpenGLWindow->pickObjects(x, y, false, wp, hp, true);
    result = p[0];
    if(result)
    {
        int sz = result;
        for(int i=0; i<sz; i++)
        {
            whichType    = p[3*i+1];
            whichModel   = p[3*i+2];
            whichObject  = p[3*i+3];
            switch(whichType)
            {
            case 0:
                ModelInfo[whichModel].UpdateSelectedAtom(whichObject);
                break;
            case 1:
            case 2:
            case 3:
            case 4:
                ModelInfo[whichModel].UpdateSelectedBond(whichObject,
                                                         whichType);
                break;
            case 5:
                ModelInfo[whichModel].UpdateSelectedResidue(whichObject);
                break;
            }
        }
        update_items_modify(IsModifyEnabled());
    }
    menu()->actionRectangular->setChecked(false);
    Draw();
    return result;
}

QString CrystalViewWidget::infoAtom(int i, size_t jat)
{
    QString str;
    QString label = ModelInfo[i].atm[jat].lab.c_str();
    QTextStream(&str) << label.trimmed() << " #" << jat+1;
    for(int kat=0; kat<ModelInfo[i].atm[jat].numleg; kat++)
    {
        int j2 = ModelInfo[i].atm[jat].leg[kat];
        jav_atom_type *atomo2 = &(ModelInfo[i].atm[j2]);
        label = atomo2->lab.c_str();
        if(kat)
            QTextStream(&str) << ", ";
        else
            QTextStream(&str) << " Dist: ";
        QTextStream(&str) << label.trimmed() << ": " << ModelInfo[i].atm[jat].dist[kat];
    }
    return str;
}

QString CrystalViewWidget::infoBond(int whichModel, size_t whichObject, int whichType)
{
    jav_bond_type *bond;
    if(whichType == 1)
        bond = &(ModelInfo[whichModel].bonds[whichObject]);
    else if(whichType == 2)
        bond = &(ModelInfo[whichModel].bonds_ca[whichObject]);
    else if(whichType == 3)
        bond = &(ModelInfo[whichModel].bonds_bb[whichObject]);
    else
        bond = &(ModelInfo[whichModel].bonds_lig[whichObject]);
    QString at1 = ModelInfo[whichModel].atm[bond->n1-1].lab.c_str();
    QString at2 = ModelInfo[whichModel].atm[bond->n2-1].lab.c_str();
    QString str;
    QTextStream(&str) << at1.trimmed() << "-" << at2.trimmed() << " d=" <<
        bond->dist << " #" << whichObject+1;
    return str;
}


mat4x4 CrystalViewWidget::GetRotm()
{
    return mOpenGLWindow->GetRotm();
}

bool CrystalViewWidget::jav_modify(int *kptra, int *nptra, int *kptrb, int *nptrb, int *cod, float *threshold, int wm, int *spec, const char *newname, size_t idum)
{
    int which;
    float thr;
    if(threshold)
        thr = *threshold;
    else
        thr = 0.6f;
    if(wm == -1)
        which = Properties::GetActiveModel();
    else
        which = wm + 1;
    bool result =
        (jav_modify_object_(kptra,nptra, kptrb, nptrb, cod, &thr,
                            &which, spec, newname, idum) == 0);
    if(result)
    {
        UpdateInfoAtoms();
        Draw();
    }
    return result;
}

QStringList CrystalViewWidget::FillSpecies(int kadd)
{
    int whichModel = IsValidModel();
    if(whichModel == -1)
        return QStringList();
    if(kadd && !ModelInfo[whichModel].created)
        return QStringList();
    int nSpec;
    QStringList Species;
    structure_info_type StructInfo;
    if(ModelInfo[whichModel].created)
        nSpec = ModelInfo[whichModel].numSpec;
    else
    {
        int ifCalc = 0, wm=1;
        getstructureinfo_(&StructInfo, &ifCalc, &wm);
        nSpec = StructInfo.nspec;
    }

    for (int i = 0; i < nSpec; i++)
    {
        QString spec;
        if(ModelInfo[whichModel].created)
            spec = ModelInfo[whichModel].GetSpecies(i);
        else
        {
            int which = StructInfo.zspec[i] - 1;
            if(which >= 0)
                spec = get_nth_element(which);
            else
                spec = "Q";
        }
        Species << spec.trimmed();
    }
    return Species;
}

QStringList CrystalViewWidget::FillAtoms()
{
    QStringList ListaAtomi;
    int whichModel = IsValidModel();
    if(whichModel == -1)
        return QStringList();
    int numAtoms = ModelInfo[whichModel].nats;
    for(int i=0; i<numAtoms;i++)
        ListaAtomi << QString((ModelInfo[whichModel].atm[i].lab).c_str());
    return ListaAtomi;
}

QString CrystalViewWidget::ModName(int k)
{
    if(k <= 0)
        return " ";
    return QString(ModelInfo[k-1].modname.c_str());
}

Model *CrystalViewWidget::GetModel(int k)
{
    if(k <= 0)
        return nullptr;
    return &ModelInfo[k-1];
}

void CrystalViewWidget::SetPacking(bool ifClick)
{
    //da verificare

    if(ifClick)
        menu()->actionPacking->trigger();
    else
        set_symm(true);
}

void CrystalViewWidget::SetLabels()
{
    menu()->actionLabels_On_Off->trigger();
}

void CrystalViewWidget::ChangeOrtho()
{
    mOpenGLWindow->ChangeOrtho();
}

void CrystalViewWidget::ResetLights()
{
    mOpenGLWindow->ResetLights();
}

void CrystalViewWidget::SetCell(bool ifClick)
{
    if(ifClick)
        menu()->actionView_Cell_Edges->trigger();
    else
        menu()->actionView_Cell_Edges->setChecked(Properties::ifCell);
}

void CrystalViewWidget::SyncToggles()
{
    menu()->actionView_Cell_Edges->blockSignals(true);
    menu()->actionView_Cell_Edges->setChecked(Properties::ifCell);
    menu()->actionView_Cell_Edges->blockSignals(false);
}

void CrystalViewWidget::updateCenter()
{
//se aperto options dialog, aggiorna il centro
}

bool CrystalViewWidget::IsMapVisible(int whichMap)
{
    if(whichMap >= Properties::nMaps)
        return false;
    if(MapInfo[whichMap].created && MapInfo[whichMap].ifVis)
        return true;
    return false;
}

void CrystalViewWidget::RunMCAll()
{
    Properties::RunMCAll(ModelInfo, MapInfo);
}

void CrystalViewWidget::DrawMaps()
{
    QOpenGLFunctions *context = mOpenGLWindow->GetContext();
    for(int i=0; i<Properties::nMaps;i++)
    {
        if (MapInfo[i].ifVis)
        {
            MapInfo[i].DrawMap(context);
            if(MapInfo[i].ifDifference > 1)      // DifferenceMap
                MapInfo[i+MAX_NUM_MAPS].DrawMap(context);
        }
    }
}

void CrystalViewWidget::SelectAtom(int x, int y)
{
    int model, atom;
    setMotionType(NO_MOVE);
    int result, *p;
    QString results;
    QString str;
    p = mOpenGLWindow->pickObjects(x, y, true);
    result = p[0];
    model = p[2];
    atom  = p[3];

    if(result)
    {
        move_center(ModelInfo[model].atm[atom].xyzc, model, atom, false);
        str = infoAtom(model, atom);
        setStatusBarText(str);
    }
}

void CrystalViewWidget::setUnselect()
{
    //mainWindow->getActionSelect_None()->trigger();
    menu()->actionSelect_None->trigger();
}

void CrystalViewWidget::addResult(int p[][2], float result_op, int toclick,
                          QString &strRes)
{
    QStringList str = {"Distance ",
                       "Angle ",
                       "Torsion Angle "};
    QStringList mu = {Aring(), Degree(), Degree()};
    std::stringstream strResults;

    strResults << QString(str.at(toclick-2)).toStdString() << "(" ;
    for(int i=0; i<toclick; i++)
    {
        QString label = ModelInfo[p[i][0]].atm[p[i][1]].lab.c_str();
        strResults << label.trimmed().toStdString();
        if(i != (toclick-1))
            strResults << ", ";
    }
    strResults << ") = ";
    strResults << result_op << " " << QString(mu.at(toclick-2)).toStdString();
    strRes = strResults.str().c_str();
    if(mGeoCalcDlg->isVisible())
        mGeoCalcDlg->setRes(QString(strResults.str().c_str()));
}

void CrystalViewWidget::Set_Hint(QString mesg)
{
    if(mGeoCalcDlg->isVisible())
        mGeoCalcDlg->setHint(mesg);
}

void CrystalViewWidget::setMotionType(int whichCalc)
{
    if(whichCalc == NO_MOVE)
    {
        mOpenGLWindow->SetMotionType(NO_MOVE, 0);
        return;
    }
    else if (whichCalc == SELECT_CENTRE)
    {
        mOpenGLWindow->SetMotionType(whichCalc, 1);
        setStatusBarText(QString("Click on one atom"));
        return;
    }
    mOpenGLWindow->SetMotionType(whichCalc, 3);
    if(mGeoCalcDlg->isVisible())
        mGeoCalcDlg->setHint(QString("Click on %1 atoms").arg(whichCalc-GEOM));
}

bool CrystalViewWidget::SelectAtoms(int x, int y)
{
    static int punti[4][2];
    int motion_type = mOpenGLWindow->GetMotionType();
    int nclicked = mOpenGLWindow->GetNClicked();
    int toclick = motion_type - GEOM;
    int result, *p;
    QString results;
    p = mOpenGLWindow->pickObjects(x, y, true);
    result = p[0];
    if(result)
    {
        punti[nclicked][0] = p[2];        // whichModel
        punti[nclicked][1] = p[3];        // whichAtom
        ModelInfo[p[2]].UpdateSelectedAtom(p[3], false);
        nclicked = (int) ModelInfo[p[2]].atoms_sel.size(); //++;
        ModelInfo[p[2]].atm[p[3]].drawLabel = 1;
        Draw();
        if(toclick == 1)                  //sphere
        {
            for(int j=0; j<ModelInfo[p[2]].nats; j++)
                if(ModelInfo[p[2]].atm[j].drawLabel < 2)
                    ModelInfo[p[2]].atm[j].drawLabel = 0;
            ModelInfo[p[2]].atm[p[3]].drawLabel = 1;
            nclicked = 0;
            SetSphereCentre(p[2], p[3]);
            if(mSphereDlg->isVisible())
                mSphereDlg->setModel(p[2]+1);
        }
        else if(nclicked == toclick)           // end
        {
            float **Atomi = new float * [toclick]();
            int  kmodel = Properties::GetActiveModel() - 1;
            for(int k=0; k<toclick; k++)
            {
                Atomi[k] = new float [3]();
                punti[k][1] = ModelInfo[kmodel].atoms_sel[k]-1;
                for(int kj=0;kj<3;kj++)
                    Atomi[k][kj] = ModelInfo[kmodel].atm[punti[k][1]].xyz[kj];
            }
            float result_op = calcDistAng(Atomi, toclick);
            DistAng_type v;
            switch(toclick)
            {
            case 2:
            case 3:
            case 4:
                v.op = toclick;
                for(int k=0; k<toclick; k++)
                {
                    v.wM[k] = punti[k][0];
                    v.wA[k] = punti[k][1];
                    //azzera le labels degli atomi per le misure
                    if(ModelInfo[v.wM[k]].atm[v.wA[k]].drawLabel < 2)
                        ModelInfo[v.wM[k]].atm[v.wA[k]].drawLabel = 0;
                }
                v.result = result_op;
                Properties::DistAng_vet.push_back(v);
                break;
            default:
                break;
            }
            //gdk_window_set_cursor(gtk_widget_get_window(widget),NULL);
            nclicked = 0;
            Set_Hint(" ");
            addResult(punti, result_op, toclick, results);
            setUnselect();
            setStatusBarText(results);
        }
        else
        {
            //azzera le labels degli atomi per le misure
            if((motion_type > GEOM) && (motion_type <= (GEOM+4))
                && (ModelInfo[p[2]].atm[p[3]].drawLabel < 2))
                ModelInfo[p[2]].atm[p[3]].drawLabel = 0;

            if(nclicked == 0)
                Set_Hint("Click on first atom.");
            if(nclicked == 1)
                Set_Hint("Click on second atom.");
            else if(nclicked == 2)
                Set_Hint("Click on third atom.");
            else if(nclicked == 3)
                Set_Hint("Click on fourth atom.");
            return false;
        }
    }
    else
        return false;
    return true;
}

void CrystalViewWidget::RiNormalizza(bool fResize)
{
    Properties::RiNormalizza(ModelInfo, MapInfo, fResize);
    if(mCentDlg->isVisible())
    {
        for(int k=0; k<Properties::nModels; k++)
            mCentDlg->NormalizzaCentroids(k);
    }
}

void CrystalViewWidget::do_actionExpand_to_Molecule()
{
    menu()->actionExpand_to_Molecule->trigger();
}

void CrystalViewWidget::do_move_atoms()
{
    int whichModel = IsValidModel();
    if(whichModel == -1)
        return;
    int nptra = (int) ModelInfo[whichModel].atoms_sel.size();
    int *kptra = new int [nptra]();
    for(int i=0; i<nptra; i++)
        kptra[i] = ModelInfo[whichModel].atoms_sel[i];
    int nptrb = 0;
    int icod = 24;
    if(jav_modify(kptra,&nptra,NULL,&nptrb,&icod))
        endModify(whichModel);
}

bool CrystalViewWidget::doPopup(int p[], QMenu *contextMenu)
{
    int whichModel, whichObject, whichType;
    bool result = true;
    QMenu* subMenu;
    QAction *newAct;

    whichType  = p[1];
    whichModel = p[2];
    whichObject  = p[3];
    m_whichObject = whichObject*10 + whichModel;
    switch (whichType)
    {
    case 0:             // Atoms
        if(!ModelInfo[whichModel].atm[whichObject].selected)
            return false;
        contextMenu->addAction(menu()->actionDelete_Selected_Atoms_or_Bonds);
        contextMenu->addMenu(menu()->menu_Change_Species);
        contextMenu->addAction(menu()->actionRename);
        // contextMenu->addAction(tr("Delete Selected atoms or bonds"),this,
        //                 SLOT(on_actionDelete_Selected_Atoms_or_Bonds_triggered()));
        // subMenu = contextMenu->addMenu(tr("Change Species"));
        // for (int i = 0; i < Properties::NSpecies; i++)
        // {
        //     int z = Properties::ListaSpecies[i]-1;
        //     QString item_lab = get_nth_element_name(z);
        //     newAct = subMenu->addAction(item_lab);
        //     connect(newAct, &QAction::triggered, this,
        //             std::bind(&CrystalViewWidget::onChangeSpecie, this, z+1));
        // }
        //newAct = subMenu->addAction("Other");
        // connect(newAct, &QAction::triggered, this,
        //         std::bind(&CrystalViewWidget::onChangeSpecie, this, OTHERS+1));
        // contextMenu->addAction(tr("Rename"),this,
        //                 SLOT(on_actionRename_triggered()));
        newAct = contextMenu->addAction(tr("Expand Contacts from this Atom"));
        connect(newAct, &QAction::triggered, this,
                std::bind(&CrystalViewWidget::onContactAtom, this, m_whichObject));
        newAct = contextMenu->addAction(tr("Expand Contacts from this Molecule"));
        connect(newAct, &QAction::triggered, this,
                std::bind(&CrystalViewWidget::onContactMol, this, m_whichObject));
        break;
    case 1:            // Bonds
        if(!ModelInfo[whichModel].bonds[whichObject].selected)
            return false;
    case 2:           // Bonds in c_alpha chain
    case 3:           // Bonds in backbone
    case 4:           // Bonds in ligands
        contextMenu->addAction(menu()->actionDelete_Selected_Atoms_or_Bonds);
        contextMenu->addAction(menu()->actionDelete_Bonds);
        // contextMenu->addAction(tr("Delete Selected atoms or bonds"),this,
        //                 SLOT(on_actionDelete_Selected_Atoms_or_Bonds_triggered()));
        // contextMenu->addAction(tr("Delete Selected bonds"),this,
        //                 SLOT(on_actionDelete_Bonds_triggered()));
        break;
    case 5:           // Residues
        if(!ModelInfo[whichModel].c_alphab[whichObject].selected)
            return false;
        newAct = contextMenu->addAction(tr("Delete selected residue"));
        connect(newAct, &QAction::triggered, this,
                std::bind(&CrystalViewWidget::on_DeleteResidues, this, m_whichObject));
        newAct = contextMenu->addAction(tr("Delete Side Chain"));
        connect(newAct, &QAction::triggered, this,
                std::bind(&CrystalViewWidget::on_DeleteSideChain, this, m_whichObject));
        newAct = contextMenu->addAction(tr("Delete aromatic ring"));
        connect(newAct, &QAction::triggered, this,
                std::bind(&CrystalViewWidget::on_DeleteAromatic, this, m_whichObject));
        break;
    }
    return result;
}

void CrystalViewWidget::updateSphereList(int kfound, int vat[], float vdist[], op_type vsym[])
{
    if(mSphereDlg->isVisible())
    {
        int whichModel = mSphereDlg->getModel() - 1;
        QList<QStandardItem *> *listSphere =
            (&ModelInfo[whichModel])->FillListSphere(kfound, vat, vdist, vsym);
        mSphereDlg->fillList(listSphere, kfound);
    }
}

void CrystalViewWidget::updateMenuDeleteModel()
{
    menu()->menu_Delete_Model->clear();
    if(Properties::nModels <= 1)
        return;

    for (int i=1; i < Properties::nModels; i++)
    {
        if(!ModelInfo[i].created)
            continue;
        QString item_lab = ModelInfo[i].modname.c_str();
        QAction *newAct = menu()->menu_Delete_Model->addAction(item_lab);
        connect(newAct, &QAction::triggered, this,
                std::bind(&CrystalViewWidget::onDeleteModel, this, i+1));
        newAct->setCheckable(false);
    }
}

void CrystalViewWidget::FillCentroidsDlg()
{
    mCentDlg->FillList();
    Draw();
}

void CrystalViewWidget::ClearCentroids(int whichModel)
{
    mCentDlg->clearCentroids(whichModel);
}

void CrystalViewWidget::AddCentroids(float xyz[], int natoms, int at[])
{
    int whichModel = Properties::GetActiveModel() - 1;
    if(whichModel == -1)
        return;
    jav_centroid_type c;
    float col[] = {0.8f, 1, 0.8f, 1};
    for (int i=0; i<3; i++)
        c.xyz[i] = xyz[i];
    c.nat = natoms;
    c.atoms = new int [c.nat]();
    for (int i=0; i<c.nat; i++)
        c.atoms[i] = at[i] - 1;
    c.visible = true;
    c.showlab = true;
    c.transparency = 50;
    c.selected = 0;
    //memcpy(c.colore, col, 4*sizeof(float));
    c.colore = ToQColor(col, 4);
    c.label = QString(ModelInfo[whichModel].atm[c.atoms[0]].lab.c_str()).trimmed();
    for (int i=1; i<c.nat; i++)
    {
        c.label.append(",");
        c.label.append(QString(ModelInfo[whichModel].atm[c.atoms[i]].lab.c_str()).trimmed());
    }
    mCentDlg->addCentroid(whichModel, c);
}

void CrystalViewWidget::setStatusBarText(const QString &Msg)
{
    ui->statusLabel->setText(fontMetrics().elidedText(Msg,Qt::ElideRight,ui->statusLabel->size().width()));
}

void CrystalViewWidget::clearStatusBarText()
{
    ui->statusLabel->clear();
}

void CrystalViewWidget::waitProcess(const QString &Msg)
{
    Attesa(Msg,ui->statusLabel);
}

void CrystalViewWidget::endProcess()
{
    FineAttesa(ui->statusLabel);
}

double CrystalViewWidget::getStep()
{
    return menu()->stepTra->text().toDouble();
}

double CrystalViewWidget::getRotStep()
{
    return menu()->stepRot->text().toDouble();
}

void CrystalViewWidget::selectAtoms(const QVector<int> &selected, int whichModel)
{
    auto md = GetModel(whichModel);
    if (md == nullptr) return;

    for (int i = 0; i < md->nats; i++)
        md->atm[i].selected = false;
    md->atoms_sel.clear();
    for (auto at : selected) {
        //qDebug() << "atom: " << at;
        md->atm[at].selected = true;
        md->atoms_sel.push_back(at+1);
    }

    Draw();
}

void CrystalViewWidget::selectAtomsAndLegs(const QVector<int> &selected, int whichModel)
{
    auto md = GetModel(whichModel);
    if (md == nullptr) return;

    for (int i = 0; i < md->nats; i++)
        md->atm[i].selected = false;
    for(int i=0; i<md->nconn_prot; i++)
        md->bonds[i].selected = false;

    md->atoms_sel.clear();
    md->bonds_sel.clear();

    for(int i=0; i<md->nconn_prot; i++) {
        auto it = std::find(selected.begin(), selected.end(), md->bonds[i].n1-1);
        auto it1 = std::find(selected.begin(), selected.end(), md->bonds[i].n2-1);
        if((it != selected.end()) && (it1 != selected.end())) {
            md->bonds[i].selected = true;
            md->bonds_sel.push_back(i+1);
        }
    }

    for (auto at : selected) {
        md->atm[at].selected = true;
        md->atoms_sel.push_back(at+1);
    }

    Draw();
}

int CrystalViewWidget::GetNz(int atomic_num)
{
    int which = Properties::GetActiveModel();
    auto md = GetModel(which);
    for (int i=0; i<md->numSpec; i++)
    {
        if(atomic_num == md->Species[i])
            return (i+1)*nzc;
    }
    return 0;
}

void CrystalViewWidget::CApplicaSimmetria(int *which)
{
#ifdef MAPS
    if(ModelInfo[*which-1].ifSymmetry)
        ModelInfo[*which-1].SettaSurface(false);
#endif
    SetActive(*which-1);
    applicasimmetria_(which);
}

void CrystalViewWidget::move_center(vet3 posi, int whichModel, int whichAtom, bool ifDrawCenter)
{
    float pos1[3] = {(float) posi.x, (float) posi.y, (float) posi.z};
    move_center(pos1, whichModel, whichAtom, ifDrawCenter);
}

void CrystalViewWidget::move_center(float posi[], int whichModel, int whichAtom, bool ifDrawCenter)
{
    float xyz1[3] = {(float) Properties::xyzC.x, (float) Properties::xyzC.y, (float) Properties::xyzC.z};
    float xyz2[3] = {posi[0], posi[1], posi[2]};
    float l[3];
    for(int j=0; j<3; j++)
        l[j] = xyz2[j]-xyz1[j];
    int nstep = 20;
    float step = 0.05f;
    float t = 0.f;
    for(int k=0; k<nstep; k++)
    {
        float xyz3[3];
        t += step;
        for(int j=0; j<3; j++)
            xyz3[j] = xyz1[j] + l[j] * t;
        for(int j=0; j<3; j++)
            Properties::xyzC.v[j] = (double) xyz3[j];
        if(Properties::ifDrawPar)
            Properties::CalcPar(Properties::xyzC, Properties::abcPar);

        RiNormalizza();
        Properties::ifAxes = 0;
        Draw();
    }
    for(int j=0; j<3; j++)
        Properties::xyzC.v[j] = posi[j];

    if(Properties::ifDrawPar)
        Properties::CalcPar(Properties::xyzC, Properties::abcPar);
    RiNormalizza();
    Properties::DrawCenter = ifDrawCenter;
    if((whichModel > -1) && (whichAtom > -1) )
    {
        for(int i=0; i<ModelInfo[whichModel].nats; i++)
            ModelInfo[whichModel].atm[i].drawLabel = 0;
        ModelInfo[whichModel].atm[whichAtom].drawLabel = 1;
    }

    Properties::ifAxes = 1;
    Draw();
#ifdef MAPS
    if(mOptionsDialog->isVisible())
        mOptionsDialog->fillPageMaps();
#endif
}

int CrystalViewWidget::importFileCOD(const QString &fileName, int tipo)
{
    const char *cStr;
    QByteArray inBytes = fileName.toUtf8();
    cStr = inBytes.constData();
    size_t sz = strlen(cStr);

    int which = 1;
    int ier = 0;
    Properties::DistAng_vet.clear();
    jav_import_structure_cod_ (cStr, &tipo, &which, &ier, sz);

    if(ier == 0)
        UpdateInfoAtoms();
    return ier;
}

void CrystalViewWidget::onActionViewCellEdges(bool state)
{
    Properties::ifCell = state;
    RiNormalizza(true);
    aggiornaProperties();
    Draw();
}

void CrystalViewWidget::onActionGrowFragmentsTriggered()
{
    QAction* act = qobject_cast<QAction*>(sender());
    if(act == nullptr)
        return;
    bool ifgrow = act->isChecked();
    Properties::ifGrow = ifgrow;
    int result = 0;
    int current_wm = IsValidModel();
    if(current_wm < 0)
        return;

    for(int k=0; k<Properties::nModels; k++)
    {
        if(Properties::nModels > 1)
            SetActive(k);
        if(ModelInfo[k].nats <= 0)
            continue;
        int wm = k + 1;
        result += jav_grow_fragment_(&wm);
    }
    if(!result)
    {
        act->setChecked(false);
        Properties::ifGrow = false;
    }
    SetActive(current_wm);
}

void CrystalViewWidget::onActionAssembleTriggered()
{
    int nptra = 0;
    int nptrb = 0;
    int icod = 5;
    jav_modify(nullptr,&nptra,nullptr,&nptrb,&icod);
}

void CrystalViewWidget::onActionMoveAtomsTriggered()
{
    setStatusBarText("Select atom to minimize distances");
    updateModify(-99);
    mOpenGLWindow->SetMotionType(SELECT_DEST, 1);
}

void CrystalViewWidget::onActionZoomInTriggered()
{
    mOpenGLWindow->on_move(2, 40.f/mOpenGLWindow->width());
}

void CrystalViewWidget::onActionZoomOutTriggered()
{
    mOpenGLWindow->on_move(2, -40.f/mOpenGLWindow->width());
}

void CrystalViewWidget::onActionPacking(bool state)
{
    set_symm(state);
}

void CrystalViewWidget::onActionExpandTriggered()
{
    QAction* act = qobject_cast<QAction*>(sender());
    if(act == nullptr)
        return;
    int whichModel = IsValidModel();
    if(whichModel == -1)
        return;
    float symlevel = Minimo(ModelInfo[whichModel].klevel, 3);
    if(symlevel >= MAX_SYMLEVEL)
        return;
    changePacking(0.1f);
    symlevel = Minimo(ModelInfo[whichModel].klevel, 3);
    act->setEnabled((symlevel < MAX_SYMLEVEL));
}

void CrystalViewWidget::onActionReduceTriggered()
{
    QAction* act = qobject_cast<QAction*>(sender());
    if(act == nullptr)
        return;
    int whichModel = IsValidModel();
    if(whichModel == -1)
        return;
    float symlevel = Massimo(ModelInfo[whichModel].klevel, 3);
    if(symlevel <= 0)
        return;
    changePacking(-0.1f);
    symlevel = Massimo(ModelInfo[whichModel].klevel, 3);
    act->setEnabled((symlevel > 0));
}

void CrystalViewWidget::changePacking(float quanto)
{
    int whichModel = IsValidModel();
    if(whichModel == -1)
        return;
    if(ModelInfo[whichModel].ifSymmetry)
    {
        for(int km=0; km<3; km++)
        {
            ModelInfo[whichModel].klevel[km] += quanto;
            if(ModelInfo[whichModel].klevel[km] > MAX_SYMLEVEL)
                ModelInfo[whichModel].klevel[km] = MAX_SYMLEVEL;
            if(ModelInfo[whichModel].klevel[km] < EPS)
                ModelInfo[whichModel].klevel[km] = 0;
        }
        int j = whichModel + 1;
        this->setCursor(Qt::WaitCursor);
        CApplicaSimmetria(&j);
        this->unsetCursor();
        Draw();
    }
}

void CrystalViewWidget::onActionResetScaleTriggered()
{
    mOpenGLWindow->SetRange(1.f);
    for(int i=0; i<Properties::nModels; i++)
        ModelInfo[i].ifResize = true;
    mOpenGLWindow->ResetView();
}

void CrystalViewWidget::onActionResetViewTriggered()
{
    this->onActionResetScaleTriggered();
    for(int j=0; j<3; j++)
    {
        if(Properties::nModels)
            Properties::xyzC.v[j] = static_cast<double>(ModelInfo[0].baricentro[j]);
        else
#ifdef MAPS
            Properties::xyzC.v[j] = Properties::centroMappa[j];
#else
            Properties::xyzC.v[j] = 0;
#endif
    }
    for(int i=0; i<Properties::nModels; i++)
        ModelInfo[i].ifResize = true;
    RiNormalizza();

    mOpenGLWindow->ResetView(true, false);
    mOpenGLWindow->ViewPlane('L', ModelInfo);
}

void CrystalViewWidget::onActionAroundXTriggered()
{
    mOpenGLWindow->on_rotate(0, 1.);
}

void CrystalViewWidget::onActionAroundX2Triggered()
{
    mOpenGLWindow->on_rotate(0, -1.);
}

void CrystalViewWidget::onActionAroundYTriggered()
{
    mOpenGLWindow->on_rotate(1, 1.);
}

void CrystalViewWidget::onActionAroundY2Triggered()
{
    mOpenGLWindow->on_rotate(1, -1.);
}

void CrystalViewWidget::onActionAroundZTriggered()
{
    mOpenGLWindow->on_rotate(2, 1.);
}

void CrystalViewWidget::onActionAroundZ2Triggered()
{
    mOpenGLWindow->on_rotate(2, -1.);
}

void CrystalViewWidget::onActionStepTriggered()
{
    bool ok;
    mOpenGLWindow->SetRotStep(menu()->stepRot->text().toDouble());
    double d = QInputDialog::getDouble(this, tr("Set rotation step"),
                                       tr("Rotation step (°):"), mOpenGLWindow->RotStep(), -180, 180, 3, &ok);
    if (ok)
    {
        mOpenGLWindow->SetRotStep(d);
        menu()->stepRot->setText(QString::number(d, 'f', menu()->stepDecimals));
    }
}

void CrystalViewWidget::onActionAllAtomsTriggered()
{
    bool stato = menu()->actionAll_Atoms->isChecked();
    GesLabels(stato, false);
}

void CrystalViewWidget::onActionDeleteAllLabelsTriggered()
{
    GesLabels(false, true);
}

void CrystalViewWidget::onActionexceptHydrogensTriggered()
{
    int whichModel = IsValidModel();
    if(whichModel == -1)
        return;
    bool stato = menu()->actionexcept_Hydrogens->isChecked();
    if(stato)
    {
        menu()->actionAll_Atoms->setChecked(false);
        menu()->actionHeavy_atoms->setChecked(false);
        //ui->actionselected_atoms->setChecked(false);
        GesLabelsSpec();
    }
    ModelInfo[whichModel].specialLab = (stato)?NONH:0;
    Draw();
}

void CrystalViewWidget::onActionHeavyAtomsTriggered()
{
    int whichModel = IsValidModel();
    if(whichModel == -1)
        return;
    bool stato = menu()->actionHeavy_atoms->isChecked();
    if(stato)
    {
        menu()->actionAll_Atoms->setChecked(false);
        menu()->actionexcept_Hydrogens->setChecked(false);
        //ui->actionselected_atoms->setChecked(false);
        GesLabelsSpec();
    }
    ModelInfo[whichModel].specialLab = (stato)?HEAVY:0;
    Draw();
}

void CrystalViewWidget::onActionselectedAtomsTriggered()
{
    int whichModel = IsValidModel();
    if(whichModel == -1)
        return;
    bool stato = true; //ui->actionselected_atoms->isChecked();
    if(stato)
    {
        menu()->actionAll_Atoms->setChecked(false);
        menu()->actionexcept_Hydrogens->setChecked(false);
        menu()->actionHeavy_atoms->setChecked(false);
        GesLabelsSpec();
    }
    ModelInfo[whichModel].SetLabelsSelected(true);

    //ModelInfo[whichModel].specialLab = (stato)?SELECTED:0;
    Draw();
}

void CrystalViewWidget::onActionAtomNameTriggered()
{
    if (!menu()->actionAtom_Name->isChecked())
        return;
    int whichModel = IsValidModel();
    if(whichModel == -1)
        return;
    ModelInfo[whichModel].labelType = LABEL_ATOM;
    set_label_status();
    Draw();
}

void CrystalViewWidget::onActionAtomSpecieTriggered()
{
    if (!menu()->actionAtom_Specie->isChecked())
        return;
    int whichModel = IsValidModel();
    if(whichModel == -1)
        return;
    ModelInfo[whichModel].labelType = LABEL_SPECIES;
    set_label_status();
    Draw();
}

void CrystalViewWidget::onActionAtomNumberTriggered()
{
    if (!menu()->actionAtom_Number->isChecked())
        return;
    int whichModel = IsValidModel();
    if(whichModel == -1)
        return;
    ModelInfo[whichModel].labelType = LABEL_NUMBER;
    set_label_status();
    Draw();
}

void CrystalViewWidget::onActionShowAllTriggered()
{
    QAction* act = qobject_cast<QAction*>(sender());
    if(act == nullptr)
        return;
    if(!act->isChecked())
        return;
    int whichModel = IsValidModel();
    if(whichModel == -1)
        return;
    ModelInfo[whichModel].ifDrawH = true;
    for(int i=0; i<ModelInfo[whichModel].nats; i++)
        ModelInfo[whichModel].atm[i].hidden = false;
    Draw();
}

void CrystalViewWidget::onActionHideSelectedTriggered()
{
    QAction* act = qobject_cast<QAction*>(sender());
    if(act == nullptr)
        return;
    if(!act->isChecked())
        return;
    int whichModel = IsValidModel();
    if(whichModel == -1)
        return;
    ModelInfo[whichModel].ifDrawH = true;
    for(int i=0; i<ModelInfo[whichModel].nats; i++)
        ModelInfo[whichModel].atm[i].hidden = false;
    size_t natoms = ModelInfo[whichModel].atoms_sel.size();
    for(size_t i=0; i<natoms; i++)
    {
        int k = ModelInfo[whichModel].atoms_sel.at(i)-1;
        ModelInfo[whichModel].atm[k].hidden = true;
    }
    Draw();
}

void CrystalViewWidget::onActionHideUnselectedTriggered()
{
    QAction* act = qobject_cast<QAction*>(sender());
    if(act == nullptr)
        return;
    if(!act->isChecked())
        return;
    int whichModel = IsValidModel();
    if(whichModel == -1)
        return;
    ModelInfo[whichModel].ifDrawH = true;
    for(int i=0; i<ModelInfo[whichModel].nats; i++)
        ModelInfo[whichModel].atm[i].hidden = true;
    size_t natoms = ModelInfo[whichModel].atoms_sel.size();
    for(size_t i=0; i<natoms; i++)
    {
        int k = ModelInfo[whichModel].atoms_sel.at(i)-1;
        ModelInfo[whichModel].atm[k].hidden = false;
    }
    Draw();
}

void CrystalViewWidget::onActionHideHydrogensTriggered()
{
    QAction* act = qobject_cast<QAction*>(sender());
    if(act == nullptr)
        return;
    if(!act->isChecked())
        return;
    int whichModel = IsValidModel();
    if(whichModel == -1)
        return;
    ModelInfo[whichModel].ifDrawH = false;
    aggiornaProperties(whichModel);
    Draw();
}

void CrystalViewWidget::onActionaTriggered()
{
    mOpenGLWindow->ViewPlane('a', ModelInfo);
}

void CrystalViewWidget::onActionbTriggered()
{
    mOpenGLWindow->ViewPlane('b', ModelInfo);
}

void CrystalViewWidget::onActioncTriggered()
{
    mOpenGLWindow->ViewPlane('c', ModelInfo);
}

void CrystalViewWidget::onActiona2Triggered()
{
    mOpenGLWindow->ViewPlane('A', ModelInfo);
}

void CrystalViewWidget::onActionb2Triggered()
{
    mOpenGLWindow->ViewPlane('B', ModelInfo);
}

void CrystalViewWidget::onActionc2Triggered()
{
    mOpenGLWindow->ViewPlane('C', ModelInfo);
}

void CrystalViewWidget::onActionLeastSquarePlaneTriggered()
{
    mOpenGLWindow->ViewPlane('L', ModelInfo);
}

void CrystalViewWidget::onActionSelectedPlaneTriggered()
{
    mOpenGLWindow->ViewPlane('S', ModelInfo);
    int wm = IsValidModel();
    if(wm != -1)
        endModify(wm);
}

void CrystalViewWidget::onActionPolyhedraTriggered()
{
    QAction* act = qobject_cast<QAction*>(sender());
    if(act == nullptr)
        return;
    int result = 0;

    int whichModel = IsValidModel() + 1;
    if(whichModel == 0)
        return;
    if(ModelInfo[whichModel-1].nats > 0)
    {
        result++;
        ModelInfo[whichModel-1].ifPoly = act->isChecked();
        jav_polyhedra_(&whichModel);
    }
}

void CrystalViewWidget::onActionPolyhedraPropertiesTriggered()
{
    if(IsValidModel() == -1)
        return;
    mOptionsDialog->setOptions();
    mOptionsDialog->show();
    mOptionsDialog->SetPage(PAGE_POLY);
}

void CrystalViewWidget::onActionShowMultipleBondsTriggered()
{
    QAction* act = qobject_cast<QAction*>(sender());
    if(act == nullptr)
        return;
    Properties::ShowMultBonds = act->isChecked();
    if(Properties::ShowMultBonds)
    {
        for(int i=1; i<=Properties::nModels; i++)
            jav_show_multbonds_(&i);
    }
    else
        Draw();
    int whichModel = IsValidModel();
    if(whichModel >= 0)
        aggiornaProperties(whichModel);
}

void CrystalViewWidget::onActionHydrogenBondsTriggered()
{
    QAction* act = qobject_cast<QAction*>(sender());
    if(act == nullptr)
        return;

    bool ifhbonds = act->isChecked();
    int result = 0;
    int current_wm = Properties::GetActiveModel() - 1;
    Properties::ifHBonds = ifhbonds;
    for(int k=0; k<Properties::nModels; k++)
    {
        if(Properties::nModels > 1)
            SetActive(k);
        if(ModelInfo[k].nats <= 0)
            continue;
        int wm = k + 1;
        result += jav_hydrogen_bonds_(&wm);
    }
    if(!result)
    {
        act->setChecked(false);
        Properties::ifHBonds = false;
    }
    menu()->actionDelete_Hanging_Contacts->setEnabled(Properties::ifHBonds || Properties::ifShortc);
    if(!Properties::ifHBonds)
    {
        menu()->actionDelete_Hanging_Contacts->setEnabled(false);
        Properties::ifDelHang = menu()->actionDelete_Hanging_Contacts->isChecked();
    }
    SetActive(current_wm);
}

void CrystalViewWidget::onActionShortContactsTriggered()
{
    QAction* act = qobject_cast<QAction*>(sender());
    if(act == nullptr)
        return;

    bool ifshcont = act->isChecked();
    int result = 0;
    int current_wm = Properties::GetActiveModel() - 1;
    Properties::ifShortc = ifshcont;
    for(int k=0; k<Properties::nModels; k++)
    {
        if(Properties::nModels > 1)
            SetActive(k);
        if(ModelInfo[k].nats <= 0)
            continue;
        int wm = k + 1;
        result += jav_short_contacts_(&wm);
    }
    if(!result)
    {
        act->setChecked(false);
        Properties::ifShortc = false;
    }
    menu()->actionDelete_Hanging_Contacts->setEnabled( Properties::ifHBonds || Properties::ifShortc);
    if(!Properties::ifShortc)
    {
        menu()->actionDelete_Hanging_Contacts->setChecked(false);
        Properties::ifDelHang = menu()->actionDelete_Hanging_Contacts->isChecked();
    }
    SetActive(current_wm);
}

void CrystalViewWidget::onActionDeleteHangingContactsTriggered()
{
    QAction* act = qobject_cast<QAction*>(sender());
    if(act == nullptr)
        return;

    bool ifdelhang = false;
    ifdelhang = act->isChecked();
    Properties::ifDelHang = ifdelhang;
    int current_wm = Properties::GetActiveModel() - 1;

    for (int i=0;i<ModelInfo[current_wm].nlegsH; i++)
    {
        // hang atom?
        int n2 = abs(ModelInfo[current_wm].legsH[i].n2);
        if((Properties::ifDelHang)  && (ModelInfo[current_wm].legsH[i].ordleg == 1))
            ModelInfo[current_wm].atm[n2-1].hidden = true;
        else
            ModelInfo[current_wm].atm[n2-1].hidden = false;
    }

    for (int i=0;i<ModelInfo[current_wm].nlegsW; i++)
    {
        // hang atom?
        int n2 = abs(ModelInfo[current_wm].legsW[i].n2);
        if((Properties::ifDelHang)  && (ModelInfo[current_wm].legsW[i].ordleg == 1))
            ModelInfo[current_wm].atm[n2-1].hidden = true;
        else
            ModelInfo[current_wm].atm[n2-1].hidden = false;
    }
    ModelInfo[current_wm].ifResize = true;
    RiNormalizza();
    Draw();
}

void CrystalViewWidget::onActionEditHBondsTriggered()
{
    mHBOptionsDialog->SaveHBoptions();
    mHBOptionsDialog->show();
}

void CrystalViewWidget::onActionShowFourierMapTriggered()
{
    QAction* act = qobject_cast<QAction*>(sender());
    if(act == nullptr)
        return;
    MapInfo[0].ifVis = act->isChecked()?1:0;
    Draw();
}

void CrystalViewWidget::onActionComputeFoTriggered()
{
    int ier;
    Lanciajav(FourierType::FOUR_FO, 1, &ier);
    menu()->actionShow_Fourier_Map->setChecked(ier == 0);
}

void CrystalViewWidget::onActionCompute2FoFcTriggered()
{
    int ier;
    Lanciajav(FourierType::FOUR_2FO_FC, 1, &ier);
    menu()->actionShow_Fourier_Map->setChecked(ier == 0);
}

void CrystalViewWidget::onActionComputeFoFcTriggered()
{
    int ier;
    Lanciajav(FourierType::FOUR_FO_FC, 1, &ier);
    menu()->actionShow_Fourier_Map->setChecked(ier == 0);
}

void CrystalViewWidget::onActionComputeFcTriggered()
{
    int ier;
    Lanciajav(FourierType::FOUR_FC, 1, &ier);
    menu()->actionShow_Fourier_Map->setChecked(ier == 0);
}

void CrystalViewWidget::onActionAddHydrogensTriggered(int code)
{
    AddHydro(code);
}

void CrystalViewWidget::onActionNormaliseHydrogensTriggered()
{
    int whichModel = IsValidModel() + 1;
    if(whichModel == 0)
        return;
    updateModify(-99);
    int code = 0;
    jav_normalize_structure_(&code, ModelInfo[whichModel-1].distHn, &whichModel);
    mNormHydroDlg->setModel(whichModel-1);
    mNormHydroDlg->setDist(ModelInfo[whichModel-1].distHn);
    mNormHydroDlg->show();
}

void CrystalViewWidget::onActionBuildBondsTriggered()
{
    int nptra = 0;
    int nptrb = 0;
    int icod = 4;
    jav_modify(nullptr,&nptra,nullptr,&nptrb,&icod);
}

void CrystalViewWidget::onActionBondSettingsTriggered()
{
    if (mBondSettingsDialog->on_bondsetting() == 0)
        mBondSettingsDialog->show();
}

void CrystalViewWidget::onActionCompleteTriggered()
{
    int icod = 9;
    int whichModel = IsValidModel();
    if(whichModel == -1)
        return;
    int nptra = (int) ModelInfo[whichModel].atoms_sel.size();
    if((nptra < 3) || (nptra > 5))
    {
        ErrMsg("Sorry. You should select 3, 4 or 5 atoms.");
        return;
    }
    double bond_d;
    bool result = DistanceFromUI("Complete", &bond_d);
    if(result)
    {
        int *kptra = new int [nptra]();
        for(int i=0; i<nptra; i++)
            kptra[i] = ModelInfo[whichModel].atoms_sel[i];
        int nptrb = (int) (bond_d*1000);
        if(jav_modify(kptra,&nptra,nullptr,&nptrb,&icod))
            endModify(whichModel);
    }
}

void CrystalViewWidget::onActionRegularizeTriggered()
{
    int icod = 10;
    int whichModel = IsValidModel();
    if(whichModel == -1)
        return;
    int nptra = (int) ModelInfo[whichModel].atoms_sel.size();
    if(nptra < 3)
    {
        ErrMsg("Sorry. You should select more than 2 atoms.");
        return;
    }
    double bond_d;
    bool result = DistanceFromUI("Regularize", &bond_d);
    if(result)
    {
        int *kptra = new int [nptra]();
        for(int i=0; i<nptra; i++)
            kptra[i] = ModelInfo[whichModel].atoms_sel[i];
        int nptrb = (int) (bond_d*1000);
        if(jav_modify(kptra,&nptra,nullptr,&nptrb,&icod))
            endModify(whichModel);
    }
}

void CrystalViewWidget::onActionEnantiomorphTriggered()
{
    int icod = 11;
    int nptra = 0;
    int nptrb = 0;
    jav_modify(nullptr,&nptra,nullptr,&nptrb,&icod);
}

void CrystalViewWidget::onActionTransformTriggered()
{
    int whichModel = IsValidModel();
    if(whichModel == -1)
        return;
    if(Properties::sy.nsymop == 0)
        return;
    updateModify(3);
    mTransformDlg->Fill(ModelInfo[whichModel].atoms_sel, whichModel);
    mTransformDlg->show();
}

void CrystalViewWidget::onActionStandardizationOfCrystalStructureTriggered()
{
    int whichModel = Properties::GetActiveModel() - 1;
    if(whichModel == -1)
        return;
    jav_standardization_(&(++whichModel));
}

void CrystalViewWidget::onActionOptimizeGeometryTriggered()
{
    on_OptimizeGeometry();
}

void CrystalViewWidget::onActionOptimizeSelectedAtomsTriggered()
{
    on_OptimizeGeometry(true);
}

void CrystalViewWidget::onActionMOPACTriggered()
{
    int kmodel = IsValidModel();
    if(kmodel == -1)
        return;
    mMopacDlg->openMOPAC(kmodel+1);
    mMopacDlg->show();
}

void CrystalViewWidget::onActionViewMOPACOutputTriggered()
{
    if (!mMopacDlg->getOutFileName().isNull()) {
        TextEditor *m_text = new TextEditor(this);
        m_text->LoadFile(mMopacDlg->getOutFileName());
        m_text->show();
    } else
        QMessageBox::warning(this, tr("Error"), tr("Can't find the MOPAC output file"));
}

int CrystalViewWidget::getExportFileCode(const QString filter)
{
    QStringList list = FilesToExport.split(";;");
    int nfilt = list.lastIndexOf(filter);
    int tipo = 0;
    if (nfilt >= 0)
        tipo = FILT_IN_WRITE[nfilt];
    return tipo;
}

int CrystalViewWidget::getImportFileCode(const QString filter)
{
    QStringList list = FilesToImport.split(";;");
    int nfilt = list.lastIndexOf(filter);
    int tipo = 0;
    if (nfilt >= 0)
        tipo = FILT_IN_READ[nfilt];
    return tipo;
}

void CrystalViewWidget::onActionExportStructureTriggered()
{
    int whichModel = IsValidModel();
    if(whichModel == -1)
        return;
    QString nome = ModelInfo[whichModel].modname.c_str();
    QSettings settings;
    QString selectedFilter = settings.value(EXPORT_FILTER_KEY).toString();
    int code = 2;
    QString fileName = SaveDialog::run(this,
                                       tr("Export Structure As"),
                                       settings.value(DEFAULT_DIR_KEY).toString(),
//                                       QFileInfo(nome).baseName(),
                                       nome,
                                       FilesToExport,
                                       "cif",
                                       selectedFilter,
                                       &code);
    settings.setValue(EXPORT_FILTER_KEY,selectedFilter);
    if (!fileName.isEmpty()) {
        qInfo() << "Code: " << code;
        settings.setValue(DEFAULT_DIR_KEY,QFileInfo(fileName).absolutePath());

        int tipo = CrystalViewWidget::getExportFileCode(selectedFilter);
        if(code != 0)
           tipo += 1000;
        exportFile(fileName,tipo);
    }
}

void CrystalViewWidget::onActionOptimizeCrystalStructureByDFTDTriggered()
{
    int whichModel = IsValidModel();
    if(whichModel == -1)
        return;
    //QString nome = ModelInfo[whichModel].fileName;
    QString nome = ModelInfo[whichModel].modname.c_str();
    QSettings settings;
    QString selectedFilter = "Quantum Espresso Input(*.in)";
    QString fileName = SaveDialog::run(this, tr("Export crystal structure as"),
                                       settings.value(DEFAULT_DIR_KEY).toString(),
//                                       QFileInfo(nome).baseName(),
                                       nome,
                                       "Quantum Espresso Input (*.in);;"
                                       "NWCHEM Input (*.nw);;"
                                       "ABINIT Input (*.in);;"
                                       "CRYSTAL Input (*.d12)",
                                       "in", selectedFilter);
    if (!fileName.isEmpty()) {
        int code = getExportFileCode(selectedFilter);

        QFile file(fileName);
        if (!file.open(QFile::WriteOnly | QFile::Text)) {
            QMessageBox::warning(this, "Warning", "Cannot save file: " + file.errorString());
            return;
        }
        file.close();
        exportFile(fileName, code);
    }
}

void CrystalViewWidget::onActionOverlayStructuresTriggered()
{
    int whichModel = IsValidModel();
    if(whichModel == -1)
        return;
    if(ModelInfo[whichModel].nats <= 0)
        return;
    QSettings settings;
    QString selectedFilter;
    QString fileName = QFileDialog::getOpenFileName(this,
                                                    tr("Select file to read from"), settings.value(DEFAULT_DIR_KEY).toString(),
                                                    FilesToImport, &selectedFilter);
    if (fileName.isEmpty())
        return;
    LastModel = fileName;
    Attesa("Overlaying structures: please wait.", ui->statusLabel);
    //const char *filnam = fileName.toStdString().c_str();
    //overlay_struct_(filnam, strlen(filnam));
    overlay_struct_(fileName.toStdString().c_str(),fileName.length());
    FineAttesa(ui->statusLabel);
}

void CrystalViewWidget::onActionOverlayInfoTriggered()
{
    int whichModel = IsValidModel();
    if(whichModel == -1)
        return;
    if(LastModel.isEmpty() || (Properties::nModels <= 1))
    {
        ErrMsg("Sorry. You should load another model to overlay.");
        return;
    }
    if(mOverlay->isVisible())
    {
        mOverlay->raise();
        return;
    }
    QStringList mdList;
    for(int i=1; i<Properties::nModels; i++)
        mdList.append(ModelInfo[i].modname.c_str());
    mOverlay->fillComboModels(mdList);
    FillOverlayInfo(1, 0.6f);
    mOverlay->show();
}

void CrystalViewWidget::onDeleteModel(int kmodel)
{
    int Which = kmodel;
    ModelInfo[Which-1].freeModel();
    for(int i=Which; i<Properties::nModels; i++)
        ModelInfo[i-1] = ModelInfo[i];
    ModelInfo[Properties::nModels-1].created = 0;
    Properties::nModels--;
    Properties::DistAng_vet.clear();
    aggiornaProperties(-1);
    if(Properties::GetActiveModel() == Which)
        Properties::SetActiveModel(1);
    updateMenuDeleteModel();
    jav_delete_model_(&Which);
    RiNormalizza();
    Draw();
}

void CrystalViewWidget::on_DistChanged()
{
    FillOverlayInfo();
}

void CrystalViewWidget::on_OverlaySelectionChanged(QVector<int> sel)
{
    int whichModel = mOverlay->getModel() - 1;
    for(int i=0; i<ModelInfo[whichModel].nats; i++)
        ModelInfo[whichModel].atm[i].selected = false;
    ModelInfo[whichModel].atoms_sel.clear();
    if(!sel.empty())
    {
        int nsel = sel.count();
        for(int j=0;j<nsel;j++)
        {
            int riga = sel[j];
            int natom = compare_info->nv1[riga] - 1;
            ModelInfo[whichModel].atm[natom].selected = true;
            ModelInfo[whichModel].atoms_sel.push_back(natom+1);
        }
        Draw();
        update_items_modify(true, false);
    }
}

void CrystalViewWidget::on_OverlayModelSelected()
{
    FillOverlayInfo();
}

void CrystalViewWidget::onActionCentroidsTriggered()
{
    if(mCentDlg->isVisible())
    {
        mCentDlg->raise();
        return;
    }
    mCentDlg->show();
}

void CrystalViewWidget::on_centroidChanged()
{
    Draw();
}

void CrystalViewWidget::onActionPowderDiffractionPatternTriggered()
{
    int kmodel = IsValidModel();
    if(kmodel == -1)
        return;
    mPdPDialog->show();
    mPdPDialog->FillPowderPatternDlg(kmodel, ModelInfo[kmodel].fileName);
}

void CrystalViewWidget::onActionPairDistributionFunctionPDFTriggered()
{
    int kmodel = IsValidModel();
    if(kmodel == -1)
        return;
    mPDFDialog->show();
    mPDFDialog->FillPDFDlg(kmodel+1, ModelInfo[kmodel].fileName);
}

void CrystalViewWidget::onActionBondValenceSumTriggered()
{
    int kmodel = IsValidModel();
    if(kmodel == -1)
        return;
    if (mBondValenceDlg->on_bondvs(kmodel+1) > 0) return;
    mBondValenceDlg->show();
}

void CrystalViewWidget::onActionStructureInfoTriggered(int infoCode)
{
    mInfoDialog->show();
    mInfoDialog->FillInfo(infoCode);
}

void CrystalViewWidget::onActionGeometryTriggered()
{
    if(Properties::nModels == 0)
    {
        ErrMsg("Sorry. No models loaded.");
        return;
    }
    mGeoCalcDlg->clear();
    mGeoCalcDlg->show();
    setUnselect();
}

void CrystalViewWidget::onActionSphereTriggered()
{
    mOpenGLWindow->SetMotionType(GEOM+1, 3);
    mSphereDlg->show();
}

void CrystalViewWidget::onActionRotateTriggered(int axis, float sign)
{
    mOpenGLWindow->on_rotate(axis, sign);
}

void CrystalViewWidget::onActionTranslateTriggered(int axis, float sign)
{
    if (axis == 2) {
        float val = menu()->stepTra->text().toFloat();
        mOpenGLWindow->on_move(axis, 2*sign*val/Properties::npixels);
    } else {
        mOpenGLWindow->move_pix(axis, sign);
    }
}

void CrystalViewWidget::onActionClearMeasurementsTriggered()
{
    if(Properties::nModels == 0)
        return;
    for(int k=0; k<Properties::nModels; k++)
        for(int i=0; i<ModelInfo[k].nats; i++)
            if(ModelInfo[k].atm[i].drawLabel < 2)
                ModelInfo[k].atm[i].drawLabel = 0;
    Properties::DistAng_vet.clear();
    setStatusBarText(" ");
    Draw();
}

void CrystalViewWidget::onActionSelectionModeTriggered(bool checked)
{
    if(checked)
        mOpenGLWindow->SetMotionType(ON_SELECT, 1);
    //mainWindow->getActionAtoms_or_Bonds()->setChecked(checked);
    menu()->actionAtoms_or_Bonds->setChecked(checked);
}

void CrystalViewWidget::onActionDistancesTriggered(bool checked)
{
    if(checked)
        mOpenGLWindow->SetMotionType(GEOM+2, 3);
    else
        mOpenGLWindow->SetMotionType(NO_MOVE, 0);
}

void CrystalViewWidget::onActionAnglesTriggered(bool checked)
{
    if(checked)
        mOpenGLWindow->SetMotionType(GEOM+3, 3);
    else
        mOpenGLWindow->SetMotionType(NO_MOVE, 0);
}

void CrystalViewWidget::onActionTorsionAnglesTriggered(bool checked)
{
    if(checked)
        mOpenGLWindow->SetMotionType(GEOM+4, 3);
    else
        mOpenGLWindow->SetMotionType(NO_MOVE, 0);
}

void CrystalViewWidget::onActionSelectNoneTriggered()
{
    for(int i=0; i<Properties::nModels; i++)
    {
        for(int j=0; j<ModelInfo[i].nats; j++)
            ModelInfo[i].atm[j].selected = false;
        for(int j=0; j<ModelInfo[i].nconn_prot; j++)
            ModelInfo[i].bonds[j].selected = false;
        ModelInfo[i].atoms_sel.clear();
        ModelInfo[i].bonds_sel.clear();
        ModelInfo[i].resid_sel.clear();
        if(ModelInfo[i].IsProtein())
        {
            for(int j=0; j<(int)ModelInfo[i].c_alphab.size(); j++)
                ModelInfo[i].c_alphab[j].selected = false;
        }
    }
    Draw();
    update_items_modify(true);
    setStatusBarText(" ");
}

void CrystalViewWidget::onActionInvertSelectionTriggered()
{
    for(int whichModel=0; whichModel< Properties::nModels; whichModel++)
    {
        ModelInfo[whichModel].atoms_sel.clear();
        ModelInfo[whichModel].bonds_sel.clear();
        for(int j=0; j<ModelInfo[whichModel].nats; j++)
        {
            bool stato = ModelInfo[whichModel].atm[j].selected;
            ModelInfo[whichModel].atm[j].selected = !stato;
            if(!stato)
                ModelInfo[whichModel].atoms_sel.push_back(j+1);
        }
        for(int j=0; j<ModelInfo[whichModel].nconn_prot; j++)
        {
            bool stato = ModelInfo[whichModel].bonds[j].selected;
            ModelInfo[whichModel].bonds[j].selected = !stato;
            if(!stato)
                ModelInfo[whichModel].bonds_sel.push_back(j+1);
        }
        if(ModelInfo[whichModel].IsProtein())
        {
            for(int j=0; j<(int)ModelInfo[whichModel].c_alphab.size(); j++)
            {
                bool stato = ModelInfo[whichModel].c_alphab[j].selected;
                ModelInfo[whichModel].c_alphab[j].selected = !stato;
                if(!stato)
                    ModelInfo[whichModel].resid_sel.push_back(j+1);
            }
        }
    }
    update_items_modify(true);
    Draw();
}

void CrystalViewWidget::onActionImportFragment(PolyhedraDialog::geoms geom, bool ifAdd)
{
    mPolyhedraDialog->set_polyhedron_type(geom, ifAdd);
    mPolyhedraDialog->show();
}

void CrystalViewWidget::onActionFromSmilesTriggered(int ifAdd)
{
    importFromSmiles(ifAdd);
}

void CrystalViewWidget::onActionFromCODTriggered(int ifAdd)
{
    int WhichModel = 1;
    mCODDialog->showCOD(WhichModel,ifAdd);
}

void CrystalViewWidget::onActionUndoTriggered()
{
    int code = -1;
    jav_undo_redo_(&code);
    UpdateInfoAtoms();
}

void CrystalViewWidget::onActionRedoTriggered()
{
    int code = 1;
    jav_undo_redo_(&code);
    UpdateInfoAtoms();
}

void CrystalViewWidget::onActionDeleteSelectedAtomsOrBondsTriggered()
{
    int whichModel = IsValidModel();
    if(whichModel == -1)
        return;
    int nptra = (int) ModelInfo[whichModel].atoms_sel.size();
    int *kptra = new int[nptra]();
    for(int i=0; i<nptra; i++)
        kptra[i] = ModelInfo[whichModel].atoms_sel[i];
    int nptrb = (int) ModelInfo[whichModel].bonds_sel.size();
    int *kptrb = new int [nptrb]();
    for(int i=0; i<nptrb; i++)
        kptrb[i] = ModelInfo[whichModel].bonds_sel[i];
    int icod = 1;
    if(jav_modify(kptra,&nptra,kptrb,&nptrb,&icod,NULL,whichModel))
    {
        Properties::DistAng_vet.clear();
        endModify(whichModel);
    }
}

void CrystalViewWidget::onActionDeleteBondsTriggered()
{
    int whichModel = IsValidModel();
    if(whichModel == -1)
        return;
    int nptra = 0;
    int nptrb = (int) ModelInfo[whichModel].bonds_sel.size();
    int *kptrb = new int [nptrb]();
    for(int i=0; i<nptrb; i++)
        kptrb[i] = ModelInfo[whichModel].bonds_sel[i];
    int icod = 1;
    if(jav_modify(NULL,&nptra,kptrb,&nptrb,&icod))
        endModify(whichModel);
}

void CrystalViewWidget::onActionDeleteDuplicateAtomsTriggered()
{
    int whichModel = IsValidModel();
    if(whichModel == -1)
        return;
    int nptra = 0;
    int nptrb = 0;
    int icod = 31;
    bool ok;
    static double dinput = 0.6;
    double dist = QInputDialog::getDouble(this, "Set threshold value",
                                          "Threshold:", dinput,
                                          0.01, 2.0, 3, &ok);
    if (ok){
        dinput = dist;
        float threshold = static_cast<float>(dinput);
        if(jav_modify(NULL,&nptra,NULL,&nptrb,&icod,&threshold))
            endModify(whichModel);
    }
}

void CrystalViewWidget::onActionRenameTriggered()
{
    int k = IsValidModel();
    if(k == -1)
        return;
    int nptr = (int) ModelInfo[k].atoms_sel.size();
    if(nptr <= 0)
        return;
    QString title("Atom(s): ");
    for(int i=0; i<nptr; i++)
    {
        title.append(QString(ModelInfo[k].atm[ModelInfo[k].atoms_sel.at(i)-1].lab.c_str()).trimmed());
        if(i != (nptr-1))
            title.append(", ");
    }
    title.append("<br/>New name:");
    bool ok;
    QString newName = QInputDialog::getText(this, tr("Rename"),
                                            title, QLineEdit::Normal, tr(""), &ok);
    if (ok && !newName.isEmpty())
    {
        int nptra = (int) ModelInfo[k].atoms_sel.size();
        int *kptra = new int [nptra]();
        for(int i=0; i<nptra; i++)
            kptra[i] = ModelInfo[k].atoms_sel[i];
        int nptrb = 0;
        int newspec = 0;
        int icod = 6;
        const char *newname = newName.toStdString().c_str();
        size_t sz = strlen(newname);
        if(jav_modify(kptra,&nptra,NULL,&nptrb,&icod,NULL,k, &newspec,newname,sz))
            endModify(k);
    }
}

void CrystalViewWidget::onActionRenumberTriggered()
{
    int whichModel = IsValidModel();
    if(whichModel == -1)
        return;
    QString newName;
    int num;
    bool result = mRenumDlg->ShowAndRun(&newName, &num);

    if(result)
    {
        const char *nome = newName.toStdString().c_str();
        size_t sz = strlen(nome);
        int nptra = (int) ModelInfo[whichModel].atoms_sel.size();
        int *kptra = new int [nptra]();
        for(int i=0; i<nptra; i++)
            kptra[i] = ModelInfo[whichModel].atoms_sel[i];
        int nptrb = 0;
        int icod = 8;
        if(jav_modify(kptra,&nptra,nullptr,&nptrb,&icod,nullptr,whichModel, &num,nome,sz))
            endModify(whichModel);
    }
}

void CrystalViewWidget::onActionSorBySpeciesTriggered()
{
    int whichModel = IsValidModel();
    if(whichModel == -1)
        return;
    if(ModelInfo[whichModel].nats <= 0)
        return;
    int nptra = 0;
    int nptrb = 0;
    int icod = 7;
    jav_modify(nullptr,&nptra,nullptr,&nptrb,&icod);
}

void CrystalViewWidget::onActionAddBondsTriggered()
{
    int whichModel = IsValidModel();
    if(whichModel == -1)
        return;
    int nptra = (int) ModelInfo[whichModel].atoms_sel.size();
    if(nptra < 2)
    {
        ErrMsg("Sorry. You should select 2 atoms at least.");
        return;
    }
    int *kptra = new int [nptra]();
    for(int i=0; i<nptra; i++)
        kptra[i] = ModelInfo[whichModel].atoms_sel[i];
    int nptrb = 0;
    int icod = 2;
    if(jav_modify(kptra,&nptra,nullptr,&nptrb,&icod))
        endModify(whichModel);
}

void CrystalViewWidget::onActionFromFileTriggered()
{
    QSettings settings;
    QString selectedFilter;
    QString fileName = QFileDialog::getOpenFileName(this,
                                                    tr("Select file to read from"), settings.value(DEFAULT_DIR_KEY).toString(),
                                                    FilesToImport, &selectedFilter);
    if (!fileName.isEmpty()) {

        QStringList list = FilesToImport.split(";;");
        int nfilt = list.lastIndexOf(selectedFilter);
        int tipo = 0;
        if (nfilt >= 0)
            tipo = FILT_IN_READ[nfilt];
        const char *cStr;
        QByteArray inBytes = fileName.toUtf8();
        cStr = inBytes.constData();
        size_t sz = strlen(cStr);
        int icod = 22;
        int nptra = 1, nptrb = tipo, idum = 0;
        int *kptra = new int();
        kptra[0] = 0;
        int km = IsValidModel();
        if(km == -1) km = 0;
        //jav_modify(kptra, &nptra, nullptr, &nptrb, &icod, NULL,-1, &idum,
        jav_modify(kptra, &nptra, nullptr, &nptrb, &icod, NULL,km, &idum,
                   cStr, sz);
    }
}

void CrystalViewWidget::onActionDuplicateAllTriggered()
{
    int nptra = 0;
    int nptrb = 0;
    int icod = 29;
    jav_modify(nullptr,&nptra,nullptr,&nptrb,&icod);
}

void CrystalViewWidget::onActionDuplicateSelectedAtomsTriggered()
{
    int whichModel = IsValidModel();
    if(whichModel < 0)
        return;
    if(ModelInfo[whichModel].atoms_sel.empty())
        return;
    int nptra = (int) ModelInfo[whichModel].atoms_sel.size();
    int *kptra = new int [nptra]();
    for(int i=0; i<nptra; i++)
        kptra[i] = ModelInfo[whichModel].atoms_sel[i];
    int nptrb = 0;
    int icod = 29;
    jav_modify(kptra,&nptra,nullptr,&nptrb,&icod);
    endModify(whichModel);
}

void CrystalViewWidget::onChangeSpecie(int whichZ)
{
    int whichModel = IsValidModel();
    if(whichModel == -1)
        return;
    int whichSpec;
    if(whichZ < OTHERS)
    {
        int i;
        for (i=0; i< ModelInfo[whichModel].numSpec; i++)
        {
            if(ModelInfo[whichModel].Species[i] == whichZ)
                break;
        }
        if( i == ModelInfo[whichModel].numSpec)
            return;
        whichSpec = i+1;
    }
    else
    {
        bool ok;
        TableWindow *mTableWindow = new TableWindow(this);
        int result = mTableWindow->GetZ(&ok);
        if (!ok || (result < 0))
            return;
        whichSpec = -(result+1);
    }
    int nptra = (int) ModelInfo[whichModel].atoms_sel.size();
    int *kptra = new int [nptra]();
    for(int i=0; i<nptra; i++)
        kptra[i] = ModelInfo[whichModel].atoms_sel[i];
    int nptrb = 0;
    int icod = 3;
    if(jav_modify(kptra,&nptra,NULL,&nptrb,&icod,NULL,whichModel, &whichSpec))
        endModify(whichModel);
}

void CrystalViewWidget::onLabel_by_spec(int z)
{
    QAction* act = qobject_cast<QAction*>(sender());
    if(act == nullptr)
        return;
    int whichModel = IsValidModel();
    if(whichModel == -1)
        return;
    bool stato = act->isChecked();
    if(stato)
    {
        menu()->actionAll_Atoms->setChecked(false);
        menu()->actionHeavy_atoms->setChecked(false);
        menu()->actionexcept_Hydrogens->setChecked(false);
        //ui->actionselected_atoms->setChecked(false);
    }
    ModelInfo[whichModel].specialLab = stato?-1:0;
    ModelInfo[whichModel].LabelS[z] = stato?1:0;
    Draw();
}

void CrystalViewWidget::onDelete_by_spec(int z)
{
    if(selezionaSpecie(z))
    {
        int whichModel = Properties::GetActiveModel() - 1;
        int nptra = (int) ModelInfo[whichModel].atoms_sel.size();
        int *kptra = new int[nptra]();
        for(int i=0; i<nptra; i++)
            kptra[i] = ModelInfo[whichModel].atoms_sel[i];
        int nptrb = 0;
        int icod = 1;
        if(jav_modify(kptra,&nptra,nullptr,&nptrb,&icod,nullptr,whichModel))
        {
            Properties::DistAng_vet.clear();
            endModify(whichModel);
        }
    }
}

void CrystalViewWidget::onSelect_by_spec(int z)
{
    selezionaSpecie(z);
    update_items_modify(IsModifyEnabled());
}

void CrystalViewWidget::onActionStructurePropertiesTriggered(int prop)
{
    ModifyProperties(prop);
}

void CrystalViewWidget::onAddAtom(int whichZ)
{
    int whichModel = IsValidModel();
    if(whichModel == -1)
        return;
    int whichSpec;
    if(whichZ < OTHERS)
    {
        int i;
        for (i=0; i< ModelInfo[whichModel].numSpec; i++)
        {
            if(ModelInfo[whichModel].Species[i] == whichZ)
                break;
        }
        if( i == ModelInfo[whichModel].numSpec)
            return;
        whichSpec = i+1;
    }
    else
    {
        bool ok;
        TableWindow *mTableWindow = new TableWindow(this);
        int result = mTableWindow->GetZ(&ok);
        if (!ok || (result < 0))
            return;
        whichSpec = -(result+1);
    }
    int nptra = (int) ModelInfo[whichModel].atoms_sel.size();
    int *kptra = new int [nptra]();
    for(int i=0; i<nptra; i++)
        kptra[i] = ModelInfo[whichModel].atoms_sel[i];
    int nptrb = defaultBondMult+1;
    int icod = 32;
    if(jav_modify(kptra,&nptra,NULL,&nptrb,&icod,NULL,whichModel, &whichSpec))
        endModify(whichModel);
}

void CrystalViewWidget::onChangeMult(int which)
{
    QAction *action = qobject_cast<QAction *>(sender());
    if(action->isChecked())
        defaultBondMult = which;
}

void CrystalViewWidget::onActionSpecie(int actionIndex)
{
    QAction *action = qobject_cast<QAction *>(sender());
    qInfo() << "Action name: " << action->text() << action->objectName() << action->data().toString();
    if (action->data().toString() == "delete") {
        // Delete
        if (actionIndex == -1) {      // Delete ghosts
            onDelete_by_spec(OTHERS);
            return;
        }

        int z = Properties::ListaSpecies[actionIndex]-1;
        onDelete_by_spec(z+1);
    } else if (action->data().toString() == "change") {
        // Change specie
        if (actionIndex == -1) {     // Other
            onChangeSpecie(OTHERS +1);
            return;
        }

        int z = Properties::ListaSpecies[actionIndex]-1;
        onChangeSpecie(z+1);
    } else if (action->data().toString() == "selection") {
        // Select specie
        if (actionIndex == -1) {     // Select ghosts
            onSelect_by_spec(OTHERS);
            return;
        }
        int z = Properties::ListaSpecies[actionIndex]-1;
        onSelect_by_spec(z+1);
    } else if (action->data().toString() == "add") {
        // Add atom
        if (actionIndex == -1) {     // Add ghosts
            onAddAtom(OTHERS+1);
            return;
        }
        int z = Properties::ListaSpecies[actionIndex]-1;
        onAddAtom(z+1);
    }
}

void CrystalViewWidget::onActionAtomsOrBondsTriggered()
{
    menu()->actionSelection_Mode->trigger();
}

void CrystalViewWidget::onActionRectangularTriggered(bool checked)
{
    qInfo() << "Checked: " << checked << "from onActionRectangularTriggered()";
    if(checked)
        mOpenGLWindow->SetMotionType(RECT_SELECT, 1);
}

void CrystalViewWidget::onActionMoleculeTriggered(bool checked)
{
    qInfo() << "Checked: " << checked << "from onActionMoleculeTriggered()";
    if(checked)
        mOpenGLWindow->SetMotionType(MOL_SELECT, 1);
}

void CrystalViewWidget::onActionZoomModeTriggered(bool checked)
{
    if(checked)
        mOpenGLWindow->SetMotionType(ZOOM_MODE, 4);
}

void CrystalViewWidget::onActionSelectAllTriggered()
{
    for(int i=0; i<Properties::nModels; i++)
    {
        ModelInfo[i].atoms_sel.clear();
        ModelInfo[i].bonds_sel.clear();
        ModelInfo[i].resid_sel.clear();
        for(int j=0; j<ModelInfo[i].nats; j++)
        {
            ModelInfo[i].atm[j].selected = true;
            ModelInfo[i].atoms_sel.push_back(j+1);
        }
        for(int j=0; j<ModelInfo[i].nconn_prot; j++)
        {
            ModelInfo[i].bonds[j].selected = true;
            ModelInfo[i].bonds_sel.push_back(j+1);
        }
        if(ModelInfo[i].IsProtein())
        {
            for(int j=0; j<(int)ModelInfo[i].c_alphab.size(); j++)
            {
                ModelInfo[i].c_alphab[j].selected = true;
                ModelInfo[i].resid_sel.push_back(j+1);
            }
        }
    }
    Draw();
    update_items_modify(IsModifyEnabled());
    setStatusBarText(" ");
}

void CrystalViewWidget::onActionExpandToMoleculeTriggered()
{
    int whichModel = IsValidModel()+1;
    if(whichModel == 0)
        return;
    int nptra = (int) ModelInfo[whichModel-1].atoms_sel.size();
    int *kptra = new int [ModelInfo[whichModel-1].nats]();
    for(int i=0; i<nptra; i++)
        kptra[i] = ModelInfo[whichModel-1].atoms_sel[i];
    for(int i=nptra; i<ModelInfo[whichModel-1].nats; i++)
        kptra[i] = 0;
    jav_expand_selection_(kptra,&nptra,&whichModel);
    ModelInfo[whichModel-1].atoms_sel.clear();
    for(int i=0; i<nptra; i++)
    {
        if(kptra[i] > 0)
        {
            ModelInfo[whichModel-1].atoms_sel.push_back(kptra[i]);
            ModelInfo[whichModel-1].atm[kptra[i]-1].selected = true;
        }
    }
    Draw();
}

void CrystalViewWidget::onActionPreferencesTriggered()
{
    mOptionsDialog->show();
    mOptionsDialog->setOptions();
}

void CrystalViewWidget::onActionLabelsOnOff(bool state, QAction *actionAllAtoms)
{
    int whichModel = IsValidModel();
    if(whichModel == -1)
        return;
    ModelInfo[whichModel].ifLabels = state;
    bool oldState = actionAllAtoms->blockSignals(true);
    actionAllAtoms->setChecked(ModelInfo[whichModel].ifLabels);
    actionAllAtoms->blockSignals(oldState);
    aggiornaProperties(whichModel);
    Draw();
}

void CrystalViewWidget::setModelNameFromFile(const QString &fileName)
{
    int which = Properties::GetActiveModel() - 1;
    if (which >= 0)
    {
        ModelInfo[which].fileName = fileName;
        ModelInfo[which].modname = QFileInfo(fileName).baseName().toStdString();
        //aggiornaProperties(which);
    }
}

void CrystalViewWidget::onActionExportStructureImageTriggered()
{
    QSettings settings;
    QString selectedFilter("PNG(*.png)");
    int code = 0;
    if(GetBgType() <= ColorWidget::TYPE_FLAT)
        code = 1;
    QString fileName = SaveDialog::run(this,
                                       tr("Export Graphics"),
                                       settings.value(DEFAULT_DIR_KEY).toString(),
                                       QFileInfo(mainWindow->getCurrentFile()).baseName(),
                                       "Windows icon(*.ico,*.cur);;"
                                       "PNG(*.png);;"
                                       "BMP(*.bmp);;"
                                       "JPEG(*.jpeg,*jpe,*jpg);;"
                                       "TIFF(*.tiff,*.tif)",
                                       "png",
                                       selectedFilter,
                                       &code);
    if (!fileName.isEmpty()) {
        settings.setValue(DEFAULT_DIR_KEY,QFileInfo(fileName).absolutePath());

        QImage pix1 = mOpenGLWindow->grabFramebuffer(); //            grab(rectangle);
        QImage pix = pix1.convertToFormat(QImage::Format_ARGB32);
        if(code != 0)
        {
            QColor bg = Properties::BgColor;
            for (int x(0); x != pix.width(); ++x)
            {
                for (int y(0); y != pix.height(); ++y)
                {
                    QColor color(pix.pixel(x,y));
                    if(color == bg)
                        color.setAlpha(0);
                    pix.setPixel(x, y, color.rgba());
                }
            }
        }
        if(!pix.save(fileName))
            QMessageBox::warning(this, tr("Save Error"), tr("The image could not be saved to \"%1\".")
                                                             .arg(QDir::toNativeSeparators(fileName)));
    }
}

void CrystalViewWidget::onActionExport3DTriggered()
{
    QSettings settings;
    QString selectedFilter("STereoLithography file format (*.stl)");
    if(Properties::nModels <= 0)
        return;
    QString fileName = SaveDialog::run(this,
                                       tr("Export Scene"),
                                       settings.value(DEFAULT_DIR_KEY).toString(),
                                       QFileInfo(mainWindow->getCurrentFile()).baseName(),
                                       "STereoLithography file format (*.stl)",
                                       "stl",
                                       selectedFilter);
    if (!fileName.isEmpty()) {
        settings.setValue(DEFAULT_DIR_KEY,QFileInfo(fileName).absolutePath());

        std::vector<vet3> xyzat;
        xyzat.clear();
        StartExport();
        for (int i=0; i<Properties::nModels; i++)
            ModelInfo[i].exportScene(xyzat);
        EndExport(fileName, xyzat);
    }
}

void CrystalViewWidget::endModify(int whichModel, bool exitSelect)
{
    if(exitSelect)
        menu()->actionManipulation_Mode->setChecked(true);
    if(!ModelInfo[whichModel].created)
        return;
    ModelInfo[whichModel].atoms_sel.clear();
    for(int i=0; i<ModelInfo[whichModel].nats; i++)
        ModelInfo[whichModel].atm[i].selected = false;
    ModelInfo[whichModel].bonds_sel.clear();
    for(int i=0; i<ModelInfo[whichModel].nconn_prot; i++)
        ModelInfo[whichModel].bonds[i].selected = false;
    if(ModelInfo[whichModel].IsProtein())
    {
        ModelInfo[whichModel].resid_sel.clear();
        for(size_t i=0; i<ModelInfo[whichModel].c_alphab.size(); i++)
            ModelInfo[whichModel].c_alphab[i].selected = false;
    }
    update_items_modify(true);
}

bool CrystalViewWidget::IsModifyEnabled()
{
    return menu()->menu_Modify->isEnabled();
}

void CrystalViewWidget::on_DeleteResidues(int which)
{
    int whichModel = which % 10;
    int nsel = (int) ModelInfo[whichModel].resid_sel.size();
    for(int i=0; i<nsel; i++)
    {
        int j = ModelInfo[whichModel].resid_sel.at(i) - 1;
        ModelInfo[whichModel].DeleteResidue(j);
    }
    endModify(whichModel);
    ModelInfo[whichModel].PreparaTabelle();
    Draw();
}

void CrystalViewWidget::on_DeleteSideChain(int which)
{
    int whichModel = which % 10;
    int nsel = (int) ModelInfo[whichModel].resid_sel.size();
    for(int i=0; i<nsel; i++)
    {
        int j = ModelInfo[whichModel].resid_sel.at(i) - 1;
        ModelInfo[whichModel].DeleteSideChain(j);
    }
    endModify(whichModel);
    ModelInfo[whichModel].PreparaTabelle();
    Draw();
}

void CrystalViewWidget::on_DeleteAromatic(int which)
{
    int whichModel = which % 10;
    int nsel = (int) ModelInfo[whichModel].resid_sel.size();
    for(int i=0; i<nsel; i++)
    {
        int j = ModelInfo[whichModel].resid_sel.at(i) - 1;
        ModelInfo[whichModel].DeleteRing(j);
    }
    endModify(whichModel);
    ModelInfo[whichModel].PreparaTabelle();
    Draw();
}

void CrystalViewWidget::onContactAtom(int which)
{
    int whichAtom = which / 10 + 1;
    int whichModel = which % 10 + 1;
    int code = 1, ncontact=0;
    jav_expand_contacts_(&whichAtom, &whichModel, &code, &ncontact);
}

void CrystalViewWidget::onContactMol(int which)
{
    int whichAtom = which / 10 + 1;
    int whichModel = which % 10 + 1;
    int code = 2, ncontact=0;
    jav_expand_contacts_(&whichAtom, &whichModel, &code, &ncontact);
}

void CrystalViewWidget::on_Rejected()
{
    for(int i=0; i<Properties::nModels; i++)
    {
        for(int j=0; j<ModelInfo[i].nats; j++)
            if(ModelInfo[i].atm[j].drawLabel < 2)
                ModelInfo[i].atm[j].drawLabel = 0;
    }
    mOpenGLWindow->SetMotionType(NO_MOVE, 0);
    SetSphereCentre(-1, -1, false);
    SetSphereRadius(-1, false);
    Properties::DrawSphere = false;
    setUnselect();
    Draw();
    updateModify(1);
}

void CrystalViewWidget::on_trasformDlgClosed()
{
    updateModify(1);
}

void CrystalViewWidget::on_SelectedCentre(QString lab0)
{
    QString lab = lab0.trimmed();
    int wm = 0, wa = -1;
    for(int i=0; i<ModelInfo[wm].nats; i++)
    {
        QString lab1 = QString(ModelInfo[wm].atm[i].lab.c_str()).trimmed();
        if(lab.compare(lab1, Qt::CaseInsensitive) == 0)
        {
            wa = i;
            break;
        }
    }
    if(wa > -1)
        SetSphereCentre(wm, wa, true);
}

void CrystalViewWidget::on_SelectedRadius(float val)
{
    SetSphereRadius(val);
}

void CrystalViewWidget::on_SelectedRow(QVector<int> v)
{
    int whichModel = mSphereDlg->getModel() - 1;
    for(int i=0; i<ModelInfo[whichModel].nats; i++)
        ModelInfo[whichModel].atm[i].selected = false;
    ModelInfo[whichModel].atoms_sel.clear();
    for(int i=0; i<v.count(); i++)
    {
        int k = v[i];
        ModelInfo[whichModel].atm[k].selected = true;
        ModelInfo[whichModel].atoms_sel.push_back(k+1);
    }
    Draw();
}

void CrystalViewWidget::on_propertiesDialogClosed()
{
    menu()->actionSelect_None->trigger();
    updateModify(1);
}

void CrystalViewWidget::on_selectedAtom(int which)
{
    int km = IsValidModel();
    if(km == -1)
        return;
    ModelInfo[km].ClearAtomsSelection();
    ModelInfo[km].SelezionaAtomo(which);
    Draw();
}

void CrystalViewWidget::on_selectedBond(int which)
{
    int km = IsValidModel();
    if(km == -1)
        return;
    ModelInfo[km].ClearBondsSelection();
    int nl = ModelInfo[km].nconn_prot;
    int nl_au = ModelInfo[km].nlegs_au;
    if(nl == nl_au)
        ModelInfo[km].SelezionaBond(which);
    else
    {
        int nn1 = mPropDlg->ModelEdit->bonds[which].n1;
        int nn2 = mPropDlg->ModelEdit->bonds[which].n2;
        ModelInfo[km].SelezionaBond(which, nn1, nn2);
    }
    Draw();
}

void CrystalViewWidget::on_selectedAngle(int which)
{
    int km = IsValidModel();
    if(km == -1)
        return;
    ModelInfo[km].ClearAtomsSelection();
    ModelInfo[km].ClearBondsSelection();
    ModelInfo[km].SelezionaAngolo(which);
    Draw();
}

void CrystalViewWidget::on_selectedTorsion(int which)
{
    int km = IsValidModel();
    if(km == -1)
        return;
    ModelInfo[km].ClearAtomsSelection();
    ModelInfo[km].ClearBondsSelection();
    ModelInfo[km].SelezionaTorsion(which);
    Draw();
}

void CrystalViewWidget::on_normHydroDlgClosed(int code, int nelems)
{
    int whichModel = IsValidModel() + 1;
    if(whichModel == 0)
        return;
    if(code != 3)
    {
        float *v = mNormHydroDlg->getDistances();
        for(int i=0; i<nelems; i++)
            ModelInfo[whichModel-1].distHn[i] = v[i];
    }
    jav_normalize_structure_(&code, ModelInfo[whichModel-1].distHn, &whichModel);
    if(code == 2)
        updateModify(-99);
    else
        updateModify(1);
}

bool CrystalViewWidget::selezionaSpecie(int z)
{
    bool found = false;
    int whichModel = IsValidModel();
    if(whichModel == -1)
        return found;
    for(int j=0; j<ModelInfo[whichModel].nats; j++)
    {
        int spec = ModelInfo[whichModel].atm[j].Z;
        if(spec == z)
        {
            found = true;
            break;
        }
    }
    ModelInfo[whichModel].atoms_sel.clear();
    if(found)
    {
        for(int j=0; j<ModelInfo[whichModel].nats; j++)
        {
            ModelInfo[whichModel].atm[j].selected = false;
            int spec = ModelInfo[whichModel].atm[j].Z;
            if(spec == z)
            {
                ModelInfo[whichModel].atm[j].selected = true;
                ModelInfo[whichModel].atoms_sel.push_back(j+1);
            }
        }
    }
    update_items_modify(true);
    Draw();
    return found;
}

bool CrystalViewWidget::DistanceFromUI(QString title, double *val)
{
    bool ok;
#if QT_VERSION < QT_VERSION_CHECK(5, 10, 0)
    double d = QInputDialog::getDouble(this, title, "Bond Length",
                                       1.390, 0.9, 1.6, 3, &ok);
#else
    double d = QInputDialog::getDouble(this, title, "Bond Length",
                                       1.390, 0.9, 1.6, 3, &ok, Qt::WindowFlags(), 0.01);
#endif
    if (ok)
        *val = d;
    return ok;
}

QAction *CrystalViewWidget::getAction(QString label)
{
    QList<QAction*> actions = menu()->menuLabel->actions();
    for (int i = 0; i < actions.size(); ++i)
    {
        if (actions.at(i)->property("text") == label)
            return actions.at(i);
    }
    return nullptr;
}

void CrystalViewWidget::AddHydro(int icod)
{
    int whichModel = IsValidModel();
    if(whichModel == -1)
        return;
    if(ModelInfo[whichModel].nats <= 0)
        return;
    size_t ptra = ModelInfo[whichModel].atoms_sel.size();
    int nptrb = 0;
    int nptra = static_cast<int> (ptra);
    int *kptra = new int [ptra]();
    for(size_t i=0; i<ptra; i++)
        kptra[i] = ModelInfo[whichModel].atoms_sel[i];
    if(jav_modify(kptra,&nptra,nullptr,&nptrb,&icod))
    {
        Properties::DistAng_vet.clear();
        endModify(whichModel);
    }
}

void CrystalViewWidget::set_label_status()
{
    bool stato = menu()->actionAll_Atoms->isChecked() || menu()->actionHeavy_atoms->isChecked() ||
                 menu()->actionexcept_Hydrogens->isChecked(); // || ui->actionselected_atoms->isChecked();
    if(!stato)
        menu()->actionAll_Atoms->setChecked(true);
}

void CrystalViewWidget::GesLabels(bool stato, bool force)
{
    int whichModel = IsValidModel();
    if(whichModel == -1)
        return;
    ModelInfo[whichModel].ToggleLabels(stato);
    aggiornaProperties(whichModel);
    if(stato || force)
    {
        ModelInfo[whichModel].specialLab = 0;
        menu()->actionexcept_Hydrogens->setChecked(false);
        menu()->actionHeavy_atoms->setChecked(false);
        ModelInfo[whichModel].SetLabelsSelected(false);
        //ui->actionselected_atoms->setChecked(false);
        GesLabelsSpec();
    }
    Draw();
}

void CrystalViewWidget::GesLabelsSpec()
{
    for (int i = 0; i < Properties::NSpecies; i++)
    {
        QString item_lab = get_nth_element(Properties::ListaSpecies[i]-1);
        item_lab.append(" Atoms");
        QAction *newAct = getAction(item_lab);
        if(newAct != nullptr)
            newAct->setChecked(false);
    }
}

void CrystalViewWidget::UpdateMenus()
{
    menu()->actionDeleteGhost->setVisible(true);
    menu()->actionSelectGhost->setVisible(true);
    menu()->actionChangeSpecieOther->setVisible(true);
    menu()->actionAddAtomOther->setVisible(true);

    int nspecMin = qMin(Properties::NSpecies, (int) menu()->MaxSpeciesNr);
    for (int i = 0; i < nspecMin; i++)
    {
        QString item_lab = get_nth_element(Properties::ListaSpecies[i]-1);
        item_lab.append(" Atoms");
        menu()->deleteSpecieActions.at(i)->setText(item_lab);
        menu()->deleteSpecieActions.at(i)->setVisible(true);
        menu()->changeSpecieActions.at(i)->setText(item_lab);
        menu()->changeSpecieActions.at(i)->setVisible(true);
        menu()->selectSpecieActions.at(i)->setText(item_lab);
        menu()->selectSpecieActions.at(i)->setVisible(true);
        menu()->addAtomActions.at(i)->setText(item_lab);
        menu()->addAtomActions.at(i)->setVisible(true);
    }
    for (int i = nspecMin; i < menu()->MaxSpeciesNr; i++)
    {
        menu()->deleteSpecieActions.at(i)->setVisible(false);
        menu()->changeSpecieActions.at(i)->setVisible(false);
        menu()->selectSpecieActions.at(i)->setVisible(false);
        menu()->addAtomActions.at(i)->setVisible(false);
    }

    foreach (QAction *action, menu()->bondTypeActions) {
        action->setVisible(true);
    }
    menu()->bondTypeActions.at(defaultBondMult)->setChecked(true);

    return;

    // QStringList radios = {"Single", "Double", "Triple"};
    // menu()->menuD_elete_All_Types->clear();
    // menu()->menuSele_ct_Type->clear();
    // menu()->menu_Change_Species->clear();
    // menu()->menuAdd_At_om->clear();
    // QList<QAction*> actions = menu()->menuLabel->actions();
    // for (int i = 0; i < actions.size(); ++i)
    // {
    //     if (actions.at(i)->property("iconText") == "All Atoms")
    //         continue;
    //     else if (actions.at(i)->property("iconText") == "Delete All Labels")
    //         continue;
    //     else if (actions.at(i)->property("iconText") == "Except Hydrogens")
    //         continue;
    //     else if (actions.at(i)->property("iconText") == "Heavy Atoms")
    //         continue;
    //     else if (actions.at(i)->property("iconText") == "Selected Atoms")
    //         continue;
    //     else
    //         menu()->menuLabel->removeAction(actions.at(i));
    // }

    // QAction *newAct = menu()->menuD_elete_All_Types->addAction("Ghosts");
    // connect(newAct, &QAction::triggered, this, std::bind(&CrystalViewWidget::onDelete_by_spec, this, 0));
    // newAct->setCheckable(false);
    // newAct = menu()->menuSele_ct_Type->addAction("Ghosts");
    // connect(newAct, &QAction::triggered, this, std::bind(&CrystalViewWidget::onSelect_by_spec, this, OTHERS));
    // newAct->setCheckable(false);

    // for (int i = 0; i < Properties::NSpecies; i++)
    // {
    //     int z = Properties::ListaSpecies[i]-1;
    //     QString elem = get_nth_element(Properties::ListaSpecies[i]-1);
    //     QString item_lab = elem;
    //     item_lab.append(" Atoms");
    //     newAct = menu()->menuLabel->addAction(item_lab);
    //     connect(newAct, &QAction::triggered, this, std::bind(&CrystalViewWidget::onLabel_by_spec, this, z));
    //     newAct->setCheckable(true);
    //     newAct = menu()->menuD_elete_All_Types->addAction(item_lab);
    //     connect(newAct, &QAction::triggered, this, std::bind(&CrystalViewWidget::onDelete_by_spec, this, z+1));
    //     newAct->setCheckable(false);

    //     newAct = menu()->menuSele_ct_Type->addAction(item_lab);
    //     connect(newAct, &QAction::triggered, this, std::bind(&CrystalViewWidget::onSelect_by_spec, this, z+1));
    //     newAct->setCheckable(false);

    //     item_lab = get_nth_element_name(Properties::ListaSpecies[i]-1);
    //     newAct = menu()->menu_Change_Species->addAction(item_lab);
    //     connect(newAct, &QAction::triggered, this, std::bind(&CrystalViewWidget::onChangeSpecie, this, z+1));
    //     newAct->setCheckable(false);

    //     newAct = menu()->menuAdd_At_om->addAction(item_lab);
    //     connect(newAct, &QAction::triggered, this, std::bind(&CrystalViewWidget::onAddAtom, this, z+1));
    //     newAct->setCheckable(false);
    // }
    // newAct = menu()->menu_Change_Species->addAction("Other");
    // connect(newAct, &QAction::triggered, this, std::bind(&CrystalViewWidget::onChangeSpecie, this, OTHERS+1));
    // newAct->setCheckable(false);

    // newAct = menu()->menuAdd_At_om->addAction("Other");
    // connect(newAct, &QAction::triggered, this, std::bind(&CrystalViewWidget::onAddAtom, this, OTHERS+1));
    // newAct->setCheckable(false);
    // menu()->menuAdd_At_om->addSeparator();
    // QActionGroup *MultGroup = new QActionGroup(this);
    // for(int i=0; i<3; i++)
    // {
    //     newAct = menu()->menuAdd_At_om->addAction(radios.at(i));
    //     connect(newAct, &QAction::triggered, this, std::bind(&CrystalViewWidget::onChangeMult, this, i));
    //     newAct->setCheckable(true);
    //     MultGroup->addAction(newAct);
    //     if(i == defaultBondMult)
    //         newAct->setChecked(true);
    // }
}

void CrystalViewWidget::SetActive(int which)
{
    Q_UNUSED(which);
    qDebug() << "FIX subroutine " <<  __FUNCTION__;
}

void CrystalViewWidget::SetSphereCentre(int wm, int wa, bool ifRedraw)
{
    if((wm != -1) && (wa != -1))
    {
        Properties::sphereCentre = wa + 1;
        Properties::DrawSphere = true;
        Properties::SetCentreCoords(ModelInfo[wm].atm[wa].xyzN);
        float dist = Properties::sphereRadius;
        int kat = Properties::sphereCentre;
        int kmodel = wm+1;
        jav_sphere_(&kat,&dist, &kmodel);
    }
    else
    {
        Properties::sphereCentre = -1;
        Properties::DrawSphere = false;
    }
    if(mSphereDlg->isVisible())
    {
        QString lab("");
        if(Properties::DrawSphere)
            lab.append(ModelInfo[wm].atm[wa].lab.c_str());
        mSphereDlg->SetCentre(lab.trimmed());
        if(Properties::DrawSphere)
            mSphereDlg->SetRadius(Properties::sphereRadius);
        else
            mSphereDlg->SetRadius(1.5f);
    }
    if(ifRedraw)
        Draw();
}

void CrystalViewWidget::on_OptimizeGeometry(bool onlySelected)
{
    int whichModel = IsValidModel() + 1;
    if(whichModel <= 0)
        return;
    if(ModelInfo[whichModel-1].nats <= 0)
        return;
    int nptra = 0;
    int *kptra = nullptr;
    if(onlySelected)
    {
        nptra = (int) ModelInfo[whichModel-1].atoms_sel.size();
        kptra = new int [nptra]();
        for(int i=0; i<nptra; i++)
            kptra[i] = ModelInfo[whichModel-1].atoms_sel[i];
    }
    jav_optimize_geometry_(&whichModel, kptra, &nptra);
}

void CrystalViewWidget::FillOverlayInfo(int km, float dist)
{
    if(abs(dist) < 1e-6)
        dist = mOverlay->getDist();
    int whichModel;
    int overlayModel;
    int n0, n1, n2, nass;
    if(km <= 0)
    {
        whichModel = mOverlay->getModel() - 1;
        overlayModel = mOverlay->getOverlayModel();
    }
    else
    {
        whichModel = km - 1;
        overlayModel = Properties::nModels;
    }
    if(compare_info != nullptr)
        delete compare_info;
    compare_info = new compare_info_type ();
    jav_get_info_overlay1_(&dist, &nass, &n0, &n1, &n2, &(compare_info->distmin),
                           &(compare_info->distm), &(compare_info->rmsd),
                           &(compare_info->rmsd_nonH),
                           &(compare_info->errph),
                           &(compare_info->nref), &overlayModel);
    compare_info->nfound = n0;
    compare_info->nass = nass;
    atom_type *at1    = new atom_type [n1]();
    atom_type *at2    = new atom_type [n2]();
    int       *nv1    = new int [nass]();
    int       *nv2    = new int [nass]();
    float     *vdist  = new float [nass]();
    jav_get_info_overlay2_(&dist, nv1, nv2, vdist, at1, at2,
                           compare_info->shift, &overlayModel);
    compare_info->at1    = at1;
    compare_info->at2    = at2;
    compare_info->nv1    = nv1;
    compare_info->nv2    = nv2;
    compare_info->vdist  = vdist;

    mOverlay->fillResults(compare_info, ModelInfo[whichModel].modname.c_str(), n1, n2);
}

void CrystalViewWidget::set_symm(bool symcheck)
{
    int whichModel = IsValidModel();
    if(whichModel == -1)
        return;
    int i = whichModel + 1;
    ModelInfo[whichModel].ifSymmetry = symcheck;
    aggiornaProperties(whichModel);
    aggiornaMenuSym();
    CApplicaSimmetria(&i);
}

void CrystalViewWidget::aggiornaMenuSym()
{
    int whichModel = IsValidModel();
    if(whichModel == -1)
        return;
    bool ifsy = ModelInfo[whichModel].ifSymmetry;
    //ui->actionPacking->setChecked(ifsy);
    float symlevel = Minimo(ModelInfo[whichModel].klevel, 3);
    menu()->actionExpand->setEnabled(ifsy && (symlevel < MAX_SYMLEVEL));
    symlevel = Massimo(ModelInfo[whichModel].klevel, 3);
    menu()->actionReduce->setEnabled(ifsy && (symlevel > 0));
}

void CrystalViewWidget::SetSphereRadius(float r, bool ifRedraw)
{
    if(r > 0)
        Properties::sphereRadius = r;
    if(Properties::sphereCentre > 0)
    {
        float dist = Properties::sphereRadius;
        int kat = Properties::sphereCentre;
        int kmodel = 1;
        if(mSphereDlg->isVisible())
            kmodel = mSphereDlg->getModel();
        jav_sphere_(&kat,&dist, &kmodel);
    }
    if(ifRedraw)
        Draw();
}

GLint CrystalViewWidget::getMaxSamples()
{
    GLint maxSamples = 0;
    QOffscreenSurface sfc;
    sfc.create();
    QOpenGLContext ctx;
    if(ctx.create()) {
        ctx.makeCurrent(&sfc);
        glGetIntegerv(GL_MAX_SAMPLES, &maxSamples);
        ctx.doneCurrent();
    }
    return maxSamples;
}

void CrystalViewWidget::ModifyProperties(int which)
{
    int km = IsValidModel();
    if(km == -1)
        return;
    updateModify(-99);
    mPropDlg->setModel(km+1);
    mPropDlg->setTipo(which);
    int code = 0;
    int selbond = 1;
    int whichModel = km + 1;
    int nlegs = ModelInfo[whichModel-1].nlegs_au;
    int natoms = ModelInfo[whichModel-1].nats_au;
    if(mPropDlg->ModelEdit != nullptr)
        delete mPropDlg->ModelEdit;
    mPropDlg->ModelEdit = ModelInfo[whichModel-1].Clone();
    switch (which)
    {
    case 0:
    {
        atom_type *atml = new atom_type [natoms]();
        jav_edit_structure_(&code, atml, &natoms, &whichModel);
        mPropDlg->ModelEdit->nats = natoms;
        mPropDlg->ModelEdit->FillAtoms(atml);
    }
    break;
    case 1:
    {
        f_bond_type *legm = new f_bond_type [nlegs]();
        jav_edit_bond_properties_(&code, legm, &whichModel, &selbond);
        mPropDlg->ModelEdit->FillLegs(&nlegs, &nlegs, legm, true);
        mPropDlg->ModelEdit->CopyAtoms(ModelInfo[whichModel-1].atm_au,
                                       ModelInfo[whichModel-1].nats_au);
    }
    break;
    case 2:
    {
        getangle_(&whichModel);
        int nangs = ModelInfo[whichModel-1].nangbonds;
        jav_anglebond_type *angs = new jav_anglebond_type [nangs]();
        int angolo = 1;
        jav_edit_angle_properties_(&code, angs, &whichModel, &angolo);
        mPropDlg->ModelEdit->CopyAtoms(ModelInfo[whichModel-1].atm_au,
                                       ModelInfo[whichModel-1].nats_au);
        mPropDlg->ModelEdit->CopyAtomsAU(ModelInfo[whichModel-1].atm_au,
                                         ModelInfo[whichModel-1].nats_au);
        mPropDlg->ModelEdit->CopySpecies(ModelInfo[whichModel-1].LabSpec,
                                         ModelInfo[whichModel-1].numSpec);
        mPropDlg->ModelEdit->FillAngs(nangs, ModelInfo[whichModel-1].anglebonds);
    }
    break;
    case 3:
    {
        gettorsion_(&whichModel);
        int ntorsions = ModelInfo[whichModel-1].ntorsions;
        jav_torsion_type ang = ModelInfo[whichModel-1].torsions[0];
        float tval = 0;
        jav_edit_torsion_properties_(&code, &ang, &tval, &whichModel);
        mPropDlg->ModelEdit->CopyAtoms(ModelInfo[whichModel-1].atm_au,
                                       ModelInfo[whichModel-1].nats_au);
        mPropDlg->ModelEdit->CopyAtomsAU(ModelInfo[whichModel-1].atm_au,
                                         ModelInfo[whichModel-1].nats_au);
        mPropDlg->ModelEdit->CopySpecies(ModelInfo[whichModel-1].LabSpec,
                                         ModelInfo[whichModel-1].numSpec);
        mPropDlg->ModelEdit->FillTors(ntorsions, ModelInfo[whichModel-1].torsions);
    }
    default:
        break;
    }
    mPropDlg->show();
}

void CrystalViewWidget::importFromSmiles(int ifAdd)
{
    bool ok;
    QString title = ifAdd ? "Add SMILES" : "Import SMILES";
    static QString sinput;
    QString text = QInputDialog::getText(this, title,
                                         "Enter SMILES fragment:", QLineEdit::Normal,
                                         sinput,&ok);
    if (ok && !text.isEmpty()){
        sinput = text;

        int nptra = ifAdd, nptrb = 0, spec = 0;
        float thr = 0;
        int code = 35;
        int wm = 0;
        QApplication::setOverrideCursor(Qt::WaitCursor);
        (void) jav_modify(nullptr, &nptra, nullptr, &nptrb, &code, &thr, wm, &spec, text.toStdString().c_str(), (size_t)text.length());
        QApplication::restoreOverrideCursor();
    }
}

void CrystalViewWidget::exportFile(const QString &fileName, int tipo)
{
    const char *cStr;
    QByteArray inBytes = fileName.toUtf8();
    cStr = inBytes.constData();
    size_t sz = strlen(cStr);

    int which = 1;
    int ifD = 0;
    if(tipo >= 1000)
    {
        tipo -= 1000;
        ifD = 1;
    }
    jav_export_structure_(cStr, &tipo, &which, &ifD, sz);
}

CrystalViewMenu *CrystalViewWidget::menu()
{
    return mainWindow->getCrystalViewMenu();
}

#if defined(JAV_WIDGET)
extern "C" void get_info_fromc_(int vetinfo[], int *wm)
{
    for(int i=0; i<9;i++)
        vetinfo[i] = 0;
    int whichModel = *wm - 1;
    if(whichModel < 0)
        return;
    for(int i=0; i<9;i++)
        vetinfo[i] = CRYSTALVIEW->GetInfo(i, whichModel);
}

extern "C" void set_undoredo_(int *but, int *stato, char *str=nullptr)
{
    bool bstato = (*stato != 0);
    CRYSTALVIEW->SetUndoRedo(*but, bstato, str);
}

extern "C" void passamodello_(atom_type *atml, f_bond_type *legm,
                              int *nats, int *nats_au, int *nlegs, int *nlegs_au, element_type *celem, int *nelem,
                              int *icent, int *whichModel, int *nmodelli,
                              int *ifResize)
{
    Q_UNUSED(icent);
    int ifPDB = 1;
    bool ifNewM = true;
    Properties::SetActiveModel(1); //active model

    CRYSTALVIEW->PassaModello(atml, legm, nats, nats_au, nlegs, nlegs_au,
                              celem, nelem, whichModel, nmodelli, &ifPDB, nullptr, &ifNewM, *ifResize);
}

extern "C" void set_usedefaultb_(float B[3][3])
{
    if(CRYSTALVIEW->IsPreviewMode())
        return;
    Properties::set_usedefaultb(B);
}

extern "C" void jav_draw()
{
    CRYSTALVIEW->Draw();
}

extern "C" void passapoliedri_(int *npoly, polyhedron_type *poly)
{
    int whichModel = CRYSTALVIEW->IsValidModel();
    if(whichModel == -1)
        return;
    auto md = CRYSTALVIEW->GetModel(whichModel+1);
    md->FillPolyhedra(*npoly, poly);
    md->FillPolyhedraStyles();
    //    set_values_for_polyhedra(dlg, gl);
}

extern "C" void jav_passa_legami_(f_bond_type *legm, int *nleg, int *icode)
{
    int wm = Properties::GetActiveModel();
    auto md = CRYSTALVIEW->GetModel(wm);

    switch(*icode)
    {
    case 1:
        md->nlegsH = *nleg;
        if(md->legsH != nullptr)
        {
            delete [] md->legsH;
            md->legsH = nullptr;
        }
        if(*nleg > 0)
        {
            size_t nlegsH = static_cast<size_t>(md->nlegsH);
            md->legsH = new jav_bond_type [nlegsH]();
            for (size_t i=0;i<nlegsH; i++)
            {
                md->legsH[i].n1 = legm[i].n1;
                int n2 = abs(legm[i].n2);
                md->legsH[i].n2 = legm[i].n2;
                md->legsH[i].dist = legm[i].dist;
                md->legsH[i].ordleg = legm[i].ord;
                md->legsH[i].selected = false;
                // hang atom?
                if(Properties::ifDelHang)
                {
                    if(legm[i].ord == 1)
                        md->atm[n2-1].hidden = true;
                }
            }
        }
        break;
    case 2:
        md->nlegsW = *nleg;
        if(md->legsW != nullptr)
        {
            delete [] md->legsW;
            md->legsW = nullptr;
        }
        if(*nleg > 0)
        {
            size_t nlegsW = static_cast<size_t>(md->nlegsW);
            md->legsW = new jav_bond_type [nlegsW]();
            for (size_t i=0;i<nlegsW; i++)
            {
                md->legsW[i].n1 = legm[i].n1;
                int n2 = abs(legm[i].n2);
                md->legsW[i].n2 = legm[i].n2;
                md->legsW[i].dist = legm[i].dist;
                md->legsW[i].ordleg = legm[i].ord; // 06082018
                md->legsW[i].selected = false;
                // hang atom?
                if(Properties::ifDelHang)
                {
                    //                 if(legm[i].n2 < 0)
                    if(legm[i].ord == 1)
                        md->atm[n2-1].hidden = true;
                }
            }
        }
        break;
    }
    CRYSTALVIEW->UpdateInfoAtoms();
    CRYSTALVIEW->Draw();
}

extern "C" void get_packing_info_(float klevelv[], int *kfitopt, int *kmod)
{
    int whichModel = *kmod;
    if(whichModel < 1)
        return;
    auto md = CRYSTALVIEW->GetModel(whichModel);
    for(int i=0; i<3; i++)
        klevelv[i] = md->klevel[i];
    *kfitopt = md->FitOpt+1;
}

extern "C" void setname_(char *nome, int *kmod)
{
    int whichModel = *kmod;
    if((whichModel < 1) || (whichModel > MAX_NUM_MODELS))
        return;
    auto md = CRYSTALVIEW->GetModel(whichModel);
    if(md->created)
    {
        md->modname = QFileInfo(nome).baseName().toStdString();
        //setModelName(whichModel);
        CRYSTALVIEW->updateMenuDeleteModel();
    }
}

extern "C" void passaangle_(int *wM, jav_anglebond_type *angle, int *nangles)
{
    int k = *wM;
    auto md = CRYSTALVIEW->GetModel(k);
    md->FillAngs(*nangles, angle);
}

extern "C" void passatorsion_(int *wM, jav_torsion_type *torsion, int *ntorsions)
{
    int k = *wM;
    auto md = CRYSTALVIEW->GetModel(k);
    md->FillTors(*ntorsions, torsion);
}

extern "C" void set_title_(char *title)
{
    MainWindow *mw = getMainWindow();
    mw->setCrystalDockTitle(title);
}

extern "C" void setmodify_(int *active)
{
    Properties::defaultModify = *active;
    CRYSTALVIEW->updateModify(Properties::defaultModify);
}

extern "C" void cleardistang_()
{
    for(int k=0; k<Properties::nModels; k++)
    {
        auto md = CRYSTALVIEW->GetModel(k+1);
        for(int i=0; i<md->nats; i++)
            if(md->atm[i].drawLabel < 2)
                md->atm[i].drawLabel = 0;
    }
    Properties::DistAng_vet.clear();
}

extern "C" void set_cell(int ifC)
{
    Properties::ifCell = (ifC != 0);
    CRYSTALVIEW->aggiornaProperties();
    CRYSTALVIEW->SetCell(false);
}

extern "C" void set_labels(int action, int which)
{
    auto md = CRYSTALVIEW->GetModel(which);
    if(md->created)
    {
        md->ifLabels = (action != 0);
        CRYSTALVIEW->aggiornaProperties(which);

        //update action Labels_On_Off
        MainWindow *mainWindow = getMainWindow();
        mainWindow->getCrystalViewMenu()->actionLabels_On_Off->setChecked(action != 0);
    }
}

extern "C" void set_polyhedra_button(int action)
{
    MainWindow *mainWindow = getMainWindow();
    bool ifPoly = mainWindow->getCrystalViewMenu()->actionPolyhedra->isChecked();

    bool actionPoly = (action != 0);
    if(actionPoly != ifPoly) {
        mainWindow->getCrystalViewMenu()->actionPolyhedra->trigger();
    }
}

extern "C" void set_grow_fragment_button(int action)
{
    MainWindow *mainWindow = getMainWindow();
    bool ifGrow = mainWindow->getCrystalViewMenu()->actionGrow_Fragments->isChecked();

    bool actionGrow = (action != 0);
    if(actionGrow != ifGrow) {
        mainWindow->getCrystalViewMenu()->actionGrow_Fragments->trigger();
    }
}

extern "C" void set_contacts_button(int action)
{
    bool actionContacts = (action != 0);

    MainWindow *mainWindow = getMainWindow();
    bool ifContacts = mainWindow->getCrystalViewMenu()->actionShort_Contacts->isChecked();
    if(actionContacts != ifContacts) {
        mainWindow->getCrystalViewMenu()->actionShort_Contacts->trigger();
    }

    ifContacts = mainWindow->getCrystalViewMenu()->actionHydrogen_Bonds->isChecked();
    if(actionContacts != ifContacts) {
        mainWindow->getCrystalViewMenu()->actionHydrogen_Bonds->trigger();
    }

    ifContacts = mainWindow->getCrystalViewMenu()->actionDelete_Hanging_Contacts->isChecked();
    if(actionContacts != ifContacts) {
        mainWindow->getCrystalViewMenu()->actionDelete_Hanging_Contacts->trigger();
    }
}

extern "C" void set_symmetry_button(int action)
{
    MainWindow *mainWindow = getMainWindow();
    bool ifSymmetry = mainWindow->getCrystalViewMenu()->actionPacking->isChecked();

    bool actionSymmetry = (action != 0);
    if(actionSymmetry != ifSymmetry) {
        mainWindow->getCrystalViewMenu()->actionPacking->trigger();
    }
}

extern "C" void put_found_(int *kfound, int *vat, float *vdist, op_type *vsym)
{
    CRYSTALVIEW->updateSphereList(*kfound, vat, vdist, vsym);
}

extern "C" void end_centroids_()
{
    int whichModel = Properties::GetActiveModel() - 1;
    if(whichModel == -1)
        return;
    CRYSTALVIEW->FillCentroidsDlg();
    CRYSTALVIEW->RiNormalizza();
    CRYSTALVIEW->Draw();
}

extern "C" void passa_centroid_(int *ind, float xyz[3], int *natoms, int *at)
{
    Q_UNUSED(ind);
    CRYSTALVIEW->AddCentroids(xyz, *natoms, at);
}

extern "C" void start_centroids_()
{
    int whichModel = Properties::GetActiveModel() - 1;
    if(whichModel == -1)
        return;
    CRYSTALVIEW->ClearCentroids(whichModel);
}

extern "C" void open_glwindow()
{
    MainWindow *mw = getMainWindow();
    mw->showCrystalDock();
    //CRYSTALVIEW->setMinimumWidth(0);
    //mw->resizeMyDocks();
}

extern "C" int isopened_glwindow_()
{
    return (CRYSTALVIEW != nullptr && CRYSTALVIEW->isVisible());
}

CrystalViewWidget *getJav()
{
    return CRYSTALVIEW;
}
#endif
