#ifndef GLOBALOPTDIALOG_H
#define GLOBALOPTDIALOG_H

#include "solutionsdialog.h"
#include <QDialog>
#include <QCheckBox>
#include <QAbstractButton>

class MainWindow;

typedef struct {
    float res;
    float resmin;
    float resmax;
    int cftype;
    int nrun;
    int seed;
    int algo;
    int cfvet[8];
    int ncfvet;
    int nexport;
} c_gopt_info_type;

typedef struct {
  float t_ini;       // Initial temperature
  float temp_fact;   // Temperature reduction factor
  int   nt;          // Number of iterations before temperature reduction
  int   autoti;      // calcolo automatico t_ini?
  int   autont;      // calcolo automatico nt?
         } sa_conditions;

typedef struct {
  int   niter;       // number of iterations
  int   autont;      // calcolo automatico di niter
         } bb_bc_Conditions;

namespace Ui {
class GlobalOptDialog;
}

class GlobalOptDialog : public QDialog
{
    Q_OBJECT

public:    
    explicit GlobalOptDialog(MainWindow *mw);
    ~GlobalOptDialog();
    int setDialog(bool ifButton);
    void waitSignalIsEmitted();

public slots:
    void reject() override;

private slots:
    void on_resDoubleSpinBox_valueChanged(double arg1);
    void on_autoCyclesCheckBox_clicked(bool checked);
    void on_autoMovesCheckBox_clicked(bool checked);
    void on_autoTempCheckBox_clicked(bool checked);
    void on_buttonBox_clicked(QAbstractButton *button);
    void on_randomizePushButton_clicked();
    void on_pushButton_clicked();


private:
    enum algotypes {SA_LOCAL = 1, BBBC};
    Ui::GlobalOptDialog *ui;
    MainWindow *mWindow;
    c_gopt_info_type ginfo;
    sa_conditions SACond;
    bb_bc_Conditions BBCond;
    int numPhase;
    static const QStringList CFSTRING;

    SolutionsDialog *solDialog;
    int myCheckState(const QCheckBox *checkBox);
    void setConditionsTab();
    void setPhaseTab();
    void addPhaseTab(const QString &phaseName = "Crystal");
    void removePhaseTab();
    void getConditionsTab();
    void setParameterNumber(int add);
    void updatePhaseParameters();
};

#endif // GLOBALOPTDIALOG_H
