 MODULE indexing_interop

 use iso_c_binding

 implicit none 

   type, bind(C) :: c_index_settings
     integer(c_int)               :: nlines           ! number of lines used for searching
     integer(c_int), dimension(6) :: sys              ! crystal system tested
     real(c_float)                :: abcmax           ! min, max value on cell dimensions
     real(c_float)                :: betamin, betamax ! min, max beta angle for monoclinic cell
     real(c_float)                :: volmin, volmax   ! min, max volume in ang**3
     real(c_float)                :: mol_weight       ! molecular weight
     real(c_float)                :: dens             ! measured density
     real(c_float)                :: delden           ! absolute error on measured density
     real(c_float)                :: eps              ! peak position error
     integer(c_int)               :: zero_s           ! estimate zero point
     integer(c_int)               :: zero_ref         ! zero-point refinement
     real(c_float)                :: zero             ! experimental zero-point
     integer(c_int)               :: n_imp            ! max impurity lines
     integer(c_int)               :: search_type      ! search type: option (dicvol), ngrid (McMaille)
     real(c_float)                :: fom              ! Minimum figure-of-merit
     real(c_float)                :: width            ! Windth of the peak (~2*FWHM) (McMaille only)
   end type c_index_settings

   type, bind(C) :: c_cell_info_type
     real(c_float), dimension(6)      :: par               ! cell parameters
     real(c_float), dimension(6)      :: sd          ! st. dev. on parameters
     real(c_float)                    :: vol               ! cell volume
     real(c_float)                    :: m20      ! m20
     real(c_float)                    :: alt_fom      ! alternative f.o.m
     real(c_float)                    :: mcm20        ! f.o.m used in mcmaille 
     real(c_float)                    :: zshift      ! zero shift
     integer(c_int)                   :: nund       ! num. of unindexed lines
     character(c_char), dimension(3)  :: sys               ! crystal system
     character(c_char), dimension(21) :: syminfo           ! additional symmetry information
     integer(c_int)                   :: pcode             ! indexing program code
   end type c_cell_info_type


   private :: cast_settings_to_c, cast_settings_to_f

 contains

   elemental function cast_settings_to_c(setting) result(c_setting)
   use indexing, only: index_settings
   type(index_settings), intent(in) :: setting
   type(c_index_settings)           :: c_setting
!
   c_setting%nlines      = int(setting%nlines,c_int)
   c_setting%sys         = int(setting%sys,c_int)
   c_setting%abcmax      = real(setting%abcmax,c_float)
   c_setting%betamin     = real(setting%betamin,c_float)
   c_setting%betamax     = real(setting%betamax,c_float)
   c_setting%volmin      = real(setting%volmin,c_float)
   c_setting%volmax      = real(setting%volmax,c_float)
   c_setting%mol_weight  = real(setting%mol_weight,c_float) 
   c_setting%dens        = real(setting%dens,c_float)
   c_setting%delden      = real(setting%delden,c_float)
   c_setting%eps         = real(setting%eps,c_float)
   c_setting%zero_s      = int(setting%zero_s,c_int)
   c_setting%zero_ref    = int(setting%zero_ref,c_int)
   c_setting%zero        = real(setting%zero,c_float)
   c_setting%n_imp       = int(setting%n_imp,c_int)
   c_setting%search_type = int(setting%search_type,c_int)    
   c_setting%fom         = real(setting%fom,c_float)
   c_setting%width       = real(setting%width,c_float)            
!
   end function cast_settings_to_c

!-------------------------------------------------------------------------------------------------------  

   elemental function cast_settings_to_f(c_setting) result(setting)
   use indexing, only: index_settings
   type(c_index_settings), intent(in) :: c_setting
   type(index_settings)               :: setting
!
   setting%nlines      =  c_setting%nlines       
   setting%sys         =  c_setting%sys          
   setting%abcmax      =  c_setting%abcmax       
   setting%betamin     =  c_setting%betamin     
   setting%betamax     =  c_setting%betamax     
   setting%volmin      =  c_setting%volmin      
   setting%volmax      =  c_setting%volmax      
   setting%mol_weight  =  c_setting%mol_weight   
   setting%dens        =  c_setting%dens         
   setting%delden      =  c_setting%delden       
   setting%eps         =  c_setting%eps          
   setting%zero_s      =  c_setting%zero_s       
   setting%zero_ref    =  c_setting%zero_ref     
   setting%zero        =  c_setting%zero         
   setting%n_imp       =  c_setting%n_imp        
   setting%search_type =  c_setting%search_type 
   setting%fom         =  c_setting%fom          
   setting%width       =  c_setting%width        
!
   end function cast_settings_to_f

!-------------------------------------------------------------------------------------------------------  
   function cast_cell_to_c(cell) result(c_cell)
   use indexing, only: cell_info_type
   use strutil, only: toCString
   type(cell_info_type), intent(in) :: cell
   type(c_cell_info_type)           :: c_cell
!
   c_cell%par     =  real(cell%par,c_float)    
   c_cell%sd      =  real(cell%sd,c_float)     
   c_cell%vol     =  real(cell%vol,c_float)          
   c_cell%m20     =  real(cell%m20,c_float)          
   c_cell%alt_fom =  real(cell%alt_fom,c_float)      
   c_cell%mcm20   =  real(cell%mcm20,c_float)        
   c_cell%zshift  =  real(cell%zshift,c_float)       
   c_cell%nund    =  int(cell%nund,c_int)
   c_cell%sys(:len_trim(cell%sys)+1) = toCString(cell%sys)
   c_cell%syminfo(:len_trim(cell%syminfo)+1) = toCString(cell%syminfo)
   c_cell%pcode   =  int(cell%pcode,c_int)
!
   end function cast_cell_to_c

!-------------------------------------------------------------------------------------------------------  

   subroutine open_indexation_win(indx_set0, indx_set_forC) bind(C,name="open_indexation_win")
   USE indexing
   USE General, only: alambda
   USE enable_amb
   implicit none
   type(c_index_settings), dimension(3) :: indx_set0
   type(c_index_settings), dimension(3) :: indx_set_forC
!
!  Init default values
   call toolbar_set_state('continue'//char(0),0)  ! Disable 'Next'
   indx_set0(1) = cast_settings_to_c(init_index_settings('ntreor',alambda))
   indx_set0(2) = cast_settings_to_c(init_index_settings('dicvol',alambda))
   indx_set0(3) = cast_settings_to_c(init_index_settings('mcmaille',alambda))
   indx_set_forC(1:3) = cast_settings_to_c(indx_set(1:3))
!
   call abilita_tasti('indexwin',0)
   call data_for_index()
!
   end subroutine open_indexation_win

!-------------------------------------------------------------------------------------------------------  

   subroutine finish_indexC(indx_set_fromC, sel) bind(C,name="finish_indexC")
   USE indexing
   type(c_index_settings), dimension(3), intent(in) :: indx_set_fromC
   logical(c_bool), dimension(3), intent(in)        :: sel
!
   call finish_index(cast_settings_to_f(indx_set_fromC),logical(sel))
!
   end subroutine finish_indexC

!-------------------------------------------------------------------------------------------------------  

   subroutine export_dicvolC() bind(C,name="export_dicvolC")
!
!  Create input file for dicvol
   USE peak_mod, only: pkind
   USE General, only:StructureName, alambda
   USE indexing

   call export_dicvol('dicvol_inp',pkind%getx(),StructureName,alambda,indx_set(DICVOL_PROG))

   end subroutine export_dicvolC

!-------------------------------------------------------------------------------------------------------  

   subroutine export_mcmailleC() bind(C,name="export_mcmailleC")
!
!  Create input file for mcmaille
   USE peak_mod, only: pkind
   USE General, only:StructureName, alambda
   USE indexing

   call export_mcmaille('mcmaille.dat',pkind%getx(),pkind%gety(),StructureName,alambda,indx_set(MCMAILLE_PROG))

   end subroutine export_mcmailleC

!-------------------------------------------------------------------------------------------------------  

   subroutine LoadPeaksC(filename, length, tipo, ier) bind(C,name="LoadPeaksC")
   use strutil
   use iso_c_binding, only: c_char, c_int
   character(c_char), intent(in)      :: filename(*)
   integer(c_int), intent(in), value  :: length
   integer(c_int), intent(in), value  :: tipo
   integer(c_int), intent(out)        :: ier
   character(len=:), allocatable      :: filenam

   filenam = toFortranString(filename,length)
   call LoadPeaks(filenam,tipo,ier)

   end subroutine LoadPeaksC

!-------------------------------------------------------------------------------------------------------  

   subroutine SavePeaksC(filename, length, tipo) bind(C,name="SavePeaksC")
   use strutil
   use iso_c_binding, only: c_char, c_int
   character(c_char), intent(in)      :: filename(*)
   integer(c_int), intent(in), value  :: length
   integer(c_int), intent(in), value  :: tipo
   character(len=:), allocatable      :: filenam

   filenam = toFortranString(filename,length)

   call SavePeaks(filenam,tipo)
 
   end subroutine SavePeaksC 

 END MODULE indexing_interop 
