#ifndef INDEXINGDIALOG_H
#define INDEXINGDIALOG_H

#include <QDialog>
#include <QAbstractButton>

class MainWindow;

typedef struct {
     int    nlines;           // number of lines used for searching
     int    syst[6];          // crystal system tested
     float  abcmax;           // min, max value on cell dimensions
     float  betamin, betamax; // min, max beta angle for monoclinic cell
     float  volmin, volmax;   // min, max volume in ang**3
     float  mol_weight;       // molecular weight
     float  dens;             // measured density
     float  delden;           // absolute error on measured density
     float  eps;              // peak position error
     int    zero_s;           // estimate zero point
     int    zero_ref;         // zero-point refinement
     float  zero;             // experimental zero-point
     int    n_imp;            // max impurity lines
     int    search_type;      // search type: option (dicvol), ngrid (McMaille)
     float  fom;              // Minimum figure-of-merit
     float  width;            // Windth of the peak (~2*FWHM) (McMaille only)
                } index_settings;

namespace Ui {
class IndexingDialog;
}

class IndexingDialog : public QDialog
{
    Q_OBJECT

public:
    explicit IndexingDialog(MainWindow *mw);
    ~IndexingDialog();

    enum ProgramType {NTREOR09, DICVOL, MCMAILLE};
    void setIndexing(ProgramType prog);
    void setOutFileName(const QString &fileName, ProgramType prog);    
    QString getOutFileName(ProgramType prog);

private slots:
    void onCheckBoxNtreor09Toggled(bool checked);
    void onCheckBoxDicvolToggled(bool checked);
    void onCheckBoxMcmailleToggled(bool checked);
    void onButtonBoxClicked(QAbstractButton *button);
    void enableBlackBoxMode(bool enabled);
    void disableBlackBoxMode(bool enabled);
    void reject() override;

private:
    Ui::IndexingDialog *ui;
    MainWindow *mWindow;
    index_settings defaultSettings[3], currentSettings[3], savedSettings[3];    
    QString outFileName[3];

    void setWidgetState(const index_settings indSettings[]);
    void getWidgetState(index_settings indSettings[]);
    void uncheckPrograms();
    bool isProgramSelected();
};

#endif // INDEXINGDIALOG_H
