#include "myfiledialog.h"
#include <QPushButton>
#include <QGridLayout>
#include <QLabel>
#include <QDebug>
#include <QList>
#include <QMessageBox>

MyFileDialog::MyFileDialog(QWidget *parent) : QFileDialog(parent)
{
    setOption(QFileDialog::DontUseNativeDialog);
    setFileMode(QFileDialog::ExistingFile);
    outputFile =  "";

    QGridLayout* mainLayout = dynamic_cast <QGridLayout*>(this->layout());
    if( !mainLayout){
        qDebug()<<"mainLayout is NULL";
    }else{
        QList<QLineEdit *> list = findChildren<QLineEdit *>();
        if (!list.empty())
            inpFile = list.at(0);
        QHBoxLayout *hbl =new QHBoxLayout(0);
        editButton =new QPushButton(QString("Edit"));
        hbl->addWidget(editButton);
        int num_rows = mainLayout->rowCount();
        int num_cols = mainLayout->columnCount();
        mainLayout->addLayout(hbl, num_rows, num_cols-1);
        editButton->setEnabled(false);
        hbl =new QHBoxLayout(0);
        QLabel *label = new QLabel("Output File:");
        hbl->addWidget(label);
        mainLayout->addLayout(hbl, num_rows, 0, 1, 1); //, 1, -1);
        connect(editButton, SIGNAL(pressed()), this, SLOT(onEdit()));
        hbl =new QHBoxLayout(0);
        outFile = new QLineEdit(this);
        hbl->addWidget(outFile);
        mainLayout->addLayout(hbl, num_rows, 1, num_rows-2, 1); //, 1, -1);
        connect(this, SIGNAL(currentChanged(QString)), this, SLOT(onCurrentChanged(QString)));
        connect(inpFile, SIGNAL(textChanged(QString)), this, SLOT(onTextChanged(QString)));
        connect(outFile, SIGNAL(textChanged(QString)), this, SLOT(onOutChanged(QString)));
    }
}

void MyFileDialog::onEdit()
{
//    QString fileName = selectedFiles()[0];
    QString fileName = selectedFiles().at(0);
    if (!fileName.isEmpty())
    {
        QFileInfo fi(fileName);
        if(!fi.isFile())
            return;
        emit editSelected(fileName, this);
    }
}

void MyFileDialog::onCurrentChanged(QString current)
{
    QString str = QFileInfo(current).baseName() + ".out";
    outFile->setText(str);
    outputFile = str;
    editButton->setEnabled(true);
}


void MyFileDialog::onTextChanged(QString current)
{
    QString path = directory().absolutePath() + "/" + current;
    if (QFileInfo::exists(path))
    {
        QString str = QFileInfo(current).baseName() + ".out";
        outFile->setText(str);
        outputFile = str;
        editButton->setEnabled(true);        
    }
    else
    {
        outFile->setText("");
        outputFile = "";
        editButton->setEnabled(false);
    }
}

void MyFileDialog::onOutChanged(QString current)
{
    outputFile = current;
}


void MyFileDialog::accept()
{
    QString path = directory().absolutePath() + "/" + outputFile;
    if(QFileInfo::exists(path))
    {
        int ret = QMessageBox::warning(this, "Output file",
                                       outputFile + " already exists.\nDo you want to overwrite it?",
                                       QMessageBox::Yes | QMessageBox::No, QMessageBox::No);
        if (ret == QMessageBox::No)
            return;
    }

    QFileDialog::accept();
}
