#include <QMessageBox>
#include "powderpatternsettings.h"
#include "ui_powderpatternsettings.h"

#include <QDebug>

PowderPatternSettings::PowderPatternSettings(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::PowderPatternSettings)
{
    ui->setupUi(this);

//#ifdef Q_OS_MACOS
//    ui->formLayout->setFormAlignment(Qt::AlignLeft | Qt::AlignTop);
//    ui->formLayout->setLabelAlignment(Qt::AlignLeft | Qt::AlignVCenter);
//#endif

    //Align label_1 to waveLWidget
    ui->label_1->setMinimumSize(ui->waveL1->columnSize());
}

PowderPatternSettings::~PowderPatternSettings()
{
    delete ui;    
}

void PowderPatternSettings::FillForm(int wm, xpd_option_type *xpd)
{
    xpdG = xpd;
    m_Model = wm;
    ui->waveL1->setComboAsUserDefined(xpd->wave[0]);
    ui->waveL1->setSeconWave(xpd->nwave > 1 ? Qt::Checked : Qt::Unchecked, static_cast<double>(xpd->wave[1]), static_cast<double>(xpd->ratio[1]));
    //ui->waveL1->setComboIndex(0);
    //ui->waveL1->setUserWave(xpd->wave[0]);
    //ui->wavel->setText(QString::number(xpd->wave[0], 'f', 5));
    //ui->useWavel2->setChecked(xpd->nwave > 1);
    //ui->wavel2->setText(QString::number(xpd->wave[1], 'f', 5));
    //ui->ratio->setText(QString::number(xpd->ratio[1], 'f', 5));
    ui->tmin->setValue(xpd->ttmin);
    ui->tmax->setValue(xpd->ttmax);
    ui->step->setValue(xpd->ttstep);
    ui->fwhm->setText(QString::number(xpd->fwhm, 'f', 5));
    ui->scale->setText(QString::number(xpd->scal, 'f', 5));
}

bool PowderPatternSettings::ReadForm(xpd_option_type *xpd)
{
    QString msg;
    if(!ui->waveL1->isValidInput(msg))
    {
        QMessageBox::warning(this, tr("Error"),msg,QMessageBox::Cancel);
        return false;
    }

    xpd->nwave = ui->waveL1->nWaves();
    xpd->wave[0] = ui->waveL1->wave1().toFloat();
    xpd->ratio[0] = 1;
    //xpd->wave[0] = ui->wavel->text().toFloat();
    //xpd->nwave = (ui->useWavel2->isChecked())?2:1;
    if(xpd->nwave > 1)
    {
        xpd->wave[1] = ui->waveL1->wave2().toFloat();
        xpd->ratio[1] = ui->waveL1->ratio().toFloat();
        //xpd->wave[1] = ui->wavel2->text().toFloat();
        //xpd->ratio[1] = ui->ratio->text().toFloat();
    }
    xpd->ttmin = ui->tmin->value();
    xpd->ttmax = ui->tmax->value();
    xpd->ttstep = ui->step->value();
    xpd->fwhm = ui->fwhm->text().toFloat();
    xpd->scal = ui->scale->text().toFloat();
    return true;
}

void PowderPatternSettings::on_applyButton_clicked()
{
    if(ReadForm(xpdG))
        jav_powder_pattern_(&m_Model, xpdG);
}

void PowderPatternSettings::on_closeButton_clicked()
{
    QDialog::reject();
}

void PowderPatternSettings::on_okButton_clicked()
{
    if(ReadForm(xpdG))
    {
        jav_powder_pattern_(&m_Model, xpdG);
        QDialog::accept();
    }
}
