#include "preforientationwidget.h"
#include "ui_preforientationwidget.h"

#include <QIntValidator>

PrefOrientationWidget::PrefOrientationWidget(QWidget *parent) :
    QWidget(parent),
    ui(new Ui::PrefOrientationWidget)
{
    ui->setupUi(this);

    QIntValidator *hklValidator = new QIntValidator(this);
    ui->hLineEdit->setValidator(hklValidator);
    ui->kLineEdit->setValidator(hklValidator);
    ui->lLineEdit->setValidator(hklValidator);
    QDoubleValidator *gValidator = new QDoubleValidator(0.0, 100.0, 8, this);
    ui->gfactorLineEdit->setValidator(gValidator);
}

PrefOrientationWidget::~PrefOrientationWidget()
{
    delete ui;
}

void PrefOrientationWidget::setWidget(int hkl[], float gpar, int code)
{
    for (int i=0; i < 3; i++) hklSav[i] = hkl[i];
    gparSav = gpar;

    ui->hLineEdit->setText(QString::number(hkl[0]));
    ui->kLineEdit->setText(QString::number(hkl[1]));
    ui->lLineEdit->setText(QString::number(hkl[2]));
    ui->gfactorLineEdit->setText(QString::number(gpar,'f',4));
    ui->poCheckBox->setChecked(code);
}

void PrefOrientationWidget::getPO(int hkl[], float &gpar, int &code)
{
    hkl[0] = ui->hLineEdit->text().toInt();
    hkl[1] = ui->kLineEdit->text().toInt();
    hkl[2] = ui->lLineEdit->text().toInt();
    gpar = ui->gfactorLineEdit->text().toFloat();
    code = ui->poCheckBox->isChecked() ? 1 : 0;

    //reset changes
    for (int i = 0; i < 3; i++) hklSav[i] = hkl[i];
    gparSav = gpar;
}

bool PrefOrientationWidget::isChecked()
{
    return ui->poCheckBox->isChecked();
}

bool PrefOrientationWidget::hasValueChanged()
{
    if (hklSav[0] != ui->hLineEdit->text().toInt()) return true;
    if (hklSav[1] != ui->kLineEdit->text().toInt()) return true;
    if (hklSav[2] != ui->lLineEdit->text().toInt()) return true;

    if (gparSav != ui->gfactorLineEdit->text().toFloat()) return true;

    return false;
}

void PrefOrientationWidget::on_poCheckBox_clicked(bool checked)
{
    emit poCheckBoxClicked(checked);
}
