module riet_typedef
   
   use atom_basic, only: atom_type, SIZELAB
   use iso_c_binding

   implicit none

   type, bind(C) :: crystal_t
!qt      character(len=300)       :: cname
!qt      real, dimension(6)       :: cell
!qt      integer, dimension(6)    :: cellcod
!qt      character(len=16)        :: spgname
      real                     :: scal
      integer                  :: scalcod
!qt      integer                  :: kopcorr = 0
!qt      integer, dimension(3)    :: hklpo
!qt      integer                  :: gcode
!qt      real                     :: gpar = 1.0
!qt      logical                  :: riding_model
!qt      real                     :: adp_factor = 1.2
   end type crystal_t

   type,  bind(C) :: dataset_t
     real(c_float)  :: zero=0.0
     real(c_float)  :: sdisp=0.0
     real(c_float)  :: stran=0.0
     integer(c_int) :: zcode,sdcode,stcode
     integer(c_int) :: bcode
   end type dataset_t

   type, bind(C) :: c_refine_condition_type
     integer(c_int)                  :: algo          
     integer(c_int)                  :: auto_coefb    
     integer(c_int)                  :: auto_prof     
     integer(c_int)                  :: auto_stru     
     integer(c_int)                  :: ncauto        
     integer(c_int)                  :: typewei       
     real(c_float)                   :: eps           
     integer(c_int)                  :: maxcy         
     integer(c_int)                  :: raction = 0   
     integer(c_int), dimension(5)    :: print_options
   end type c_refine_condition_type

   type, bind(C) :: c_atom_type
     character(c_char), dimension(SIZELAB+1) :: lab
     real(c_float), dimension(3)             :: xyz
     real(c_float)                           :: biso
     real(c_float)                           :: occ
     integer(c_int), dimension(5)            :: rcod
     integer(c_int)                          :: doc
     integer(c_int)                          :: kscatt
   end type c_atom_type

   type, bind(C) :: c_profile_function
     real(c_float), dimension(10)  :: par
     integer(c_int), dimension(10) :: rcod
   end type c_profile_function

end module riet_typedef

module rinterface
!
!  Procedures to interface refinement with GUI
!
   !use atom_basic, only: atom_type, SIZELAB
   !use iso_c_binding
   use riet_typedef
   use rrestr, only: restraint_type

   implicit none
   enum, bind(c)
     enumerator :: StartEndCycle=1, EachCycle, CorrelMatrix, LSQMatrix, CreateCIF
   end enum

   private cast_atom_to_c_v, cast_atom_to_c_s
   interface cast_atom_to_c
     module procedure cast_atom_to_c_v, cast_atom_to_c_s
   end interface

!qt   type(crystal_t), dimension(:), allocatable :: cryph
!qt   type(dataset_t), dimension(:), allocatable :: datas

   private :: new_crystal_t, numcryst
!   type(restraint_type), allocatable, private :: restraints_list(:)

contains

!qt   subroutine make_crystal_t()
!qt   use variables, only: cryst
!qt   use elements
!qt   use atom_type_util
!qt   use crystal_phase
!qt   integer :: nph
!qt!
!qt   call new_crystal_t(cryph,numphase(cryst))
!qt   do nph=1,numphase(cryst)
!qt      cryph(nph)%cname = cryst(nph)%cr_name
!qt      cryph(nph)%cell = cryst(nph)%get_cellpar()
!qt      cryph(nph)%cellcod = cryst(nph)%get_cellcod()
!qt      cryph(nph)%spgname = cryst(nph)%spg%symbol_xhm
!qt      cryph(nph)%scal = cryst(nph)%scal
!qt      cryph(nph)%scalcod = cryst(nph)%scode
!qt      cryph(nph)%riding_model = cryst(nph)%riding_model
!qt      cryph(nph)%adp_factor = cryst(nph)%adp_factor
!qt   enddo
!qt!
!qt   end subroutine make_crystal_t
!qt
!----------------------------------------------------------------------------------------------------
!qt
!qt   subroutine make_dataset_t()
!qt   USE variables, only: dataset
!qt   integer :: nd
!qt   call new_dataset_t(datas,1)
!qt   do nd=1,1
!qt      datas(nd)%zero = dataset(nd)%zero
!qt      datas(nd)%sdisp = dataset(nd)%sdisp
!qt      datas(nd)%stran = dataset(nd)%stran
!qt
!qt      datas(nd)%zcode = dataset(nd)%zcode
!qt      datas(nd)%sdcode = dataset(nd)%sdcode
!qt      datas(nd)%stcode = dataset(nd)%stcode
!qt
!qt      !datas(nd)%cond = dataset(nd)%cond
!qt      datas(nd)%bcode = dataset(nd)%bcode(1)
!qt   enddo
!qt   end subroutine make_dataset_t
!qt
!----------------------------------------------------------------------------------------------------

   function cast_crystal_to_c(fcrystal) result(ccryst)
   use crystal_phase
   type(crystal_phase_t), intent(in) :: fcrystal
   type(crystal_t)                   :: ccryst
!
   ccryst%scal = real(fcrystal%scal, c_float)
   ccryst%scalcod = int(fcrystal%scode, c_int)
!
   end function cast_crystal_to_c

!----------------------------------------------------------------------------------------------------

   subroutine cast_crystal_to_f(ccrystal,fcrystal)
   use crystal_phase
   type(crystal_t), intent(in)          :: ccrystal
   type(crystal_phase_t), intent(inout) :: fcrystal
!
   fcrystal%scal = real(ccrystal%scal)
   fcrystal%scode = int(ccrystal%scalcod)
!
   end subroutine cast_crystal_to_f

!----------------------------------------------------------------------------------------------------

   function cast_refcond_to_c(fcond) result(ccond)
   use progtype, only: refine_condition_type
   use strutil
   use arrayutil
   type(refine_condition_type), intent(in) :: fcond
   type(c_refine_condition_type)           :: ccond
!
   ccond%algo       = int(fcond%algo, c_int)       
   ccond%auto_coefb = logical_to_int(fcond%auto_coefb)
   ccond%auto_prof  = logical_to_int(fcond%auto_prof)
   ccond%auto_stru  = logical_to_int(fcond%auto_stru)
   ccond%ncauto     = int(fcond%ncauto, c_int)
   ccond%typewei    = int(fcond%typewei-1, c_int)    
   ccond%eps        = real(fcond%eps, c_float)
   ccond%maxcy      = int(fcond%maxcy, c_int)
   ccond%raction    = int(fcond%raction, c_int)
   ccond%print_options(StartEndCycle) = logical_to_int(index(fcond%sprint,'S')>0)
   ccond%print_options(EachCycle) = logical_to_int(index(fcond%sprint,'V')>0)
   ccond%print_options(CorrelMatrix) = logical_to_int(index(fcond%sprint,'C')>0)
   ccond%print_options(LSQMatrix) = logical_to_int(index(fcond%sprint,'M')>0)
   ccond%print_options(CreateCIF) = logical_to_int(index(fcond%sprint,'E')>0)
!
   end function cast_refcond_to_c

!----------------------------------------------------------------------------------------------------

   subroutine cast_refcond_to_f(ccond, fcond)
   use progtype, only: refine_condition_type
   use strutil
   use arrayutil
   type(c_refine_condition_type), intent(in)  :: ccond
   type(refine_condition_type), intent(inout) :: fcond
!
   fcond%algo       = int(ccond%algo)
   fcond%auto_coefb = int_to_logical(ccond%auto_coefb)
   fcond%auto_prof  = int_to_logical(ccond%auto_prof)
   fcond%auto_stru  = int_to_logical(ccond%auto_stru)
   fcond%ncauto     = int(ccond%ncauto)
   fcond%typewei    = int(ccond%typewei+1)
   fcond%eps        = real(ccond%eps)
   fcond%maxcy      = int(ccond%maxcy)
   fcond%raction    = int(ccond%raction)
   fcond%sprint = ' '
   if (ccond%print_options(StartEndCycle) == 1) fcond%sprint(StartEndCycle:StartEndCycle) = 'S'
   if (ccond%print_options(EachCycle) == 1) fcond%sprint(EachCycle:EachCycle) = 'V'
   if (ccond%print_options(CorrelMatrix) == 1) fcond%sprint(CorrelMatrix:CorrelMatrix) = 'C'
   if (ccond%print_options(LSQMatrix) == 1) fcond%sprint(LSQMatrix:LSQMatrix) = 'M'
   if (ccond%print_options(CreateCIF) == 1) fcond%sprint(CreateCIF:CreateCIF) = 'E'
!
   end subroutine cast_refcond_to_f

!----------------------------------------------------------------------------------------------------

   elemental function cast_dataset_to_c(fdat) result(cdat)
   use datasetmod, only: dataset_type
   type(dataset_type), intent(in) :: fdat
   type(dataset_t)                :: cdat
!
   cdat%zero   = real(fdat%zero, c_float)
   cdat%sdisp  = real(fdat%sdisp, c_float)
   cdat%stran  = real(fdat%stran, c_float)
   cdat%zcode  = int(fdat%zcode, c_int)
   cdat%sdcode = int(fdat%sdcode, c_int)
   cdat%stcode = int(fdat%stcode, c_int)
   cdat%bcode  = int(fdat%bcode(1), c_int)
!
   end function cast_dataset_to_c

!----------------------------------------------------------------------------------------------------

   subroutine cast_dataset_to_f(cdat, fdat)
   use datasetmod, only: dataset_type
   type(dataset_t), intent(in)       :: cdat
   type(dataset_type), intent(inout) :: fdat
!
   fdat%zero   = real(cdat%zero)
   fdat%sdisp  = real(cdat%sdisp)
   fdat%stran  = real(cdat%stran)
   fdat%zcode  = int(cdat%zcode)
   fdat%sdcode = int(cdat%sdcode)
   fdat%stcode = int(cdat%stcode)
   fdat%bcode(:)  = int(cdat%bcode)
!
   end subroutine cast_dataset_to_f

!----------------------------------------------------------------------------------------------------
 
   function cast_atom_to_c_s(f_atm) result(c_atm)
   use strutil
   use arrayutil
   type(atom_type), intent(in) :: f_atm
   type(c_atom_type)           :: c_atm
!
   c_atm%lab(:len_trim(f_atm%lab)+1) = toCString(f_atm%lab)
   c_atm%xyz = f_atm%xc
   c_atm%biso = f_atm%biso
   c_atm%occ = f_atm%ocry
   c_atm%rcod(:) = f_atm%rcod(:)
   c_atm%doc = logical_to_int(f_atm%doc)
   c_atm%kscatt = f_atm%kscatt()
!
   end function cast_atom_to_c_s

!----------------------------------------------------------------------------------------------------

   function cast_atom_to_c_v(f_atm) result(c_atm)
   use strutil
   type(atom_type), dimension(:), intent(in) :: f_atm
   type(c_atom_type), dimension(size(f_atm)) :: c_atm
   integer                                   :: i
!
   do i=1,size(f_atm)
      c_atm(i) = cast_atom_to_c_s(f_atm(i))
   enddo
!
   end function cast_atom_to_c_v

!----------------------------------------------------------------------------------------------------

   function cast_profile_function_to_c(fprof) result(cprof)
   use profile_function
   type(profile_function_t), intent(in) :: fprof
   type(c_profile_function)             :: cprof
!
   cprof%par = real(fprof%par%val,c_float)
   cprof%rcod = int(fprof%par%rcod, c_int)
!
   end function cast_profile_function_to_c

!----------------------------------------------------------------------------------------------------
 
   subroutine cast_profile_function_to_f(fprof, cprof, changed)
   use profile_function
   type(profile_function_t), intent(inout) :: fprof
   type(c_profile_function), intent(in)    :: cprof
   integer, intent(in)                     :: changed
!
   if (changed == 1) fprof%par%val = real(cprof%par)
   fprof%par%rcod = int(cprof%rcod)
!
   end subroutine cast_profile_function_to_f 

!----------------------------------------------------------------------------------------------------

   subroutine set_refcond(c_rcond) bind(C,name="set_refcond")
   USE RefinecomRef, only: rfcond
   type (c_refine_condition_type), intent(in) :: c_rcond   
!
   call cast_refcond_to_f(c_rcond,rfcond)  
!
   end subroutine set_refcond

!----------------------------------------------------------------------------------------------------

   subroutine get_dataset(cdat) bind(C,name="get_dataset")
   use variables, only: dataset
   type(dataset_t), intent(out) :: cdat
   cdat = cast_dataset_to_c(dataset(1))
   end subroutine get_dataset

!----------------------------------------------------------------------------------------------------

   subroutine set_dataset(cdat) bind(C,name="set_dataset")
   use variables, only: dataset
   type(dataset_t), intent(in) :: cdat
   call cast_dataset_to_f(cdat,dataset(1))
   end subroutine set_dataset

!----------------------------------------------------------------------------------------------------

   subroutine get_profile_functions(data_id, phase_id, typefun, cprof) bind(C,name="get_profile_functions")
   use variables, only: dataset
   use profile_function
   integer(c_int), intent(in), value                   :: data_id, phase_id
   integer(c_int), intent(out)                         :: typefun
   type(c_profile_function), dimension(*), intent(out) :: cprof
!
   typefun = dataset(data_id)%typefun(phase_id)
   cprof(PEARSON7+1) = cast_profile_function_to_c(dataset(data_id)%pear(phase_id))
   cprof(PVOIG+1)    = cast_profile_function_to_c(dataset(data_id)%pvoi(phase_id))
   cprof(TCHZ+1)     = cast_profile_function_to_c(dataset(data_id)%tch(phase_id))
!
   end subroutine get_profile_functions

!----------------------------------------------------------------------------------------------------

   subroutine set_profile_functions(data_id, phase_id, typefun, cprof, changed) bind(C,name="set_profile_functions")
   use variables, only: dataset
   use profile_function
   integer(c_int), intent(in), value                  :: data_id, phase_id
   integer(c_int), intent(in), value                  :: typefun
   type(c_profile_function), dimension(*), intent(in) :: cprof
   integer(c_int), dimension(*), intent(in)           :: changed
!
   dataset(data_id)%typefun(phase_id) = typefun
   call cast_profile_function_to_f(dataset(data_id)%pear(phase_id), cprof(PEARSON7+1), changed(PEARSON7+1))
   call cast_profile_function_to_f(dataset(data_id)%pvoi(phase_id), cprof(PVOIG+1), changed(PVOIG+1))
   call cast_profile_function_to_f(dataset(data_id)%tch(phase_id), cprof(TCHZ+1), changed(TCHZ+1))
!
   end subroutine set_profile_functions

!----------------------------------------------------------------------------------------------------

   function crystal_number() bind(C,name="crystal_number")
   use variables, only: cryst
   use crystal_phase
   integer(c_int) :: crystal_number
   crystal_number = numphase(cryst)
   end function crystal_number

!----------------------------------------------------------------------------------------------------

   function crystal_phase_name_len(idph) bind(C,name="crystal_phase_name_len")
   use variables, only: cryst
   integer(c_int), value :: idph
   integer(c_int)        :: crystal_phase_name_len
   crystal_phase_name_len = len_trim(cryst(idph)%cr_name)
   end function crystal_phase_name_len

!----------------------------------------------------------------------------------------------------

   subroutine crystal_phase_name(idph,phase_name) bind(C,name="crystal_phase_name")
   use variables, only: cryst
   use strutil
   integer, intent(in), value                :: idph
   character(kind=c_char,len=1), intent(out) :: phase_name(*)
   integer                                   :: len
!
   len = len_trim(cryst(idph)%cr_name)+1
   phase_name(:len) = toCString(cryst(idph)%cr_name)
!
   end subroutine crystal_phase_name

!----------------------------------------------------------------------------------------------------

   function crystal_atom_number(idph) bind(C,name="crystal_atom_number")
   use variables, only: cryst
   integer(c_int), intent(in), value :: idph
   integer(c_int)                    :: crystal_atom_number
   crystal_atom_number = cryst(idph)%natoms()
   end function crystal_atom_number

!----------------------------------------------------------------------------------------------------

   subroutine get_crystal_atoms(idph,c_atm) bind(C,name="get_crystal_atoms")
   use variables, only: cryst
   integer(c_int), intent(in), value :: idph
   type(c_atom_type), intent(out)    :: c_atm(*)
   integer                           :: nat
!
   nat = cryst(idph)%natoms()
   c_atm(:nat) = cast_atom_to_c(cryst(idph)%at(:))
!
   end subroutine get_crystal_atoms 

!----------------------------------------------------------------------------------------------------

   subroutine put_crystal_atoms(idph,c_atm,coord) bind(C,name="put_crystal_atoms")
   use variables, only: cryst
   use arrayutil
   integer(c_int), intent(in), value :: idph
   type(c_atom_type), intent(in)     :: c_atm(*)
   integer, intent(in), value        :: coord
   integer                           :: nat
   integer                           :: i
!
   nat = cryst(idph)%natoms()
!
!  Copy only rilevant information
   do i=1,nat
      cryst(idph)%at(i)%biso = c_atm(i)%biso
      cryst(idph)%at(i)%ocry = c_atm(i)%occ
      cryst(idph)%at(i)%rcod(1:5) = c_atm(i)%rcod(1:5)
      cryst(idph)%at(i)%doc =  int_to_logical(c_atm(i)%doc)
   enddo
   if (coord == 1) then
       do i=1,nat
          cryst(idph)%at(i)%xc = c_atm(i)%xyz
       enddo
   endif
!
   end subroutine put_crystal_atoms

!----------------------------------------------------------------------------------------------------

   subroutine put_crystal_atom_parameter(idph, idatom, idpar, pvalue)  &
   bind(C,name="put_crystal_atom_parameter")
   use variables, only: cryst, dataset
   use ilmviewmod, only: active_phase
   use rietv, only: setRInt, write_rietveld_messages, Count2
   use RefinecomRef, only: rfcond
   use view, only: vedinew
   use setref, only: set_rcode_atoms
   use general, only: is_anomalous
   integer(c_int), intent(in), value :: idph, idatom, idpar
   real(c_float), intent(in), value  :: pvalue
   logical                           :: update
   integer                           :: code
   enum, bind(c)
      enumerator :: ID_COL, LABEL_COL, X_COL, Y_COL, Z_COL, XYZREF_COL,  &
                    B_COL, BREF_COL, OCC_COL, OCCREF_COL
   end enum
!
   update = .false.
   select case(idpar)
      case (X_COL, Y_COL, Z_COL)
         cryst(idph)%at(idatom)%xc(idpar-X_COL+1) = pvalue
         update = .true.

      case (B_COL)
         cryst(idph)%at(idatom)%biso = pvalue
         update = .true.

      case (OCC_COL)
         cryst(idph)%at(idatom)%och = pvalue
         update = .true.

      case (XYZREF_COL)
         if (pvalue > 0) then
             code = 1
         else
             code = 0
         endif
         call set_rcode_atoms(cryst(idph)%at,cryst(idph)%cell,cryst(idph)%spg,cryst(idph)%constr,'XYZ',code,idatom)
    
      case (BREF_COL) 
         if (pvalue > 0) then
             code = 1
         else
             code = 0
         endif
         call set_rcode_atoms(cryst(idph)%at,cryst(idph)%cell,cryst(idph)%spg,cryst(idph)%constr,'B',code,idatom)

      case (OCCREF_COL) 
         if (pvalue > 0) then
             code = 1
         else
             code = 0
         endif
         call set_rcode_atoms(cryst(idph)%at,cryst(idph)%cell,cryst(idph)%spg,cryst(idph)%constr,'O',code,idatom)
   end select

   if (update) then
       call cryst(idph)%sfactor(dataset(1)%radtype,is_anomalous)
       call setRInt(cryst(idph),rfcond)
       call Count2(rfcond)

       call vedinew(8,rescale=0)
       call write_rietveld_messages(rfcond)

       if (active_phase == idph) then
           call show_model(idph)
       endif
   endif
!
   end subroutine put_crystal_atom_parameter 

!----------------------------------------------------------------------------------------------------

   subroutine get_crystal_data(idph,ccryst) bind(C,name="get_crystal_data")
   use variables, only: cryst
   integer(c_int), intent(in), value :: idph
   type(crystal_t), intent(out)      :: ccryst
!
   ccryst = cast_crystal_to_c(cryst(idph))
!
   end subroutine get_crystal_data

!----------------------------------------------------------------------------------------------------

   subroutine set_crystal_data(idph,ccryst) bind(C,name="set_crystal_data")
   use variables, only: cryst
   integer(c_int), intent(in), value  :: idph
   type(crystal_t), intent(in)        :: ccryst
!
   call  cast_crystal_to_f(ccryst,cryst(idph))
!
   end subroutine set_crystal_data

!----------------------------------------------------------------------------------------------------

   subroutine get_info_crystal(numph,natoms,nelem)
   use variables, only: cryst
   use crystal_phase
   use arrayutil
   integer, intent(out)                            :: numph
   integer, dimension(:), allocatable, intent(out) :: natoms,nelem
   integer                                         :: nph
   numph = numphase(cryst)
   call new_array(natoms,numph)
   call new_array(nelem,numph)
   do nph=1,numph
      natoms(nph) = cryst(nph)%natoms()
      nelem(nph) = cryst(nph)%numelem()
   enddo
   end subroutine get_info_crystal

!----------------------------------------------------------------------------------------------------

   subroutine get_info_po(idph, hkl, gpar, code) bind(C, name="get_info_po")
   use variables, only: cryst
   integer(c_int), intent(in), value         :: idph
   integer(c_int), dimension(3), intent(out) :: hkl
   real(c_float), intent(out)                :: gpar
   integer(c_int), intent(out)               :: code
!   
   hkl(:3) = cryst(idph)%hklpo(:)
   gpar = cryst(idph)%gpar
   code = cryst(idph)%gcode
!
   end subroutine get_info_po

!----------------------------------------------------------------------------------------------------

   subroutine put_info_po(idph, hkl, gpar, code) bind (C,name="put_info_po")
   use variables, only: cryst
   integer(c_int), intent(in), value        :: idph
   integer(c_int), dimension(3), intent(in) :: hkl
   real(c_float), intent(in), value         :: gpar
   integer(c_int), intent(in), value        :: code
!
   cryst(idph)%hklpo(:) = hkl(:) 
   cryst(idph)%gpar     = gpar    
   cryst(idph)%gcode    = code    
!
   end subroutine put_info_po

!----------------------------------------------------------------------------------------------------
!qt
!qt   subroutine print_crystal_t(kpr)
!qt   use elements
!qt   use atom_type_util
!qt   integer, intent(in) :: kpr
!qt   integer             :: nph
!qt!
!qt   do nph=1,numcryst(cryph)
!qt      write(kpr,'(a)') 'Name: '//trim(cryph(nph)%cname)
!qt      write(kpr,'(a,6f12.5)') 'Cell: ',cryph(nph)%cell(:)
!qt      write(kpr,'(a,6i12)')   'Code: ',cryph(nph)%cellcod(:)
!qt      write(kpr,'(a)') 'Space group: '//trim(cryph(nph)%spgname)
!qt      write(kpr,'(a,f0.3,a,i0)')   'Scale: ',cryph(nph)%scal,' Code: ',cryph(nph)%scalcod
!qt!!mr      call print_elements(cryph(nph)%elem,kpr)
!qt!!mr      call print_atoms(cryph(nph)%at,kpr=kpr)
!qt   enddo
!qt!
!qt   end subroutine print_crystal_t

!----------------------------------------------------------------------------------------------------

   integer function numcryst(phase)
   type(crystal_t), dimension(:), allocatable, intent(in) :: phase
   if (allocated(phase)) then
       numcryst = size(phase)
   else
       numcryst = 0
   endif
   end function numcryst

!----------------------------------------------------------------------------------------------------

   subroutine new_crystal_t(vetr,n)
   type(crystal_t), allocatable, intent(inout) :: vetr(:)
   integer, intent(in)                         :: n

   if (n < 0) return
   if (numcryst(vetr) /= n) then
       if (allocated(vetr))deallocate(vetr)
       if (n > 0) allocate(vetr(n))
   endif

   end subroutine new_crystal_t

!----------------------------------------------------------------------------------------------------

   integer function numdats(dat)
   type(dataset_t), dimension(:), allocatable, intent(in) :: dat
   if (allocated(dat)) then
       numdats = size(dat)
   else
       numdats = 0
   endif
   end function numdats

!----------------------------------------------------------------------------------------------------

   subroutine new_dataset_t(vetr,n)
   type(dataset_t), allocatable, intent(inout) :: vetr(:)
   integer, intent(in)                         :: n

   if (n < 0) return
   if (numdats(vetr) /= n) then
       if (allocated(vetr))deallocate(vetr)
       if (n > 0) allocate(vetr(n))
   endif

   end subroutine new_dataset_t

!----------------------------------------------------------------------------------------------------
!#if 0
   !subroutine finishriet_glob(rfc,bc,czero,zc,sdisp,sdcode,stran,stcode,backc)
   subroutine finishriet_glob(c_rcond, c_data) bind(C, name="finishriet_glob")
!
!  Run refinement and pass global variables from fortran to C
!
   USE RefinecomRef, only: rfcond
   USE General, only:is_extraction
   USE background
   USE variables, only: dataset  !,                 cryst
         !use rrestr
   USE crystal_phase
!
   type (c_refine_condition_type), intent(inout) :: c_rcond   
   type(dataset_t), intent(inout)                :: c_data
   integer :: exit_code
!corr   integer                                 :: bc, zc, sdcode, stcode
!corr   real                                    :: czero,sdisp,stran
!corr   type(back_condition_type)               :: backc
!
   
!corr   call cast_refcond_to_f(c_rcond,rfcond)  
!corr   call cast_dataset_to_f(c_data,dataset(1))
   call riet_new(1,exit_code)
   is_extraction = is_extraction .or. rfcond%raction > 0
!
!  corrections
   c_data = cast_dataset_to_c(dataset(1))
   c_rcond = cast_refcond_to_c(rfcond)
!
!corr   czero = dataset(1)%zero
!corr   zc = dataset(1)%zcode
!corr   sdisp = dataset(1)%sdisp
!corr   sdcode = dataset(1)%sdcode
!corr   stran = dataset(1)%stran
!corr   stcode = dataset(1)%stcode
!
!  background
!corr   bc = dataset(1)%bcode(1)
!FIXME-not necessary, back is not changed during rietveld   dataset(1)%cond = backc
!
   end subroutine finishriet_glob

!----------------------------------------------------------------------------------------------------

   subroutine rietveld() bind(C,name="rietveld")
   USE RefinecomRef, only: rfcond
   USE variables, only:cryst,dataset
   USE Molcom, only: jscreen
   USE General, only: ifAutomatic
   !USE genlsq
   USE ilmviewmod
   !USE atom_type_util
   USE sinterface, only: set_cell_and_spg
   USE enable_amb
   USE rietv
!qt   USE rietv, only: rietveldexe, init_rietveld
   !USE unit_cell
   !USE rinterface
!
   implicit none
   logical                            :: modif
   integer                            :: numph, i
   integer, dimension(:), allocatable :: natoms,nelem
   interface
      subroutine open_rietdlg(rcond) bind(C,name="open_rietdlg")
      use riet_typedef
      type(c_refine_condition_type), intent(in) :: rcond
      end subroutine open_rietdlg
   end interface
!
   if (jscreen == 0 .or. ifAutomatic == 1) then   
!      Rietveld run without GUI or in automatic mode
       call rietveldexe(refine='all')
       return
   endif
!
!  FIX THIS!!!
   if (.not.cryst(1)%has_symmetry()) then
       call set_cell_and_spg(cryst(1),modif)
       if (modif .and. jav_opened()) then
           call ilmview_draw(cryst(1)%at,cryst(1)%bond,cryst(1)%elem,cryst(1)%cell,cryst(1)%spg)
       endif
   endif
!
   if (cryst(1)%natoms() == 0) then
       call open_file_selector('Import starting model for Rietveld Refinement'//char(0))
       return
   endif
!
   call init_rietveld(1,.true.,.true.,rfcond)
!
   if (.not. jav_opened()) call ilmview_open(cryst(active_phase)%cr_name)
   call ilmview_draw(cryst(active_phase)%at,cryst(active_phase)%bond,cryst(active_phase)%elem,      &
                     cryst(active_phase)%cell,cryst(active_phase)%spg)
   call jav_set_modify(3)   ! abilita solo i menu' di info
!
!  Crystal phase section
   call get_info_crystal(numph,natoms,nelem)
!qt   call make_crystal_t()
!qt   call put_info_crystal(numph,natoms,nelem,cryph)
!qt   do i=1, numph
!qt      call put_info_ph(cryst(i)%elem, cryst(i)%at,i)
!qt      call put_info_po(cryst(i)%kopcorr,cryst(i)%hklpo,cryst(i)%gcode, cryst(i)%gpar,i)
!qt   enddo
!
!  Dataset section
!qt   call make_dataset_t()
!qt   call put_info_dataset(datas)
   do i=1, numph
      call put_info_pr(dataset(1)%pear(i),         &
             dataset(1)%pvoi(i),dataset(1)%tch(i),dataset(1)%typefun(i),i,1)
   enddo
!
!  Apri finestra grafica
   call abilita_tasti('rietveld')
   !call open_rietdlg(rfcond, dataset(1)%cond)
   call open_rietdlg(cast_refcond_to_c(rfcond))
!
   end subroutine rietveld
!#endif
!----------------------------------------------------------------------------------------------------

   subroutine c_applyriet(modzero,modcell,modwei,modfun,modpo) bind(C,name="c_applyriet")
   use rietv
   use variables, only: cryst        !,dataset
   use crystal_phase, only: numphase
   integer(c_int), intent(in), value        :: modzero, modwei
   integer(c_int), dimension(*), intent(in) :: modcell, modfun, modpo
   integer                                  :: numph 
   integer, dimension(:), allocatable       :: modxyz
!
   numph = numphase(cryst)
   !allocate(modcell(numph), source=0)
   !allocate(modpo(numph), source=0)
   allocate(modxyz(numph), source=0)
   !allocate(modfun(numph), source=0)
   !modwei = 0
     !write(0,*)'MODPO:',modpo(1)   !,dataset(1)%zero
   call applyriet(numph,modzero,modcell,modpo,modxyz,modfun,modwei)
!
   end subroutine c_applyriet

!----------------------------------------------------------------------------------------------------

   subroutine c_exitriet() bind(C,name="c_exitriet")
   use rietv
   call exitriet()
   end subroutine c_exitriet

!----------------------------------------------------------------------------------------------------

   function build_restraints_list(idph, kaction) bind(C,name="build_restraints_list")
   use rrestr
   use variables, only: cryst
   use errormod
   use atom_type_util
   use OBmodule
   integer(c_int), intent(in), value           :: idph
   integer(c_int), intent(in), value           :: kaction  ! 1=build list 2=only update target
   integer(c_int)                              :: build_restraints_list
   type(atom_type),  dimension(:), allocatable :: atom
   type(error_type)                            :: err
!
   if (kaction == 1) then
       if (nrestraints(cryst(idph)%res_list) == 0) then
           call copy_atoms(atom,cryst(1)%at)
           if (cryst(idph)%res_target_type == TARG_MOL_MEC) then
               call OBOptimizeGeometry(atom, cryst(idph)%bond, cryst(idph)%cell, err)
               !if (err%signal) then
               !    call err%print()
               !endif
           endif
           call generate_restraints(atom, cryst(idph)%bond, cryst(idph)%cell,        &
                                    RESDIST, cryst(idph)%res_target_type, cryst(idph)%res_list)
           call generate_restraints(atom, cryst(idph)%bond, cryst(idph)%cell,        &
                                    RESANGLE, cryst(idph)%res_target_type, cryst(idph)%res_list)
           call generate_restraints(atom, cryst(idph)%bond, cryst(idph)%cell,        &
                                    RESPLANE, cryst(idph)%res_target_type, cryst(idph)%res_list)
       endif
   else if (kaction == 2) then
            call copy_atoms(atom,cryst(1)%at)
            if (cryst(idph)%res_target_type == TARG_MOL_MEC) then
                call OBOptimizeGeometry(atom, cryst(idph)%bond, cryst(idph)%cell, err)
                !if (err%signal) then
                !    call err%print()
                !endif
            endif
            call update_restraints_target(atom, cryst(idph)%bond, cryst(idph)%cell, &
                                     RESDIST, cryst(idph)%res_target_type, cryst(idph)%res_list)
            call update_restraints_target(atom, cryst(idph)%bond, cryst(idph)%cell, &
                                     RESANGLE, cryst(idph)%res_target_type, cryst(idph)%res_list)
            call update_restraints_target(atom, cryst(idph)%bond, cryst(idph)%cell, &
                                     RESPLANE, cryst(idph)%res_target_type, cryst(idph)%res_list)
   endif
   !call print_restraints_all(0, cryst(idph)%res_list, cryst(idph)%at, cryst(idph)%cell, .false.)  

   build_restraints_list = nrestraints(cryst(idph)%res_list)
!
   end function build_restraints_list

!----------------------------------------------------------------------------------------------------

   function nat_restraint(idph, idres) bind(C,name="nat_restraint")
   use variables, only: cryst
   integer(c_int), intent(in), value :: idres 
   integer(c_int), intent(in), value :: idph
   integer(c_int)                    :: nat_restraint
!
   nat_restraint = size(cryst(idph)%res_list(idres)%na)
!
   end function nat_restraint

!----------------------------------------------------------------------------------------------------

   subroutine c_restraint_info(idph,idres,code,vat,vtype,val,targ,sigma,active) bind(C,name="c_restraint_info")
   use variables, only: cryst
   integer(c_int), intent(in), value  :: idph
   integer(c_int), intent(in), value  :: idres
   integer(c_int)                     :: code
   integer(c_int), dimension(*)       :: vat,vtype
   integer                            :: natr
   real(c_float)                      :: val,targ,sigma
   integer(c_int)                     :: active
!
   code = cryst(idph)%res_list(idres)%code
   natr = size(cryst(idph)%res_list(idres)%na)
   vat(:natr) = cryst(idph)%res_list(idres)%na(:) - 1
   vtype(:natr) = cryst(idph)%at(cryst(idph)%res_list(idres)%na(:))%kscatt() - 1
   val = cryst(idph)%res_list(idres)%val
   targ = cryst(idph)%res_list(idres)%targ
   sigma = cryst(idph)%res_list(idres)%sigma
   active = cryst(idph)%res_list(idres)%active
!
   end subroutine c_restraint_info

!----------------------------------------------------------------------------------------------------

   subroutine c_restraint_settings_info(idph,wei,target_type) bind(C,name="c_restraint_settings_info")
   use variables, only: cryst
   integer(c_int), intent(in), value :: idph
   real(c_float), dimension(*)       :: wei
   integer(c_int)                    :: target_type
   integer                           :: i
!
   do i=1,3
      wei(i) = cryst(idph)%wres(i)
   enddo
   target_type = cryst(idph)%res_target_type
!
   end subroutine c_restraint_settings_info

!----------------------------------------------------------------------------------------------------

   subroutine c_put_restraint_info(idph, code, vat, nat, val, targ, sigma, active) &
                                   bind(C,name="c_put_restraint_info")
   use variables, only: cryst
   use rrestr
   integer(c_int), intent(in), value  :: idph
   integer(c_int), intent(in), value  :: code
   integer(c_int), dimension(*)       :: vat
   integer(c_int), intent(in), value  :: nat
   real(c_float), intent(in), value   :: val,targ,sigma
   integer(c_int), intent(in), value  :: active
   integer                            :: pos
   type(restraint_type)               :: restr
!
!  Update list of all restraints
   pos = restraint_position(cryst(idph)%res_list,vat(:nat))
   if (pos == 0) then
       return ! pos = 0 this should not possible
   else
       cryst(idph)%res_list(pos)%val   = val
       cryst(idph)%res_list(pos)%targ  = targ
       cryst(idph)%res_list(pos)%sigma = sigma
       cryst(idph)%res_list(pos)%active = active
   endif
!
!  Update list of restraints used for refinement
   pos = restraint_position(cryst(idph)%res,vat(:nat))
   if (active == 0) then
       ! delete non activated restraint from the list
       if (pos > 0) call delete_restraints(cryst(idph)%res,pos)
   else
           write(*,*)'Restraint: ',pos,val,targ,sigma,code
       if (pos == 0) then
!
!          add not present restraint
           restr%val   = val
           restr%targ  = targ
           restr%sigma = sigma
           allocate(restr%na(nat), source=vat(1:nat))
           restr%code = code
           call add_restraint_to_list(cryst(idph)%res,restr)
       else
!
!          update existing restraint
           if (code <= RESANGLE) then
               cryst(idph)%res(pos)%sigma = sigma
               cryst(idph)%res(pos)%targ = targ
           else
               cryst(idph)%res(pos)%sigma = sigma
           endif
       endif
   endif
!
   end subroutine c_put_restraint_info

!----------------------------------------------------------------------------------------------------

   subroutine c_put_restraint_settings(idph, wei) bind(C,name="c_put_restraint_settings")
   use rrestr
   use variables, only: cryst
   integer(c_int), value, intent(in) :: idph
   real(c_float), dimension(*)       :: wei
   integer                           :: i
!
   do i=1,3
      cryst(idph)%wres(i) = wei(i)
      call set_weight(cryst(idph)%res,i,cryst(idph)%wres(i))
   enddo
!
   end subroutine c_put_restraint_settings

!----------------------------------------------------------------------------------------------------

   subroutine c_put_restraint_target_type(idph, target_type) bind(C,name="c_put_restraint_target_type")
   use variables, only: cryst
   integer(c_int), value, intent(in) :: idph
   integer(c_int), value, intent(in) :: target_type
!
   cryst(idph)%res_target_type = target_type
!
   end subroutine c_put_restraint_target_type

!----------------------------------------------------------------------------------------------------

   subroutine read_restraints_file(idph, fileName, length, ier) bind(C,name="read_restraints_file")
   use rrestr
   use strutil
   use variables, only: cryst
   use errormod
   integer(c_int), value, intent(in) :: idph
   character(c_char), intent(in)     :: fileName(*)
   integer(c_int), intent(in), value :: length
   integer(c_int)                    :: ier
   character(len=:), allocatable     :: file_name
   type(error_type)             :: err
!   
   ier = 0
   file_name = toFortranString(fileName,length)
   call read_restraints_from_file(cryst(idph)%at,cryst(idph)%cell,file_name,cryst(idph)%res_list,cryst(idph)%wres,err)
   if (err%signal) then
       ier = 1
       call err%print()
   endif
!
   end subroutine read_restraints_file

!----------------------------------------------------------------------------------------------------

   function scatters_number(idph) bind(C,name="scatters_number")
   use variables, only: cryst
   integer(c_int), intent(in), value :: idph
   integer(c_int)                    :: scatters_number
!
   scatters_number = cryst(idph)%numelem()
!
   end function scatters_number

!----------------------------------------------------------------------------------------------------

   subroutine get_scatter_info(idph,kscatt,lab) bind(C,name="get_scatter_info")
   use strutil
   use variables, only: cryst
   integer(c_int), intent(in), value :: idph,kscatt
   character(c_char)                 :: lab(*)
   integer :: lens
   lens = len_trim(cryst(idph)%elem(kscatt)%lab)+1
   lab(:lens) = toCString(cryst(idph)%elem(kscatt)%lab)
   end subroutine get_scatter_info

!----------------------------------------------------------------------------------------------------

   subroutine get_constraint_info(idph,riding,riding_adp_factor,    &
                                  constr_adp_type) bind(C,name="get_constraint_info")
   use arrayutil
   use variables, only: cryst
   integer(c_int), intent(in), value :: idph
   integer(c_int)                    :: riding
   real(c_float)                     :: riding_adp_factor
   integer(c_int)                    :: constr_adp_type
!
   riding = logical_to_int(cryst(idph)%riding_model)
   riding_adp_factor =  cryst(idph)%adp_factor
   constr_adp_type = cryst(idph)%constr_adp_type
!
   end subroutine get_constraint_info

!----------------------------------------------------------------------------------------------------

   subroutine put_constraint_info(idph,riding,adp_factor,      &
                                  constr_adp_type) bind(C,name="put_constraint_info")
   use arrayutil
   use variables, only: cryst
   integer(c_int), intent(in), value :: idph
   integer(c_int), intent(in), value :: riding
   real(c_float), intent(in), value  :: adp_factor
   integer(c_int), intent(in), value :: constr_adp_type
!
   cryst(idph)%riding_model = int_to_logical(riding)
   cryst(idph)%adp_factor = adp_factor
   cryst(idph)%constr_adp_type = constr_adp_type
!
   end subroutine put_constraint_info 

!----------------------------------------------------------------------------------------------------

   integer(c_int) function constraints_number(idph) bind(C,name="constraints_number")
   use variables, only: cryst
   integer(c_int), intent(in), value :: idph
!
   constraints_number = cryst(idph)%numconstr()
!
   end function constraints_number

!----------------------------------------------------------------------------------------------------

   function nat_constraint(idph, idc) bind(C,name="nat_constraint")
   use variables, only: cryst
   integer(c_int), intent(in), value :: idc 
   integer(c_int), intent(in), value :: idph
   integer(c_int)                    :: nat_constraint
!
   nat_constraint = size(cryst(idph)%constr(idc)%vat)
!
   end function nat_constraint

!----------------------------------------------------------------------------------------------------

   subroutine c_constraint_info(idph,idc,code,vat,vpar) bind(C,name="c_constraint_info")
   use variables, only: cryst
   integer(c_int), intent(in), value  :: idph
   integer(c_int), intent(in), value  :: idc
   integer(c_int)                     :: code
   integer(c_int), dimension(*)       :: vat,vpar
   integer                            :: natc
!
   code = cryst(idph)%constr(idc)%code
   natc = size(cryst(idph)%constr(idc)%vat)
   vat(:natc) = cryst(idph)%constr(idc)%vat(:) - 1
   vpar(:natc) = cryst(idph)%constr(idc)%vpar(:) - 1
!
   end subroutine c_constraint_info

!-----------------------------------------------------------------------------

   subroutine show_model(k) bind(C,name="show_model")
   USE variables, only: cryst
   USE ilmviewmod
   integer(c_int), intent(in), value :: k
!
   active_phase = k
   call ilmview_draw(cryst(k)%at,cryst(k)%bond,cryst(k)%elem,cryst(k)%cell,cryst(k)%spg)
!
   end subroutine show_model

end module rinterface

!corr   subroutine finishriet_glob(rfc,bc,czero,zc,sdisp,sdcode,stran,stcode,backc)
!corr!
!corr!  Run refinement and pass global variables from fortran to C
!corr!
!corr   USE RefinecomRef
!corr   USE General, only:is_extraction
!corr   USE background
!corr   USE variables, only: dataset         !,cryst
!corr   USE crystal_phase
!corr         !use constraints
!corr   implicit none
!corr   type (refine_condition_type)            :: rfc   
!corr   integer                                 :: bc, zc, sdcode, stcode
!corr   real                                    :: czero,sdisp,stran
!corr   type(back_condition_type)               :: backc
!corr!
!corr!!!!!prova - pawley
!corr        !if(rfcond%raction == 1) call pawley(1)
!corr!call print_constraints(cryst(1)%constr,cryst(1)%at,0,.true.)
!corr!write(0,*)'REF=',cryst(1)%constr(1)%ref
!corr   call riet_new(1)
!corr        !if(rfcond%raction == 2) call pawley(0)
!corr   is_extraction = is_extraction .or. rfcond%raction > 0
!corr!
!corr!  corrections
!corr   czero = dataset(1)%zero
!corr   zc = dataset(1)%zcode
!corr   sdisp = dataset(1)%sdisp
!corr   sdcode = dataset(1)%sdcode
!corr   stran = dataset(1)%stran
!corr   stcode = dataset(1)%stcode
!corr!
!corr!  background
!corr   bc = dataset(1)%bcode(1)
!corr   dataset(1)%cond = backc
!corr!
!corr   end subroutine finishriet_glob

!-----------------------------------------------------------------------------

   subroutine finishriet_ph(catm,cell,cellcod,scal,scalcod,cpear,cvoi,ctch, &
                    kopcorr,hklpo,gpar,gcode,ph)
!
!  Pass values from fortran to C after running refinement
!
   USE RefinecomRef
   USE variables, only:cryst, dataset
   USE background
   USE atom_type_util
   USE profile_function
   implicit none
   type(atom_type)          :: catm(*)
   real, dimension(6)       :: cell
   integer, dimension(6)    :: cellcod
   real                     :: scal 
   integer                  :: scalcod
   type(profile_function_t) :: cpear, cvoi, ctch
   integer                  :: kopcorr
   integer, dimension(3)    :: hklpo
   real                     :: gpar
   integer                  :: gcode
   integer, intent(in)      :: ph
   integer                  :: i
!
   do i=1, cryst(ph)%natoms()
      catm(i) = cryst(ph)%at(i)
   end do
   scal = cryst(ph)%scal
   scalcod = cryst(ph)%scode
   cell(:) = cryst(ph)%get_cellpar()
   cellcod(:) = cryst(ph)%get_cellcod()
   cpear = dataset(1)%pear(ph)
   cvoi  = dataset(1)%pvoi(ph)
   ctch  = dataset(1)%tch(ph)
!corr   kopcorr = cryst(ph)%kopcorr
   hklpo = cryst(ph)%hklpo
   gpar = cryst(ph)%gpar
   gcode = cryst(ph)%gcode
!
   end subroutine finishriet_ph

!-----------------------------------------------------------------------------

   subroutine ripassa_valori_glob(rfc,bc,czero,zc,sdisp,sdcode,tran,stcode,backc)
   USE RefinecomRef
   USE background
   USE variables, only: dataset
   implicit none
   type (refine_condition_type) :: rfc   
   integer                      :: zc,bc,sdcode,stcode
   real                         :: czero,sdisp,tran
   type(back_condition_type)    :: backc
!
   rfcond = rfc
!
!  corrections
   dataset(1)%zero = czero   
   dataset(1)%zcode = zc    
   dataset(1)%sdisp = sdisp
   dataset(1)%sdcode = sdcode
   dataset(1)%stran = tran
   dataset(1)%stcode = stcode
!
!  background
   dataset(1)%cond = backc
   dataset(1)%bcode = bc
!
   end subroutine ripassa_valori_glob

!-----------------------------------------------------------------------------
!corr
!corr   subroutine ripassa_valori_ph(catm,cell,cellcod,scal,scalcod,cpear,cvoi,  &
!corr                 ctch,kopcorr,hklpo,gpar,gcode,typefun,ph)
!corr   USE atom_basic, only: atom_type
!corr   USE variables, only:cryst, dataset
!corr   USE rinterface
!corr   USE profile_function
!corr   USE constraints
!corr   implicit none
!corr   type(atom_type)          :: catm(*)
!corr   real, dimension(6)       :: cell(6)
!corr   integer, dimension(6)    :: cellcod
!corr   real                     :: scal
!corr   integer                  :: scalcod
!corr   integer                  :: typefun
!corr   integer, intent(in)      :: ph
!corr   integer                  :: i
!corr   type(profile_function_t) :: cpear , cvoi, ctch
!corr   integer                  :: kopcorr
!corr   integer, dimension(3)    :: hklpo
!corr   real                     :: gpar
!corr   integer                  :: gcode
!corr!
!corr   do i=1, cryst(ph)%natoms()
!corr      cryst(ph)%at(i) = catm(i)
!corr   end do
!corr   call cryst(ph)%set_cell(cell)
!corr   do i=1,6
!corr      call cryst(ph)%set_cellcod(i,cellcod(i))
!corr   end do
!corr   cryst(ph)%scal = scal
!corr   cryst(ph)%scode = scalcod
!corr   dataset(1)%typefun(ph) = typefun
!corr   select  case (dataset(1)%typefun(ph))
!corr   case (PEARSON7)
!corr         dataset(1)%pear(ph) = cpear
!corr   case (PVOIG)
!corr         dataset(1)%pvoi(ph) = cvoi
!corr   case (TCHZ)
!corr         dataset(1)%tch(ph)  = ctch
!corr   end select
!corr   cryst(ph)%kopcorr= kopcorr
!corr   cryst(ph)%hklpo  = hklpo  
!corr   cryst(ph)%gpar   = gpar   
!corr   cryst(ph)%gcode  = gcode  
!corr!
!corr!  Check constraints for merge dipendent equations
!corr   call check_constraints(cryst(ph)%constr)
!corr!
!corr   end subroutine ripassa_valori_ph
!corr

!---------------------------------------------------------------------------------------------   
!qt
!qt   subroutine get_constraints(ph) 
!qt! 
!qt!  passa al c i constraints
!qt! 
!qt   USE variables, only: cryst
!qt   USE constraints
!qt   implicit none
!qt   integer :: i,ph
!qt   integer :: nconstr
!qt! 
!qt   nconstr = nconstraints(cryst(ph)%constr)
!qt   call passa_num_constraints(nconstr,ph)
!qt   do i=1, nconstr
!qt      call passa_constr(cryst(ph)%constr(i)%code, cryst(ph)%constr(i)%npar, cryst(ph)%constr(i)%ref,   &
!qt                   cryst(ph)%constr(i)%vpar,cryst(ph)%constr(i)%vat,cryst(ph)%constr(i)%coef,          &
!qt                   cryst(ph)%constr(i)%sumc,i,cryst(ph)%constr(i)%typec,cryst(ph)%constr(i)%npari,ph)            
!qt   end do
!qt! 
!qt   end subroutine get_constraints
!qt            
!--------------------------------------------------------------------------------

   subroutine put_num_constraints(knconstr,ph,riding_model,adp_factor) 
! 
!  get from C the number of constraints and riding variables for the phase ph
!  prende dal c il numero di constraints e 
! 
   USE constraints
   USE variables, only: cryst
   implicit none
   integer :: knconstr,ph
   logical :: riding_model
   real    :: adp_factor

! 
!  reallocate constraints
   call resize_constraints(cryst(ph)%constr,knconstr)
!
!  update riding variables
   cryst(ph)%riding_model = riding_model  
   cryst(ph)%adp_factor = adp_factor  
! 
   end subroutine put_num_constraints
            
!--------------------------------------------------------------------------------

   subroutine put_constr(code, npar, ref, vpar, vat, coef, sumc, j, typec, npari,ph)
! 
!  prende dal C il j-esimo constraint
! 
   USE constraints
   USE variables, only: cryst
   implicit none
   integer, intent(in)                  :: j
   integer, intent(in)                  :: ref
   integer, intent(in)                  :: code 
   integer, intent(in)                  :: npar
   integer, dimension(npar), intent(in) :: vpar
   integer, dimension(npar), intent(in) :: vat
   real, dimension(npar), intent(in)    :: coef
   real, intent(in)                     :: sumc
   integer, intent(in)                  :: typec,npari
   integer                              :: ph
! 
   cryst(ph)%constr(j+1) = constraint_type(code,vpar,vat,coef,sumc,npar,npari,ref=0,typec=typec)
   if (ref == 1) then
       call refine_constraint(cryst(ph)%constr(j+1),cryst(ph)%at,1)
   endif
! 
   end subroutine put_constr

!---------------------------------------------------------------------------------------------   

   subroutine get_info(rinfo_var)
! 
!  Prende dal C il j-esimo constraint
! 
   USE RefinecomRef
   USE progtype, only: refine_info_type
! 
   implicit none
   type(refine_info_type), intent(out) :: rinfo_var
! 
   rinfo_var = rinfo
! 
   end subroutine get_info
