#include "shaderprogram.h"

#include <QOpenGLShaderProgram>
#include <QDebug>

ShaderProgram::ShaderProgram():
    m_program(nullptr)
{

}

ShaderProgram::ShaderProgram(const QString &vertexShaderFilePath, const QString &fragmentShaderFilePath)
{
    m_program = new QOpenGLShaderProgram();

    // read the shader programs from the resource
    if (!m_program->addShaderFromSourceFile(QOpenGLShader::Vertex, vertexShaderFilePath))
        qDebug() << "Vertex shader errors:\n" << m_program->log();

    if (!m_program->addShaderFromSourceFile(QOpenGLShader::Fragment, fragmentShaderFilePath))
        qDebug() << "Fragment shader errors:\n" << m_program->log();

    if (!m_program->link())
        qDebug() << "Shader linker errors:\n" << m_program->log();

}

void ShaderProgram::use()
{
    m_program->bind();
}

void ShaderProgram::destroy()
{
    delete m_program;
    m_program = nullptr;
}

void ShaderProgram::setBool(const QString &name, bool value) const
{
    m_program->setUniformValue(name.toStdString().c_str(), value);
}

void ShaderProgram::setInt(const QString &name, int value) const
{
    m_program->setUniformValue(name.toStdString().c_str(), value);
}

void ShaderProgram::setFloat(const QString &name, float value) const
{
    m_program->setUniformValue(name.toStdString().c_str(), value);
}

void ShaderProgram::setMat4(const QString &name, const QMatrix4x4 &mat4) const
{
    m_program->setUniformValue(name.toStdString().c_str(), mat4);
}

QOpenGLShaderProgram *ShaderProgram::program() const
{
    return m_program;
}
