#include "solutionsdialog.h"
#include "ui_solutionsdialog.h"
#include "nofocusdelegate.h"
#include "mainwindow.h"

extern "C" void get_solution_size(int *nsol, int *size_name);
extern "C" void get_solution_info(SolutionInfo info[], char *strcol);
extern "C" void showSolution(int solution);
extern "C" void saveSolution(int solution);

SolutionsDialog::SolutionsDialog(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::SolutionsDialog),
    lastSelection(-1)
{
    ui->setupUi(this);

    createSolutionModel();
    setSolutionTableView();
}

SolutionsDialog::~SolutionsDialog()
{
    delete ui;
}

void SolutionsDialog::setDialog(int &nsol, int nexport)
{
    int lenName;
    get_solution_size(&nsol, &lenName);
    solutionModel->setRowCount(nsol);
    if (nsol > 0) {
        auto info = new SolutionInfo[nsol];
        auto fileName = new char[lenName];
        get_solution_info(info, fileName);

        for (int r = 0; r < nsol; r++) {
            auto item = new QStandardItem();
            item->setData(info[r].run, Qt::DisplayRole);
            solutionModel->setItem(r,0,item);

            item = new QStandardItem();
            item->setData(info[r].cf, Qt::DisplayRole);
            solutionModel->setItem(r,1,item);

            if (r < nexport) {
                item = new QStandardItem(QString(fileName)+"_best"+QString::number(r+1)+".cif");
                solutionModel->setItem(r,2,item);
            }
        }
        delete [] info;
        delete [] fileName;
        ui->solutionTableView->resizeColumnsToContents();
        ui->solutionTableView->horizontalHeader()->setSectionResizeMode(2,QHeaderView::Stretch);
        if (lastSelection >= 0) ui->solutionTableView->selectRow(lastSelection);
        savedSelection = lastSelection;
    }
}

void SolutionsDialog::resetLastSelection()
{
    lastSelection = -1;
}

void SolutionsDialog::createSolutionModel()
{
    solutionModel = new QStandardItemModel(this);
    solutionProxyModel = new QSortFilterProxyModel(this);
    solutionProxyModel->setSourceModel(solutionModel);
    solutionModel->setColumnCount(3);
    const QStringList titles = {"Run Nr.", "Cost Function", "File"};
    solutionModel->setHorizontalHeaderLabels(titles);
}

void SolutionsDialog::setSolutionTableView()
{
    ui->solutionTableView->setModel(solutionProxyModel);
    ui->solutionTableView->verticalHeader()->setDefaultSectionSize(22);
    ui->solutionTableView->setSortingEnabled(true);
    ui->solutionTableView->sortByColumn(1,Qt::AscendingOrder);
    ui->solutionTableView->verticalHeader()->hide();
    ui->solutionTableView->setShowGrid(false);
    ui->solutionTableView->setAlternatingRowColors(true);
    ui->solutionTableView->setSelectionMode(QAbstractItemView::SingleSelection);
    ui->solutionTableView->setSelectionBehavior(QAbstractItemView::SelectRows);
    ui->solutionTableView->horizontalHeader()->stretchLastSection();
    ui->solutionTableView->setEditTriggers(QAbstractItemView::NoEditTriggers);
    ui->solutionTableView->setItemDelegate(new NoFocusDelegate()); //to remove focus rectangle

    connect(ui->solutionTableView->selectionModel(),&QItemSelectionModel::currentRowChanged,this,[this](const QModelIndex &current){
        lastSelection = solutionProxyModel->mapToSource(current).row();
        showSolution(lastSelection+1);
#if !defined(JAV_WIDGET)
        //Raise JAV window
        MainWindow *mw = getMainWindow();
        if (!mw) return;
        JavWindow *jav = mw->getJav();
        if (jav->isVisible()) {
            if (!this->window()->geometry().intersects(jav->geometry()))
                jav->raise();
        }
#endif
    });
}

void SolutionsDialog::on_buttonBox_clicked(QAbstractButton *button)
{
    QDialogButtonBox::StandardButton stdButton = ui->buttonBox->standardButton(button);

    if(stdButton == QDialogButtonBox::Save) {
        if (lastSelection >= 0) saveSolution(lastSelection+1);
    } else if(stdButton == QDialogButtonBox::Cancel) {
        if (savedSelection >= 0) saveSolution(savedSelection+1);
    }
}
