#include <QDebug>
#include "table.h"
#include "ui_table.h"
#include "libcomune.h"
//#include "libmath.h"
//#include "glutils.h"
#include "qt_utils.h"

#define WI 32
#define HE 24

static const int posi[N_TRUE_ELEMENTS][2] = {
{0, 0}, {0,17},
{1, 0}, {1, 1}, {1, 12}, {1, 13}, {1, 14}, {1, 15}, {1, 16}, {1, 17},
{2, 0}, {2, 1}, {2, 12}, {2, 13}, {2, 14}, {2, 15}, {2, 16}, {2, 17},
{3, 0}, {3, 1}, {3, 2}, {3, 3}, {3, 4}, {3, 5}, {3, 6}, {3, 7}, {3, 8},
{3, 9}, {3,10}, {3,11}, {3,12}, {3,13}, {3,14}, {3,15}, {3,16}, {3,17},
{4, 0}, {4, 1}, {4, 2}, {4, 3}, {4, 4}, {4, 5}, {4, 6}, {4, 7}, {4, 8},
{4, 9}, {4,10}, {4,11}, {4,12}, {4,13}, {4,14}, {4,15}, {4,16}, {4,17},
{5, 0}, {5, 1},
{7, 3}, {7, 4}, {7, 5}, {7, 6}, {7, 7}, {7, 8},
{7, 9}, {7,10}, {7,11}, {7,12}, {7,13}, {7,14}, {7,15}, {7,16}, {7,17},
{5, 3}, {5, 4}, {5, 5}, {5, 6}, {5, 7}, {5, 8},
{5, 9}, {5,10}, {5,11}, {5,12}, {5,13}, {5,14}, {5,15}, {5,16}, {5,17},
{6, 0}, {6, 1},
{8, 3}, {8, 4}, {8, 5}, {8, 6}, {8, 7}, {8, 8},
{8, 9}, {8,10}, {8,11}, {8,12}, {8,13}, {8,14}, {8,15}, {8,16}, {8,17},
{6, 3}, {6, 4}, {6, 5}, {6, 6}, {6, 7}, {6, 8}, {6, 9}, {6,10}};

TableWindow::TableWindow(QWidget *parent)
    : QDialog(parent)
    , ui(new Ui::TableWindow)
{
    ui->setupUi(this);
    QGridLayout *grid = ui->gridLayout;
    for(int k=0; k<N_TRUE_ELEMENTS; k++)
    {
        QString str = get_nth_element(k);
        QPushButton *button = new TableButton(k, true, this);
        grid->addWidget(button, posi[k][0], posi[k][1],Qt::AlignCenter);
        connect(button, &QPushButton::clicked, [=] { sp_selected(str, k); });
    }
}

TableWindow::~TableWindow()
{
    delete ui;
}

QString TableWindow::GetElement(bool *ok)
{
    m_ok = false;
    this->exec();
    *ok = m_ok;
    return elem;
}

int TableWindow::GetZ(bool *ok)
{
    m_ok = false;
    this->exec();
    *ok = m_ok;
    return m_z;
}

void TableWindow::sp_selected(QString(str), int z)
{
    elem = str;
    m_z = z;
    m_ok = true;
    this->hide();
}

TableButton::TableButton(QWidget *parent)
    : QPushButton(parent)
{

}

TableButton::TableButton(int k, bool ifText, QWidget *parent)
    : QPushButton(parent)
{
    setFixedSize(WI, HE);
    if(ifText)
    {
        QString str = get_nth_element(k);
        setText(str);
    }
    QColor color_but = get_elem_color(k);
    SetColor(color_but);
    setFocusPolicy(Qt::NoFocus);
//    if(!ifText)
//       connect(this, &QPushButton::clicked, [=] { colorbutton_clicked(k); });
}

void TableButton::SetColor(QColor color_but)
{
    QColor color1 = color_but.lighter(); //Lighter(color_but);
    QColor color2 = color_but.darker(120); //Darker(color_but);
    QString style_sheet = "QPushButton {background-color:";
    style_sheet.append("qlineargradient(x1:0, y1:0, x2:0, y2:1, stop:0.3 ");
    style_sheet.append(color1.name());
    style_sheet.append(", stop: 0.8 ");
    style_sheet.append(color_but.name());
    style_sheet.append(", stop:1 ");
    style_sheet.append( color2.name());
    style_sheet.append(
        "); border-style: outset;"
        "border-width: 2px;"
        "border-radius: 3px;"
        "border-color: gray;"
        "padding: 2px;}");
    setStyleSheet(style_sheet);
}

void TableButton::colorbutton_clicked(int k)
{
    QColor color = get_elem_color(k);
    color = QColorDialog::getColor(color, parentWidget());
    if (!color.isValid()) return;
    SetColor(color);
}

