#include "transformdialog.h"
#include "ui_transformdialog.h"
#include <QStringList>
#include <QDoubleValidator>
#include "properties.h"

extern "C" void jav_transform_model_(int *result, int *koper,
                               float T[], int *wm, int *nptra, int *kptra);

TransformDialog::TransformDialog(QWidget *parent) :
    QDialog(parent),
    ui(new Ui::TransformDialog),
    transformModel(nullptr),
    kmodel(0)
{
    QStringList headers ={ "Number", "x' y' z'"};
    ui->setupUi(this);
    transformModel = new QStandardItemModel(this);
    transformModel->setHorizontalHeaderLabels(headers);
    ui->spgView->setModel(transformModel);
    QItemSelectionModel *selectionModel = ui->spgView->selectionModel();
    connect(selectionModel, &QItemSelectionModel::currentRowChanged,
           this, &TransformDialog::rowSelectionChanged);
    QDoubleValidator *doubleValidator = new QDoubleValidator(this);
    doubleValidator->setDecimals(1);
    for(int i=0; i<3; i++)
    {
        QString nomeT = "T" + QString::number(i+1);
        QLineEdit *e = findChild<QLineEdit *>(nomeT);
        e->setValidator(doubleValidator);
    }


}

TransformDialog::~TransformDialog()
{
    delete ui;
}

void TransformDialog::on_cancelButton_clicked()
{
    doTransform(4);
    emit trasformDlgClosed();
    QDialog::reject();
}

void TransformDialog::on_transformButton_clicked()
{
     doTransform(1);
}

void TransformDialog::on_copyButton_clicked()
{
     doTransform(2);
}

void TransformDialog::fillMatrices(int RMatrix[3][3], float TMatrix[3])
{
    for(int i=0; i<3;i++)
    {
        for(int j=0;j<3;j++)
        {
            QString nomeR = "R" + QString::number(i+1) + QString::number(j+1);
            QLineEdit *e = findChild<QLineEdit *>(nomeR);
            e->setText(QString::number(RMatrix[i][j], 'f', 0));
        }
        QString nomeT = "T" + QString::number(i+1);
        QLineEdit *e = findChild<QLineEdit *>(nomeT);
        e->setText(QString::number(TMatrix[i], 'f',1));
    }
}

void TransformDialog::rowSelectionChanged(const QModelIndex &current, const QModelIndex &)
{
    int RMatrix[3][3];
    float TMatrix[3];
    int row = current.row() + 1;
    if(getMatrices(Properties::getStringSymm(&row), RMatrix, TMatrix))
        fillMatrices(RMatrix, TMatrix);
    else
        qDebug() << "Something wrong with symmetry operators";
}

QString TransformDialog::noBlank(QString str)
{
    QString outStr("");
    for(int i=0; i<str.length(); i++)
        if(str.at(i) != ' ')
           outStr.push_back(str.at(i));
    return outStr;
}

bool TransformDialog::getMatrices(QString input, int RMatrix[3][3], float TMatrix[3])
{
    QString str0 = noBlank(input.toLower().trimmed());
    QString xyz  = "xyz";
    int cifra;
    float a1, a2;

    for(int i=0; i<3; i++)
    {
        TMatrix[i] = 0;
        for(int j=0; j<3; j++)
            RMatrix[i][j] = 0;
    }
    QStringList linee;
    int k =  str0.indexOf(',');
    if(k == -1)
       return false;
    linee.append(str0.left(k));
    str0.remove(0, k+1);
    k =  str0.indexOf(',');
    if(k == -1)
       return false;
    linee.append(str0.left(k));
    str0.remove(0, k+1);
    linee.append(str0);

    for(int j=0; j<3; j++)
    {
        int ls = 0;
        QString linea = linee.at(j);
        for(int l=0; l<3; l++)
        {
            ls = linea.indexOf(xyz[l]);
            if (ls != -1)
            {
                if(ls == 0)
                {
                   cifra = 1;
                   linea.remove(ls, 1);
                }
                else
                {
                   if(linea.at(ls-1) == '+')
                   {
                      cifra = 1;
                      linea.remove(ls-1, 2);
                   }
                   else if(linea.at(ls-1) == '-')
                   {
                      cifra = -1;
                      linea.remove(ls-1, 2);
                   }
                   else
                      return false;
                }
                RMatrix[j][l] = cifra;
            }
        }
        if(!linea.isEmpty())
        {
           k = linea.indexOf('/');
           if(k != -1)
           {
              a1 = linea.left(k).toFloat();
              a2 = linea.right(linea.length()-k-1).toFloat();
              a1 = a1 / a2;
           }
           else
              a1 = linea.toFloat();
           TMatrix[j] = a1;
        }
    }
    return true;
}

void TransformDialog::doTransform(int param)
{
    int code = param;
    float T[3];
    for(int i=0; i<3; i++)
    {
        QString nomeT = "T" + QString::number(i+1);
        QLineEdit *e = findChild<QLineEdit *>(nomeT);
        T[i] = e->text().toFloat();
    }
    bool check_onlysel = false;
    if (ui->checkOnlySel->isEnabled())
        check_onlysel = ui->checkOnlySel->isChecked();
    int nptra = 0;
    int *kptra = nullptr;
    if(check_onlysel && !atomsSel.empty())
    {
             nptra = atomsSel.size();
             kptra = new int [nptra]();
             for(int i=0; i<nptra; i++)
                 kptra[i] = atomsSel[i];
    }

    int koper = ui->spgView->selectionModel()->currentIndex().row() + 1;
    if(koper > 0)
       jav_transform_model_(&code, &koper, T, &kmodel, &nptra, kptra);

}

void TransformDialog::showEvent(QShowEvent *event)
{
    doTransform(3);
    QDialog::showEvent(event);
}

void TransformDialog::Fill(std::vector<int> atoms_sel, int whichModel)
{
     ui->checkOnlySel->setEnabled(!atoms_sel.empty());
     if(!atomsSel.empty())
         atomsSel.clear();
     if(!atoms_sel.empty())
        for(size_t i=0; i<atoms_sel.size(); i++)
            atomsSel.push_back(atoms_sel[i]);
     kmodel = whichModel + 1;
     int nrows = transformModel->rowCount();
     transformModel->removeRows(0, nrows);
     for(int i=1; i<=Properties::sy.nsymop; i++)
     {
         QList<QStandardItem *> items;
         QStandardItem *item;
         item = new QStandardItem();               // Number
         item->setData(i, Qt::DisplayRole);
         items.append(item);                       // Visible
         item = new QStandardItem(QString(Properties::getStringSymm(&i)).trimmed());
         items.append(item);
         transformModel->appendRow(items);
     }
     ui->spgView->horizontalHeader()->setStretchLastSection(false);
     ui->spgView->resizeColumnsToContents();
     ui->spgView->horizontalHeader()->setStretchLastSection(true);
     ui->spgView->selectRow(0);
}
